@@ -102,6 +102,10 @@
 	   (rest (subseq v (1+ d2))))
       (list '(mlist simp) (parse-integer year) (parse-integer month) rest)))
 
+  (defvar $maxima_frontend nil)
+  (defvar $maxima_frontend_version nil)
+  (setf (symbol-value '$maxima_frontend) "wxMaxima")
+
   (defun $wxbuild_info ()
     (let ((year (sixth cl-user:*maxima-build-time*))
 	  (month (fifth cl-user:*maxima-build-time*))
@@ -119,8 +123,7 @@
       (format t "Lisp implementation type: ~a~%" (lisp-implementation-type))
       (format t "Lisp implementation version: ~a~%" (lisp-implementation-version))
       (format t "~%~%wxMaxima's idea of the directory layout is:~%User configuration dir: ~a~%" wxUserConfDir)
-      (format t "Help dir: ~a~%" wxHelpDir)
-      (format t "Maxima lisp dir: ~a~%" wxMaximaLispLocation))
+      (format t "Help dir: ~a~%" wxHelpDir))
     "")
 
   (defmfun $wxbug_report ()
@@ -1406,9 +1409,23 @@
   (defvar $wxanimate_framerate *default-framerate*)
   (defun slide-tag (images)
     (if (eql *default-framerate* $wxanimate_framerate)
-	($ldisp (list '(wxxmltag simp) (format nil "~{~a;~}" images) "slide" (if (eql $wxanimate_autoplay 't) "running=\"true\" del=\"true\"" "running=\"false\" del=\"true\"")))
-      ($ldisp (list '(wxxmltag simp) (format nil "~{~a;~}" images) "slide" (if (eql $wxanimate_autoplay 't) (format nil "fr=\"~a\" running=\"true\" del=\"true\"" $wxanimate_framerate)  (format nil "fr=\"~a\" running=\"false\" del=\"true\"" $wxanimate_framerate))
-                    ))))
+	($ldisp
+	 (list '(wxxmltag simp)
+	       (wxxml-fix-string (format nil "~{~a;~}" images))
+	       "slide"
+	       (if (eql $wxanimate_autoplay 't)
+		   "running=\"true\" del=\"yes\""
+		 "running=\"false\" del=\"yes\""))
+	 (format nil "~%"))
+      ($ldisp
+       (list '(wxxmltag simp)
+	     (wxxml-fix-string
+	      (format nil "~{~a;~}" images))
+	     "slide"
+	     (if (eql $wxanimate_autoplay 't)
+		 (format nil "fr=\"~a\" running=\"true\" del=\"yes\"" $wxanimate_framerate)
+	       (format nil "fr=\"~a\" running=\"false\" del=\"yes\"" $wxanimate_framerate)))
+       (format nil "~%"))))
 
   (defun wxanimate (scene)
     (let* ((scene (cdr scene))
@@ -1549,7 +1566,7 @@
 			 ((mlist simp) $gnuplot_term ,(if $wxplot_pngcairo '$pngcairo '$png))
 			 ((mlist simp) $gnuplot_preamble ,preamble)
 			 ((mlist simp) $gnuplot_out_file ,filename)))
-      ($ldisp `((wxxmltag simp) ,filename "img")))
+      ($ldisp `((wxxmltag simp) ,(wxxml-fix-string filename) "img")))
     "")
 
   (defun $wxplot3d (&rest args)
@@ -1568,7 +1585,7 @@
 			 ((mlist simp) $gnuplot_term ,(if $wxplot_pngcairo '$pngcairo '$png))
 			 ((mlist simp) $gnuplot_preamble ,preamble)
 			 ((mlist simp) $gnuplot_out_file ,filename)))
-      ($ldisp `((wxxmltag simp) ,filename "img")))
+      ($ldisp `((wxxmltag simp) ,(wxxml-fix-string filename) "img")))
     "")
 
 
@@ -1614,14 +1631,14 @@
 			   `(((mequal simp) $dimensions ,$wxplot_size)))))))
       (if $display_graphics
 	  (progn
-	    ($ldisp `((wxxmltag simp) ,(format nil "~a.png" filename) "img"
+	    ($ldisp `((wxxmltag simp) ,(wxxml-fix-string (format nil "~a.png" filename)) "img"
 		      ,(if file_name_spec
 			   (format nil "del=\"no\" gnuplotsource=\"~a/~a\" gnuplotdata=\"~a/~a\"" $maxima_tempdir gnuplotfilename $maxima_tempdir datafilename)
 			 (format nil "del=\"yes\" gnuplotsource=\"~a/~a\" gnuplotdata=\"~a/~a\"" $maxima_tempdir gnuplotfilename $maxima_tempdir datafilename)
 			 )
 		      ))
 	    (setq res ""))
-	(setf res `((wxxmltag simp) ,(format nil "~a.png" filename) "img")))
+	(setf res `((wxxmltag simp) ,(wxxml-fix-string (format nil "~a.png" filename)) "img")))
       res))
 
   (defmspec $wxdraw_list (args)
@@ -1646,7 +1663,7 @@
 				((mlist simp) $gnuplot_term ,(if $wxplot_pngcairo '$pngcairo '$png))
 				((mlist simp) $gnuplot_preamble ,preamble)
 				((mlist simp) $gnuplot_out_file ,filename)))
-      ($ldisp `((wxxmltag simp) ,filename "img")))
+      ($ldisp `((wxxmltag simp) ,(wxxml-fix-string filename) "img")))
     "")
 
 
@@ -1667,11 +1684,11 @@
 			       ((mlist simp) $gnuplot_preamble ,preamble)
 			       ((mlist simp) $gnuplot_out_file ,filename)))
 
-      ($ldisp `((wxxmltag simp) ,filename "img")))
+      ($ldisp `((wxxmltag simp) ,(wxxml-fix-string filename) "img")))
     "")
 
   (defun $show_image (file)
-    ($ldisp `((wxxmltag simp) ,file "img" "del=\"no\"")))
+    ($ldisp `((wxxmltag simp) ,(wxxml-fix-string file) "img" "del=\"no\"")))
 
   ;;
   ;; Port of Barton Willis's texput function.
@@ -1806,6 +1823,15 @@
 			(cdr fun))))
       (format nil "<function>~a</function><template>~a(~{&lt;~a&gt;~^, ~})</template>" fun-name fun-name args)))
 
+  ;; Add the variable val to the watch list
+  (defun wx-add-variable-name (val)
+    (format t "<variable>~a</variable>~%" (symbol-to-xml val)))
+  ;; Add all currently defined variables to the watch list
+  (defun wx-add-all-variables ()
+    (format t "<watch_variables_add>~%")
+    (mapcar #'wx-add-variable-name (cdr $values))
+    (format t "</watch_variables_add>~%"))
+
   (defun print_value (val)
     (format nil "<value>~a</value>" (symbol-to-xml val)))
 
@@ -1816,7 +1842,7 @@
 
 
   ;; A function that determines all symbols for autocompletion
-  (defun wxPrint_autoompletesymbols ()
+  (defun wxPrint_autocompletesymbols ()
     #+clisp (finish-output)
     (format t "<wxxml-symbols>")
     ;; Function names and rules
@@ -1833,12 +1859,10 @@
 	     *variable-initial-values*)
 
     ;; ezunits publishes all known units in a function.
-    ;; Currently the following lines produce a warning, though, that
-    ;; $known_units is undefined.
-    ;;	(if (boundp '$known_units)
-    ;;	    (no-warning
-    ;;	     (format t "~{~a~^$~}"
-    ;;		     (mapcar #'print_unit (cdr ($known_units))))))
+    (if (boundp '$known_units)
+    	(no-warning
+    	 (format t "~{~a~^$~}"
+    		 (mapcar #'print_unit (cdr ($known_units))))))
     (format t "</wxxml-symbols>")
     #+clisp (finish-output)
     )
@@ -1848,16 +1872,21 @@
 ;;; Communication between wxMaxima and wxMaxima about variables and directories
 ;;;
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
 ;;; Communicate the contents of variables to wxMaxima
   (defun wx-print-variable (var)
-    (format t "<variable>")
-    (format t "<name>~a</name>" (symbol-to-xml var))
-    (if (boundp var)
-	(format t "<value>~a</value>" (wxxml-fix-string(eval var))))
+    (format t "<variable>~%<name>~a</name>" (symbol-to-xml var))
+    (ignore-errors
+      (let (($display2d nil))
+	    (mtell "<value>~M</value>" (wxxml-fix-string(eval var)))))
     (format t "</variable>"))
 
+  (defun wx-query-variable (var)
+    (format t "<variables>~%<variable>~%<name>~a</name>" (wxxml-fix-string (maybe-invert-string-case var)))
+    (ignore-errors
+      (let (($display2d nil))
+	(mtell "<value>~M</value>" (wxxml-fix-string(meval (intern var))))))
+      (format t "</variable>~%</variables>~%"))
+
   (defun wx-print-variables ()
     #+clisp (finish-output)
     (format t "<variables>")
@@ -1877,7 +1906,7 @@
     (format t "<variable><name>*lisp-version*</name><value>~a</value></variable>"
 	    #+sbcl (ensure-readably-printable-string (lisp-implementation-version))
 	    #-sbcl (lisp-implementation-version))
-    (format t "</variables>")
+    (format t "</variables>~%")
     #+clisp (finish-output)
     )
 
@@ -1909,6 +1938,7 @@
 	  #+gcl (xchdir dir)
 	  #+gcl (setf *default-pathname-defaults* dir)
 
+	  (ignore-errors (setf *default-pathname-defaults* dir))
 	  #-(or allegro clisp cmu cormanlisp gcl lispworks lucid sbcl ccl ecl) (format t
            "Info: wxMathml.cpp: Changing the working dir during a maxima session isn't implemented for this lisp.")
   	  (namestring dir)
@@ -1973,7 +2003,7 @@
 
   ;; Load the initial functions (from mac-init.mac)
   (let ((*print-circle* nil))
-    (format t "<wxxml-symbols>~{~a~^$~}</wxxml-symbols>"
+    (format t "<wxxml-symbols>~{~a~^$~}</wxxml-symbols>~%"
 	    (mapcar #'$print_function (cdr ($append $functions $macros)))))
 
   (no-warning
@@ -2021,6 +2051,38 @@
   (when ($file_search "wxmaxima-init")
     ($load "wxmaxima-init"))
 
+  ;;; From Elias: Code that extracts the name of our function's parameters
+  (defun arglist-from-function (name)
+    (ignore-errors (cadr (function-lambda-expression name))
+    ))
+
+  (defun arglist-from-maxima-function (name)
+    (labels ((format-list-entry (v)
+				(etypecase v
+					   (symbol (format-sym-name v :any-sym t))
+					   (list (progn
+						   (assert (eq (caar v) 'maxima::mlist))
+						   (format nil "[~{~a~^, ~}]" (mapcar #'format-list-entry (cdr v))))))))
+	    (let ((mexpr (or (maxima::mget name 'maxima::mexpr)
+			     (maxima::mget name 'maxima::mmacro))))
+	      (cond (mexpr
+		     ;; I'm not entirely certain about how the content of the
+		     ;; function definition can vary, so let's add some assertions
+		     ;; that encodes the current understanding of the situation.
+		     (assert (eq (caar mexpr) 'lambda))
+		     (let ((arglist (second mexpr)))
+		       (assert (eq (caar arglist) 'maxima::mlist))
+		       (format nil "(~{~a~^, ~})" (mapcar #'format-list-entry (cdr arglist)))))
+		    (t
+		     "variable")))))
+
+  (defun function-signature (name)
+    (check-type name symbol)
+    (if (fboundp name)
+	(arglist-from-function name)
+      ;; ELSE: Possibly a Maxima function?
+      (arglist-from-maxima-function name)))
+  
 ;;;
 ;;; Now that we have loaded the init file we can rewrite of the function load
 ;;; (maxima/src/mload.lisp) to displays functions and variable names after
@@ -2030,49 +2092,35 @@
 ;;; We also inform wxMaxima about the name of the package and about the fact
 ;;; if it was loaded by another package.
 ;;;
+
+  (defvar *wxmaxima-nested-loads* 0 "How many load commands are nested, currently?")
+  (setf (symbol-function 'load_original_wxmaxima) (symbol-function '$load))
+  
   (no-warning
    (defun $load (filename)
-     (let ((searched-for
-	    ($file_search1 filename
-			   '((mlist) $file_search_maxima $file_search_lisp  )))
-	   type)
-       (setq type ($file_type searched-for))
-       (case type
-	     (($maxima)
-	      #+clisp (finish-output)
-	      (format t "<variables>")
-	      (format t "<variable><name>*wx-load-file-name*</name><value>~a</value></variable>"
-		      (wxxml-fix-string filename))
-	      (format t "<variable><name>*wx-load-file-start*</name><value>1</value></variable>")
-	      (format t "</variables>")
-	      ($batchload searched-for)
-	      (format t "<variables>")
-	      (format t "<variable><name>*wx-load-file-start*</name><value>0</value></variable>")
-	      (format t "</variables>")
-	      (wxPrint_autoompletesymbols))
-	     (($lisp $object)
-	      ;; do something about handling errors
-	      ;; during loading. Foobar fail act errors.
-	      (format t "<variables>")
-	      (format t "<variable><name>*wx-load-file-name*</name><value>~a</value></variable>"
-		      (wxxml-fix-string filename))
-	      (format t "<variable><name>*wx-load-file-start*</name><value>1</value></variable>")
-	      (format t "</variables>")
-	      (no-warning
-	       (load-and-tell searched-for))
-	      (format t "<variables>")
-	      (format t "<variable><name>*wx-load-file-start*</name><value>0</value></variable>")
-	      (format t "</variables>")
-	      #+clisp (finish-output)
-	      (wxPrint_autoompletesymbols))
-	     (t
-	      (merror "Maxima bug: Unknown file type ~M" type)))
-       searched-for)))
-  (force-output)
-  (format t "</suppressOutput>")
+
+     ;; Inform wxMaxima that we load a file.
+     (if (= *wxmaxima-nested-loads* 0)
+	 (progn
+	   (format t "<variables>")
+	   (format t "<variable><name>*wx-load-file-name*</name><value>~a</value></variable>"
+		   (wxxml-fix-string filename))
+	   (format t "</variables>~%")))
+     (setq *wxmaxima-nested-loads* (+ *wxmaxima-nested-loads* 1))
+     ;; Load the file
+     (unwind-protect
+	 (load_original_wxmaxima filename)
+       (progn
+	 ;; After loading the file: Tell wxMaxima we have finished loading the file
+	 ;; and what autocompletable symbols we know about.
+	 (setq *wxmaxima-nested-loads* (- *wxmaxima-nested-loads* 1))
+	 (if (= *wxmaxima-nested-loads* 0)
+	     (wxPrint_autocompletesymbols))
+	 ))))
+  (format t "</suppressOutput>~%")
   ;; Publish all new global variables maxima might contain to wxMaxima's
   ;; autocompletion feature.
-  (wxPrint_autoompletesymbols)
+  (wxPrint_autocompletesymbols)
   (wx-print-variables)
-  (force-output)
+  (finish-output)
 )
