@@ -35,15 +35,6 @@ AtCell::AtCell(Cell *parent, Configuration **config, CellPointers *cellPointers)
   m_indexCell = NULL;
 }
 
-void AtCell::SetGroup(Cell *parent)
-{
-  m_group = parent;
-  if (m_baseCell != NULL)
-    m_baseCell->SetGroupList(parent);
-  if (m_indexCell != NULL)
-    m_indexCell->SetGroupList(parent);
-}
-
 Cell *AtCell::Copy()
 {
   AtCell *tmp = new AtCell(m_group, m_configuration, m_cellPointers);
@@ -90,19 +81,18 @@ void AtCell::SetBase(Cell *base)
 
 void AtCell::RecalculateWidths(int fontsize)
 {
-  Cell::RecalculateWidths(fontsize);
   m_baseCell->RecalculateWidthsList(fontsize);
-  m_indexCell->RecalculateWidthsList(MAX(MC_MIN_SIZE, fontsize - 3));
+  m_indexCell->RecalculateWidthsList(wxMax(MC_MIN_SIZE, fontsize - 3));
   m_width = m_baseCell->GetFullWidth() + m_indexCell->GetFullWidth() +
             Scale_Px(4);
-  ResetData();
+  Cell::RecalculateWidths(fontsize);
 }
 
 void AtCell::RecalculateHeight(int fontsize)
 {
   Cell::RecalculateHeight(fontsize);
   m_baseCell->RecalculateHeightList(fontsize);
-  m_indexCell->RecalculateHeightList(MAX(MC_MIN_SIZE, fontsize - 3));
+  m_indexCell->RecalculateHeightList(wxMax(MC_MIN_SIZE, fontsize - 3));
   m_height = m_baseCell->GetMaxHeight() + m_indexCell->GetMaxHeight() -
              Scale_Px(7);
   m_center = m_baseCell->GetCenter();
@@ -143,6 +133,14 @@ wxString AtCell::ToString()
   return s;
 }
 
+wxString AtCell::ToMatlab()
+{
+  wxString s = wxT("at(");
+  s += m_baseCell->ListToMatlab();
+  s += wxT(",") + m_indexCell->ListToMatlab() + wxT(")");
+  return s;
+}
+
 wxString AtCell::ToTeX()
 {
   wxString s = wxT("\\left. ");
