@@ -49,10 +49,14 @@ wxString Cell::GetToolTip(const wxPoint &point)
 }
 
 Cell::Cell(Cell *group, Configuration **config)
+#if wxUSE_ACCESSIBILITY
+  :wxAccessible()
+#endif
 {
+  m_lastZoomFactor = -1;
+  m_clientWidth_old = -1;
+  m_group = group;
   m_textStyle = TS_DEFAULT;
-  m_toolTip = wxEmptyString;
-  m_toolTip = wxEmptyString;
   m_cellPointers = NULL;
   m_group = group;
   m_parent = group;
@@ -68,10 +72,10 @@ Cell::Cell(Cell *group, Configuration **config)
   m_width = -1;
   m_height = -1;
   m_center = -1;
-  m_breakLine = false;
+  SoftLineBreak(false);
   m_breakPage = false;
   m_forceBreakLine = false;
-  m_bigSkip = true;
+  m_bigSkip = false;
   m_isHidden = false;
   m_isBrokenIntoLines = false;
   m_highlight = false;
@@ -115,7 +119,7 @@ void Cell::SetType(CellType type)
       break;
     case MC_TYPE_LABEL:
       m_textStyle = TS_LABEL;
-      ForceBreakLineHere();
+      HardLineBreak();
       break;
     case MC_TYPE_INPUT:
       m_textStyle = TS_INPUT;
@@ -152,6 +156,8 @@ void Cell::SetType(CellType type)
       break;
   }
   ResetSize();
+  if(m_group != NULL)
+    GetGroup()->ResetSize();
 }
 
 Cell *Cell::CopyList()
@@ -192,6 +198,45 @@ void Cell::SetGroupList(Cell *parent)
   }
 }
 
+int Cell::CellsInListRecursive()
+{
+  
+  //! The number of cells the current group contains (-1, if no GroupCell)
+  int cells = 0;
+
+  Cell *tmp = this;
+
+  while(tmp != NULL)
+  {
+    cells ++;
+    std::list<Cell*> cellList = tmp->GetInnerCells();
+    for (std::list<Cell *>::iterator it = cellList.begin(); it != cellList.end(); ++it)
+    {
+      if(*it != NULL)
+        cells += (*it)->CellsInListRecursive();
+    }
+    tmp = tmp->m_next;
+  }
+  return cells;
+}
+
+void Cell::SetGroup(Cell *group)
+{
+  m_group = group;
+  if(group != NULL)
+  {
+    wxASSERT (group->GetType() == MC_TYPE_GROUP);
+  }
+  
+  std::list<Cell*> cellList = GetInnerCells();
+  for (std::list<Cell *>::iterator it = cellList.begin(); it != cellList.end(); ++it)
+  {
+    if(*it != NULL)
+      (*it)->SetGroupList(group);
+  }
+}
+
+
 /***
  * Append new cell to the end of this list.
  */
@@ -241,19 +286,29 @@ int Cell::GetMaxCenter()
       if ((tmp != this) && (tmp->m_breakLine))
         break;
       if(!tmp->m_isBrokenIntoLines)
-        m_maxCenter = MAX(m_maxCenter, tmp->m_center);
+        m_maxCenter = wxMax(m_maxCenter, tmp->m_center);
       tmp = tmp->m_nextToDraw;
     }
   }
   return m_maxCenter;
 }
 
+bool Cell::NeedsRecalculation()
+{
+  return (m_width < 0) || (m_height < 0) || (m_center < 0) ||
+    (m_currentPoint.x < 0) || (m_currentPoint.y < 0) ||
+    (m_clientWidth_old != (*m_configuration)->GetClientWidth()) ||
+    (m_lastZoomFactor != (*m_configuration)->GetZoomFactor()) ||
+    ((*m_configuration)->RecalculationForce()) ||
+    (*m_configuration)->FontChanged();
+}
+
 /***
  * Get the maximum drop of cell.
  */
 int Cell::GetMaxDrop()
 {
-  if ((m_maxDrop < 0) || ((*m_configuration)->RecalculationForce()))
+//  if ((m_maxDrop < 0) || ((*m_configuration)->RecalculationForce()))
   {
     m_maxDrop = 0;
     Cell *tmp = this;
@@ -262,7 +317,7 @@ int Cell::GetMaxDrop()
       if ((tmp != this) && (tmp->m_breakLine))
         break;
       if(!tmp->m_isBrokenIntoLines)
-        m_maxDrop = MAX(m_maxDrop, tmp->m_height - tmp->m_center);
+        m_maxDrop = wxMax(m_maxDrop, tmp->m_height - tmp->m_center);
       tmp = tmp->m_nextToDraw;
     }
   }
@@ -332,7 +387,8 @@ int Cell::GetLineWidth()
  */
 void Cell::Draw(wxPoint point)
 {
-  SetCurrentPoint(point);
+  if((m_height > 0) && (point.y > 0))
+    SetCurrentPoint(point);
 
   // Tell the screen reader that this cell's contents might have changed.
 
@@ -411,6 +467,8 @@ void Cell::RecalculateWidths(int fontsize)
 {
   ResetData();
   m_fontSize = fontsize;
+  m_clientWidth_old = (*m_configuration)->GetClientWidth();
+  m_lastZoomFactor = (*m_configuration)->GetZoomFactor();
 }
 
 /*! Is this cell currently visible in the window?.
@@ -423,15 +481,15 @@ bool Cell::DrawThisCell(wxPoint point)
 
   SetCurrentPoint(point);
 
-  if(!(*m_configuration)->ClipToDrawRegion())
-    return true;
-  
-  // If a cell is broken into lines its individual parts are displayed but
+  // If a cell is broken into lines the cells it contains are displayed but
   // not the cell itself (example: Denominator and Numerator are displayed
   // but not the horizontal line with denominator above and numerator below.
   if(m_isBrokenIntoLines)
     return false;
-
+  
+  if(!(*m_configuration)->ClipToDrawRegion())
+    return true;
+  
   return(InUpdateRegion());
 }
 
@@ -456,12 +514,31 @@ bool Cell::InUpdateRegion(const wxRect &rect)
   if (!(*m_configuration)->ClipToDrawRegion())
     return true;
 
-  if((*m_configuration)->GetUpdateRegion().Contains(m_currentPoint))
+  if((m_currentPoint.x < 0) || (m_currentPoint.y < 0))
+    return false;
+  
+  wxRect updateRegion = (*m_configuration)->GetUpdateRegion();
+
+  // If we have deferred the recalculation of the cell height but now
+  // got a draw request due to moving the mouse wheel we need to guess
+  // the cell size 
+  if(m_height < 0)
+  {
+    int height = 0;
+    if(m_next)
+      height = m_next->m_currentPoint.y - m_currentPoint.y;
+
+    if ((updateRegion.GetBottom() >= m_currentPoint.y) &&
+        (updateRegion.GetTop() <= m_currentPoint.y+height))
+    return true;
+  }
+
+  if(updateRegion.Contains(m_currentPoint))
     return true;
 
-  return (*m_configuration)->GetUpdateRegion().Intersects(rect) ||
-    (*m_configuration)->GetUpdateRegion().Contains(rect) ||
-    ((*m_configuration)->GetUpdateRegion() == rect);
+  return updateRegion.Intersects(rect) ||
+    updateRegion.Contains(rect) ||
+    (updateRegion == rect) || rect.Contains(updateRegion);
 }
 
 void Cell::DrawBoundingBox(wxDC &dc, bool all)
@@ -537,6 +614,47 @@ wxString Cell::ListToString()
   return retval;
 }
 
+wxString Cell::ToMatlab()
+{
+  return wxEmptyString;
+}
+
+wxString Cell::ListToMatlab()
+{
+	wxString retval;
+	Cell *tmp = this;
+	bool firstline = true;
+
+	while (tmp != NULL)
+	{
+	  if ((!firstline) && (tmp->m_forceBreakLine))
+	  {
+		if(!retval.EndsWith(wxT('\n')))
+		  retval += wxT("\n");
+		// if(
+		//    (tmp->GetStyle() != TS_LABEL) &&
+		//    (tmp->GetStyle() != TS_USERLABEL) &&
+		//    (tmp->GetStyle() != TS_MAIN_PROMPT) &&
+		//    (tmp->GetStyle() != TS_OTHER_PROMPT))
+		//   retval += wxT("\t");
+	  }
+	  // if(firstline)
+	  // {
+	  //   if((tmp->GetStyle() != TS_LABEL) &&
+	  //      (tmp->GetStyle() != TS_USERLABEL) &&
+	  //      (tmp->GetStyle() != TS_MAIN_PROMPT) &&
+	  //      (tmp->GetStyle() != TS_OTHER_PROMPT))
+	  //     retval += wxT("\t");
+	  // }
+	  retval += tmp->ToMatlab();
+
+	  firstline = false;
+	  tmp = tmp->m_nextToDraw;
+	}
+
+	return retval;
+}
+
 wxString Cell::ToTeX()
 {
   return wxEmptyString;
@@ -555,14 +673,6 @@ wxString Cell::ListToTeX()
     retval += tmp->ToTeX();
     tmp = tmp->m_next;
   }
-
-  // TODO: Things like {a}_{b} make the LaTeX code harder to read. But things like
-  // \sqrt{a} need us to use braces from time to time.
-  //
-  // How far I got was:
-  //
-  //  wxRegEx removeUnneededBraces1(wxT("{([a-zA-Z0-9])}([{}_a-zA-Z0-9 \\\\^_])"));
-  //  removeUnneededBraces1.Replace(&retval,wxT(" \\1\\2"),true);
   return retval;
 }
 
@@ -587,7 +697,7 @@ wxString Cell::ListToMathML(bool startofline)
   Cell *temp = this;
   while (temp)
   {
-    if (temp->ForceBreakLineHere())
+    if (temp->HardLineBreak())
       needsTable = true;
 
     if (temp->GetType() == MC_TYPE_LABEL)
@@ -609,11 +719,11 @@ wxString Cell::ListToMathML(bool startofline)
       retval += wxT("</mrow>");
 
     // Handle linebreaks
-    if ((temp != this) && (temp->ForceBreakLineHere()))
+    if ((temp != this) && (temp->HardLineBreak()))
       retval += wxT("</mtd></mlabeledtr>\n<mlabeledtr columnalign=\"left\"><mtd>");
 
     // If a linebreak isn't followed by a label we need to introduce an empty one.
-    if ((((temp->ForceBreakLineHere()) || (startofline && (this == temp))) &&
+    if ((((temp->HardLineBreak()) || (startofline && (this == temp))) &&
          ((temp->GetStyle() != TS_LABEL) && (temp->GetStyle() != TS_USERLABEL))) && (needsTable))
       retval += wxT("<mtext></mtext></mtd><mtd>");
 
@@ -777,7 +887,7 @@ wxString Cell::ListToOMML(bool WXUNUSED(startofline))
     retval += token;
 
     // Hard linebreaks aren't supported by OMML and therefore need a new equation object
-    if (tmp->ForceBreakLineHere())
+    if (tmp->HardLineBreak())
       break;
 
     tmp = tmp->m_next;
@@ -834,7 +944,7 @@ wxString Cell::ListToRTF(bool startofline)
             break;
 
           // A newline starts a new equation
-          if (tmp->ForceBreakLineHere())
+          if (tmp->HardLineBreak())
           {
             tmp = tmp->m_next;
             break;
@@ -987,7 +1097,10 @@ void Cell::ResetData()
   m_lineWidth = -1;
   m_maxCenter = -1;
   m_maxDrop   = -1;
-  m_breakLine = m_forceBreakLine;
+  std::list<Cell*> cellList = GetInnerCells();
+  for (std::list<Cell *>::iterator it = cellList.begin(); it != cellList.end(); ++it)
+    if(*it != NULL)
+      (*it)->ResetData();
 }
 
 Cell *Cell::first()
@@ -1010,11 +1123,19 @@ Cell *Cell::last()
 
 void Cell::Unbreak()
 {
+//  if(m_isBrokenIntoLines)
   ResetData();
+
   m_isBrokenIntoLines = false;
   m_nextToDraw = m_next;
   if (m_nextToDraw != NULL)
     m_nextToDraw->m_previousToDraw = this;
+
+  // Unbreak the inner cells, too
+  std::list<Cell *> innerCells = GetInnerCells();
+  for(std::list<Cell *>::iterator it = innerCells.begin(); it != innerCells.end(); ++it)
+    if(*it != NULL)
+      (*it)->Unbreak();
 }
 
 void Cell::UnbreakList()
@@ -1322,6 +1443,7 @@ wxAccStatus Cell::GetRole (int WXUNUSED(childId), wxAccRole *role)
 
 Cell::CellPointers::CellPointers(wxScrolledCanvas *mathCtrl)
 {
+  m_scrollToCell = false;
   m_cellToScrollTo = NULL;
   m_wxmxImgCounter = 0;
   m_mathCtrl = mathCtrl;
