@@ -24,11 +24,10 @@
 
 #include <wx/wx.h>
 #include <wx/config.h>
+#include <wx/display.h>
 #include <wx/fontenum.h>
 
 #include "TextStyle.h"
-#include "Dirstructure.h"
-
 
 #define MC_LINE_SKIP Scale_Px(2)
 #define MC_TEXT_PADDING Scale_Px(1)
@@ -43,7 +42,7 @@
 //! The width of the horizontally-drawn cursor
 #define MC_HCARET_WIDTH 25
 
-#if defined __WXMAC__
+#if defined __WXOSX__
 #define MC_EXP_INDENT 2
 #define MC_MIN_SIZE 10
 #define MC_MAX_SIZE 36
@@ -86,7 +85,15 @@
 class Configuration
 {
 public:
-  Dirstructure m_dirStructure;
+  //! The export formats we support for HTML equations
+  enum htmlExportFormat
+  {
+    mathJaX_TeX = 0,
+    bitmap = 1,
+    mathML_mathJaX = 2,
+    svg = 3
+  };
+
   enum drawMode
   {
     ascii,              //!< Use ascii characters only
@@ -97,6 +104,10 @@ public:
     unknown
   };
 
+  WX_DECLARE_STRING_HASH_MAP(wxString, StringHash);
+  //! A list of all symbols that can be entered using Esc-Codes
+  StringHash m_escCodes;
+
   //! Set maxima's working directory
   void SetWorkingDirectory(wxString dir)
   { m_workingdir = dir; }
@@ -110,7 +121,7 @@ public:
     
     \param dc The drawing context that is to be used for drawing objects
    */
-  Configuration(wxDC &dc);
+  Configuration(wxDC *dc = NULL);
 
   //! Set the drawing context that is currently active
   void SetContext(wxDC &dc)
@@ -119,6 +130,8 @@ public:
     m_antialiassingDC = NULL;
   }
 
+  void SetBackgroundBrush(wxBrush brush){m_BackgroundBrush = brush;}
+  wxBrush GetBackgroundBrush(){return m_BackgroundBrush;}
   void SetAntialiassingDC(wxDC &antialiassingDC)
     {m_antialiassingDC = &antialiassingDC;}
 
@@ -127,6 +140,9 @@ public:
 
   ~Configuration();
 
+  static wxString m_maximaLocation_override;
+  static wxString m_configfileLocation_override;
+  
   static double GetMinZoomFactor()
   { return 0.1; }
 
@@ -236,8 +252,6 @@ public:
 
   bool IsUnderlined(int st);
 
-  void ReadStyle();
-
   //! Force a full recalculation?
   void RecalculationForce(bool force)
   {
@@ -255,6 +269,11 @@ public:
     return m_fontEncoding;
   }
 
+  void SetFontEncoding(wxFontEncoding encoding)
+  {
+    m_fontEncoding = encoding;
+  }
+
   int GetLabelWidth()
   { return m_labelWidth * 14; }
 
@@ -267,15 +286,27 @@ public:
       return m_indent;
   }
 
+  //! Get the resolution of the display showing the worksheet
+  wxSize GetPPI(){return GetPPI(GetWorkSheet());}
+  //! Get the resolution of an arbitrary display
+  wxSize GetPPI(wxWindow *win);
+  
   //! How much vertical space is to be left between two group cells?
   int GetCursorWidth()
   {
-    if (wxGetDisplayPPI().x / 45 < 1)
+    int ppi;
+
+    if(!m_printing)
+      ppi = GetPPI().x;
+    else
+      ppi = 96;
+
+    if (ppi / 45 < 1)
       return 1;
     else
-      return wxGetDisplayPPI().x / 45;
+      return ppi / 45;
   }
-
+  
   //! The y position the worksheet starts at
   int GetBaseIndent()
   {
@@ -376,9 +407,15 @@ public:
   int GetDefaultFontSize()
   { return m_defaultFontSize; }
 
+  void SetDefaultFontSize(int fontSize)
+  { m_defaultFontSize = fontSize; }
+
   int GetMathFontSize()
   { return m_mathFontSize; }
 
+  void SetMathFontSize(double size)
+  { m_mathFontSize = size; }
+
   //! Do we want to have automatic line breaks for text cells?
   bool GetAutoWrap()
   { return m_autoWrap > 0; }
@@ -410,13 +447,26 @@ public:
   //! Do we want to indent all maths?
   bool IndentMaths(){return m_indentMaths;}
   void IndentMaths(bool indent){wxConfig::Get()->Write(wxT("indentMaths"), m_indentMaths=indent);}
-  int GetFontSize(int st)
+  int GetFontSize(TextStyle st)
   {
     if (st == TS_TEXT || st == TS_HEADING5 || st == TS_HEADING6 || st == TS_SUBSUBSECTION || st == TS_SUBSECTION || st == TS_SECTION || st == TS_TITLE)
-      return m_styles[st].fontSize;
+      return m_styles[st].FontSize();
     return 0;
   }
 
+  /*! Reads the style settings 
+
+    If a file name is given the settings are read from a file.
+  */
+  
+  void ReadStyles(wxString file = wxEmptyString);
+  
+  /*! Saves the style settings 
+
+    If a file name is given the settings are written to a file.
+  */
+  void WriteStyles(wxString file = wxEmptyString);
+  
   void Outdated(bool outdated)
   { m_outdated = outdated; }
 
@@ -573,14 +623,27 @@ public:
     return print;
   }
 
+  //! Autodetect maxima's location? (If false the user-specified location is used)
+  bool AutodetectMaxima(){return m_autodetectMaxima;}
+  //! Autodetect maxima's location?
+  void AutodetectMaxima(bool autodetectmaxima){wxConfig::Get()->Write(
+      wxT("autodetectMaxima"),
+      m_autodetectMaxima = autodetectmaxima);
+  }
+
+  //! The auto-detected maxima location
+  wxString MaximaDefaultLocation();
+
   //! Returns the location of the maxima binary.
-  wxString MaximaLocation()
-  { return m_maximaLocation; }
+  wxString MaximaLocation();
+
+  //! Returns the location of the maxima binary the user has selected.
+  wxString MaximaUserLocation(){return m_maximaUserLocation;}
 
   //! Sets the location of the maxima binary.
-  void MaximaLocation(wxString maxima)
+  void MaximaUserLocation(wxString maxima)
   {
-    wxConfig::Get()->Write(wxT("maxima"), m_maximaLocation = maxima);
+    wxConfig::Get()->Write(wxT("maxima"), m_maximaUserLocation = maxima);
   }
 
   /*! Could a maxima binary be found in the path we expect it to be in?
@@ -601,7 +664,12 @@ public:
   }
 
   //! Returns the URL MathJaX can be found at.
-  wxString MathJaXURL(){ return m_mathJaxURL;}
+  wxString MathJaXURL(){if(m_mathJaxURL_UseUser) return m_mathJaxURL; else return MathJaXURL_Auto();}
+  wxString MathJaXURL_User(){ return m_mathJaxURL;}
+  bool MathJaXURL_UseUser(){ return m_mathJaxURL_UseUser;}
+  void MathJaXURL_UseUser(bool useUser){wxConfig::Get()->Write(wxT("mathJaxURL_UseUser"),
+                                                               m_mathJaxURL_UseUser = useUser);}
+  wxString MathJaXURL_Auto(){ return wxT("https://cdnjs.cloudflare.com/ajax/libs/mathjax/2.7.5/MathJax.js?config=TeX-AMS_HTML");}
   //! Returns the URL MathJaX can be found at.
   void MathJaXURL(wxString url){wxConfig::Get()->Write(wxT("mathJaxURL"), m_mathJaxURL = url);}
   bool AntiAliasLines(){return m_antiAliasLines;}
@@ -642,7 +710,7 @@ public:
     }
   void ShowLength(int length)
     {
-      wxConfig::Get()->Write(wxT("showLength"), m_showLength = length );
+      wxConfig::Get()->Write(wxT("showLength"), m_showLength = length);
     }
   int ShowLength(){return m_showLength;}
 
@@ -673,7 +741,7 @@ public:
     \param fontSize Only relevant for math cells: Super- and subscripts can have different
     font styles than the rest.
    */
-  wxFont GetFont(int textStyle, int fontSize);
+  wxFont GetFont(TextStyle textStyle, int fontSize);
 
   //! Get the worksheet this configuration storage is valid for
   wxWindow *GetWorkSheet(){return m_workSheet;}
@@ -681,24 +749,29 @@ public:
   void SetWorkSheet(wxWindow *workSheet){m_workSheet = workSheet;}
 
   //! Get the autosave interval [in milliseconds]; 0 = no autosave
-  int AutoSaveInterval(){if (m_autoSaveInterval > 1) return m_autoSaveInterval; else return 0;}
-  //! Set the autosave interval [in milliseconds]; 0 = noautosave
-  void AutoSaveInterval(int miliseconds){wxConfig::Get()->Write(wxT("autoSaveInterval"),
-                                                  (m_autoSaveInterval = miliseconds / 1000 / 60)
-      );
+  int AutoSaveMiliseconds(){return m_autoSaveMinutes * 1000 * 60;}
+  //! Get the autosave interval [in minutes]; 0 = no autosave
+  int AutoSaveMinutes(){return m_autoSaveMinutes;}
+  //! Set the autosave interval [in minutes]; 0 = noautosave
+  void AutoSaveMinutes(int minutes){wxConfig::Get()->Write(wxT("autoSaveMinutes"),
+                                                           m_autoSaveMinutes = minutes);
   }
   
   bool GetAbortOnError(){return m_abortOnError;}
   void SetAbortOnError(bool abortOnError)
     {wxConfig::Get()->Write("abortOnError",m_abortOnError = abortOnError);}
-  
+
+  htmlExportFormat HTMLequationFormat(){return m_htmlEquationFormat;}
+  void HTMLequationFormat(htmlExportFormat HTMLequationFormat)
+    {wxConfig::Get()->Write("HTMLequationFormat", (int) (m_htmlEquationFormat = HTMLequationFormat));}
+
   //! Get the worksheet this configuration storage is valid for
   int GetAutosubscript_Num(){return m_autoSubscript;}
   void SetAutosubscript_Num(int autosubscriptnum)
     {wxConfig::Get()->Write("autosubscript",m_autoSubscript = autosubscriptnum);}
   wxString GetAutosubscript_string();
   //! Determine the default background color of the worksheet
-  wxColor DefaultBackgroundColor(){return m_defaultBackgroundColor;}
+  wxColor DefaultBackgroundColor(){return m_styles[TS_DOCUMENT_BACKGROUND].GetColor();}
   //! Do we want to save time by only redrawing the area currently shown on the screen?
   bool ClipToDrawRegion(){return m_clipToDrawRegion;}
   //! Do we want to save time by only redrawing the area currently shown on the screen?
@@ -712,7 +785,12 @@ public:
   wxRect GetVisibleRegion(){return m_visibleRegion;}
   void SetWorksheetPosition(wxPoint worksheetPosition){m_worksheetPosition = worksheetPosition;}
   wxPoint GetWorksheetPosition(){return m_worksheetPosition;}
+  wxString MaximaShareDir(){return m_maximaShareDir;}
+  void MaximaShareDir(wxString dir){m_maximaShareDir = dir;}
+  Style m_styles[NUMBEROFSTYLES];
 private:
+  //! Autodetect maxima's location?
+  bool m_autodetectMaxima;
   //! The worksheet all cells are drawn on
   wxRect m_updateRegion;
   //! Has the font changed?
@@ -721,7 +799,7 @@ private:
 
     Values <10000 mean: Auto-save is off.
   */
-  int m_autoSaveInterval;
+  int m_autoSaveMinutes;
   //! Which objects do we want to convert into subscripts if they occur after an underscore?
   int m_autoSubscript;
   //! The worksheet this configuration storage is valid for
@@ -744,7 +822,8 @@ private:
   //! Caches the information on how to draw big parenthesis for GetGrouphesisDrawMode().
   drawMode m_parenthesisDrawMode;
   wxString m_workingdir;
-  wxString m_maximaLocation;
+
+  wxString m_maximaUserLocation;
   //! Hide brackets that are not under the pointer
   bool m_hideBrackets;
   //! The scale for printing
@@ -786,6 +865,7 @@ private:
   wxString m_fontName;
   int m_defaultFontSize, m_mathFontSize;
   wxString m_mathFontName;
+  wxString m_maximaShareDir;
   bool m_forceUpdate;
   bool m_clipToDrawRegion;
   bool m_outdated;
@@ -797,12 +877,12 @@ private:
   int m_clientWidth;
   int m_clientHeight;
   wxFontEncoding m_fontEncoding;
-  style m_styles[STYLE_NUM];
   bool m_printing;
   int m_lineWidth_em;
   int m_showLabelChoice;
   bool m_fixReorderedIndices;
   wxString m_mathJaxURL;
+  bool m_mathJaxURL_UseUser;
   bool m_showCodeCells;
   bool m_copyBitmap;
   bool m_copyMathML;
@@ -815,6 +895,7 @@ private:
   bool m_useUnicodeMaths;
   bool m_indentMaths;
   bool m_abortOnError;
+  htmlExportFormat m_htmlEquationFormat;
   bool m_adjustWorksheetSizeNeeded;
   //! The rectangle of the worksheet that is currently visible.
   wxRect m_visibleRegion;
@@ -822,6 +903,8 @@ private:
   wxPoint m_worksheetPosition;
 
   wxColour m_defaultBackgroundColor;
+  //! The brush the normal cell background is painted with
+  wxBrush m_BackgroundBrush;
 };
 
 #endif // CONFIGURATION_H
