@@ -36,15 +36,6 @@ DiffCell::DiffCell(Cell *parent, Configuration **config, CellPointers *cellPoint
   m_cellPointers = cellPointers;
 }
 
-void DiffCell::SetGroup(Cell *parent)
-{
-  m_group = parent;
-  if (m_baseCell != NULL)
-    m_baseCell->SetGroupList(parent);
-  if (m_diffCell != NULL)
-    m_diffCell->SetGroupList(parent);
-}
-
 Cell *DiffCell::Copy()
 {
   DiffCell *tmp = new DiffCell(m_group, m_configuration, m_cellPointers);
@@ -99,7 +90,7 @@ void DiffCell::RecalculateWidths(int fontsize)
   m_baseCell->RecalculateWidthsList(fontsize);
   m_diffCell->RecalculateWidthsList(fontsize);
   m_width = m_baseCell->GetFullWidth() + m_diffCell->GetFullWidth();
-  ResetData();
+  Cell::RecalculateWidths(fontsize);
 }
 
 void DiffCell::RecalculateHeight(int fontsize)
@@ -107,8 +98,8 @@ void DiffCell::RecalculateHeight(int fontsize)
   Cell::RecalculateHeight(fontsize);
   m_baseCell->RecalculateHeightList(fontsize);
   m_diffCell->RecalculateHeightList(fontsize);
-  m_center = MAX(m_diffCell->GetMaxCenter(), m_baseCell->GetMaxCenter());
-  m_height = m_center + MAX(m_diffCell->GetMaxDrop(), m_baseCell->GetMaxDrop());
+  m_center = wxMax(m_diffCell->GetMaxCenter(), m_baseCell->GetMaxCenter());
+  m_height = m_center + wxMax(m_diffCell->GetMaxDrop(), m_baseCell->GetMaxDrop());
 }
 
 void DiffCell::Draw(wxPoint point)
@@ -140,6 +131,19 @@ wxString DiffCell::ToString()
   return s;
 }
 
+wxString DiffCell::ToMatlab()
+{
+  if (m_isBrokenIntoLines)
+	return wxEmptyString;
+  Cell *tmp = m_baseCell->m_next;
+  wxString s = wxT("'diff(");
+  if (tmp != NULL)
+	s += tmp->ListToMatlab();
+  s += m_diffCell->ListToMatlab();
+  s += wxT(")");
+  return s;
+}
+
 wxString DiffCell::ToTeX()
 {
   if (m_isBrokenIntoLines)
