@@ -36,6 +36,7 @@
 
 Dirstructure::Dirstructure()
 {
+  m_dirStructure = this;
   m_helpDir = ResourcesDir();
   if(wxDirExists(m_helpDir + wxT("/doc/wxmaxima")))
     m_helpDir += wxT("/doc/wxmaxima");
@@ -129,136 +130,97 @@ wxString Dirstructure::ArtDir()
   return dir;
 }
 
-wxString Dirstructure::MaximaLispLocation()
-{
-  wxString result;
-  wxString basedir = "/usr/local/share/maxima/";
-  wxDir dir (basedir);
-  if(!dir.IsOpened())
-  {
-    basedir = "/usr/share/maxima/";
-    dir.Open(basedir);
-  }
-
-  if(!dir.IsOpened())
-  {
-    basedir = "/opt/local/maxima/";
-    dir.Open(basedir);
-  }
-
-  if(!dir.IsOpened())
-  {
-    basedir = "/sw/maxima/";
-    dir.Open(basedir);
-  }
-
-  if(!dir.IsOpened())
-  {
-    wxFileName maximaLocation(MaximaDefaultLocation());
-    basedir = maximaLocation.GetPath();
-    if(wxDirExists(basedir + "/../maxima/share/maxima"))
-    {
-      basedir = basedir + "/../maxima/share/maxima";
-      dir.Open(basedir);
-    }
-  }
-
-  if(!dir.IsOpened())
-  {
-    wxFileName maximaLocation(MaximaDefaultLocation());
-    basedir = maximaLocation.GetPath();
-    if(wxDirExists(basedir + "/../share/maxima"))
-    {
-      basedir = basedir + "/../share/maxima";
-      dir.Open(basedir);
-    }
-  }
-
-  if(!dir.IsOpened())
-  {
-    wxFileName maximaLocation(MaximaDefaultLocation());
-    basedir = maximaLocation.GetPath();
-    if(wxDirExists(basedir + "/../maxima"))
-    {
-      basedir = basedir + "/../maxima";
-      dir.Open(basedir);
-    }
-  }
-    
-  if(dir.IsOpened())
-  {
-    bool more = dir.GetFirst(&result);
-    while(more)
-      more = dir.GetNext(&result);
-  }
-
-  if(result != wxEmptyString)
-  {
-    result = basedir + wxT("/") + result;
-    return result;
-  }
-
-  result = MaximaDefaultLocation();
-  if(result.EndsWith(".app"))
-  {
-    result += "/";
-    return result;
-  }
-  else
-  {
-    wxFileName maximaName(result);
-    if(maximaName.GetDirCount() > 1)
-    {
-      maximaName.RemoveLastDir();
-    }
-    else
-    {
-      wxArrayString output;
-      wxExecute(wxT("which maxima"), output, (wxEXEC_SYNC |
-                                              wxEXEC_NOEVENTS));
-      if(output.GetCount() > 0)
-      {
-        wxFileName fullpath(output[0] ,wxPATH_NATIVE);
-        return fullpath.GetPath();
-      }
-    }
-    return maximaName.GetPath();
-  }
-}
-
 wxString Dirstructure::MaximaDefaultLocation()
 {
+  wxString notFound = _("Maxima not found as %s");
+  wxString maximaLocation;
+  wxFileName exe = wxStandardPaths::Get().GetExecutablePath();
+  exe.MakeAbsolute();
+  wxString exeDir = exe.GetPathWithSep();
+  wxFileName maxima;  
 #if defined __WXMSW__
-  wxString maxima = wxGetCwd();
-  if (maxima.Right(8) == wxT("wxMaxima"))
-    maxima.Replace(wxT("wxMaxima"), wxT("bin\\maxima.bat"));
-  else
-    maxima.Append("\\maxima.bat");
-  if (!wxFileExists(maxima))
-  {
-    wxFileName exe = wxStandardPaths::Get().GetExecutablePath();
-    exe.MakeAbsolute();
-    wxString exeDir = exe.GetPathWithSep();
-    wxString maximapath = exeDir + wxT("..") + exe.GetPathSeparator() +
-      wxT("bin")  + exe.GetPathSeparator() + wxT("maxima.bat");
-  }
-  wxFileName maximapath(maxima);
-  maximapath.MakeAbsolute();
-  return maximapath.GetFullPath();
-#elif defined __WXMAC__
-  wxString command;
-  if (wxFileExists("/Applications/Maxima.app"))
-    command = wxT("/Applications/Maxima.app");
-  if (wxFileExists("/Applications/maxima.app"))
-    command = wxT("/Applications/maxima.app");
-  else if (wxFileExists("/usr/local/bin/maxima"))
-    command = wxT("/usr/local/bin/maxima");
-  else if (wxFileExists("/usr/bin/maxima"))
-    command = wxT("/usr/bin/maxima");
-  else
-    command = wxT("maxima");
-  return command;
-#else
-  return wxT("maxima");
+  maxima = exeDir + "../bin/maxima.bat";
+  maxima.MakeAbsolute();
+  if(wxFileExists(maximaLocation = maxima.GetFullPath()))
+    return maximaLocation;
+
+  wxLogMessage(wxString::Format(notFound,maximaLocation));
+  maxima = exeDir + "maxima.bat";
+  maxima.MakeAbsolute();
+  if(wxFileExists(maximaLocation = maxima.GetFullPath()))
+    return maximaLocation;
+
+  wxLogMessage(wxString::Format(notFound,maximaLocation));
+  maxima = exeDir + "bin/maxima.bat";
+  maxima.MakeAbsolute();
+  if(wxFileExists(maximaLocation = maxima.GetFullPath()))
+    return maximaLocation;
+
+  wxLogMessage(wxString::Format(notFound,maximaLocation));
+  maxima = exeDir + "../maxima.bat";
+  maxima.MakeAbsolute();
+  if(wxFileExists(maximaLocation = maxima.GetFullPath()))
+    return maximaLocation;
+
+  wxLogMessage(wxString::Format(notFound,maximaLocation));
+  maxima = exeDir + "maxima";
+  maxima.MakeAbsolute();
+  if(wxFileExists(maximaLocation = maxima.GetFullPath()))
+    return maximaLocation;
+
+  wxLogMessage(wxString::Format(notFound,maximaLocation));
+  maxima = exeDir + "bin/maxima";
+  maxima.MakeAbsolute();
+  if(wxFileExists(maximaLocation = maxima.GetFullPath()))
+    return maximaLocation;
+
+  wxLogMessage(wxString::Format(notFound,maximaLocation));
+  maxima = exeDir + "../maxima";
+  maxima.MakeAbsolute();
+  if(wxFileExists(maximaLocation = maxima.GetFullPath()))
+    return maximaLocation;
+
+  wxLogMessage(wxString::Format(notFound,maximaLocation));
+  maxima = exeDir + "../bin/maxima";
+  maxima.MakeAbsolute();
+  if(wxFileExists(maximaLocation = maxima.GetFullPath()))
+    return maximaLocation;
+
+  wxLogMessage(wxString::Format(notFound,maximaLocation));
+#elif defined __WXOSX__
+  maximaLocation =  "/Applications/Maxima.app";
+  if (wxFileExists(maximaLocation))
+    return maximaLocation;
+  
+  wxLogMessage(wxString::Format(notFound,maximaLocation));
+  maximaLocation = "/Applications/maxima.app";
+  if (wxFileExists(maximaLocation))
+    return maximaLocation;
+  
+  wxLogMessage(wxString::Format(notFound,maximaLocation));
+  maximaLocation = "/usr/local/bin/maxima";
+  if (wxFileExists(maximaLocation))
+    return maximaLocation;
+  
+  wxLogMessage(wxString::Format(notFound,maximaLocation));
+  maximaLocation = "/usr/bin/maxima";
+  if (wxFileExists(maximaLocation))
+    return maximaLocation;
+  
+  wxLogMessage(wxString::Format(notFound,maximaLocation));
+  maximaLocation = "/usr/bin/maxima";
+  if (wxFileExists(maximaLocation))
+    return maximaLocation;
+  
+  wxLogMessage(wxString::Format(notFound,maximaLocation));
+  maxima = exeDir + "maxima";
+  maxima.MakeAbsolute();
+  if(wxFileExists(maximaLocation = maxima.GetFullPath()))
+    return maximaLocation;
+
+  wxLogMessage(wxString::Format(notFound,maximaLocation));
 #endif
+  return wxT("maxima");
 }
+
+Dirstructure *Dirstructure::m_dirStructure;
