@@ -66,12 +66,6 @@ private:
   #endif
   
   int m_errorIndex;
-  //! The viewport size the linewrap was done for.
-  int m_oldViewportWidth;
-  //! The zoom factor the linewrap was done for.
-  int m_oldZoomFactor;
-  //! The font size the linewrap was done for.
-  int m_oldDefaultFontSize;
 
   //! A list of all potential autoComplete targets within this cell
   wxArrayString m_wordList;
@@ -104,6 +98,9 @@ public:
 
   ~EditorCell();
 
+  //! Insert the symbol that corresponds to the ESC command txt
+  void InsertEscCommand(wxString txt){InsertText(InterpretEscapeString(txt));}
+
   //! Get the whole maxima command that is currently under the cursor (including all arguments)
   wxString GetFullCommandUnderCursor();
 
@@ -224,6 +221,8 @@ public:
 
   wxString ToString();
 
+  wxString ToMatlab();
+
   /*! Convert the current cell to a string
   
     \param dontLimitToSelection
@@ -232,6 +231,9 @@ public:
   */
   wxString ToString(bool dontLimitToSelection);
 
+  //! Convert the current cell to Matlab code
+  wxString ToMatlab(bool dontLimitToSelection);
+
   //! Convert the current cell to LaTeX code
   wxString ToTeX();
 
@@ -504,6 +506,9 @@ public:
   //! Convert the current selection to a string
   wxString GetSelectionString();
 
+  //! The word the cursor currently is at
+  wxString GetWordUnderCaret();
+
   //! Get the command the cursor is in the arguments for.
   wxString GetCurrentCommand();
   
@@ -563,7 +568,7 @@ private:
    */
   wxArrayString StringToTokens(wxString string);
 
-#if defined __WXMAC__
+#if defined __WXOSX__
 
   bool HandleCtrlCommand(wxKeyEvent &ev);
 
