@@ -42,8 +42,7 @@ EvaluationQueue::EvaluationQueue()
 
 void EvaluationQueue::Clear()
 {
-  while (!Empty())
-    RemoveFirst();
+  m_queue.clear();
   m_size = 0;
   m_commands.clear();
   m_workingGroupChanged = false;
@@ -305,7 +304,7 @@ wxString EvaluationQueue::GetCommand()
             i++;
           else
           {
-            if ((!wxIsalnum(userLabel[i])) && (userLabel[i] != '_'))
+            if ((!wxIsalnum(userLabel[i])) && (userLabel[i] != '_') && (userLabel[i] != '[') && (userLabel[i] != ']'))
             {
               userLabel = wxEmptyString;
               break;
