@@ -37,7 +37,7 @@ ExptCell::ExptCell(Cell *parent, Configuration **config, CellPointers *cellPoint
   m_last1 = NULL;
   m_last2 = NULL;
   m_baseCell = NULL;
-  m_powCell = NULL;
+  m_exptCell = NULL;
   m_isMatrix = false;
   m_exp = new TextCell(parent, config, cellPointers, wxT("^"));
   m_open = new TextCell(parent, config, cellPointers, wxT("("));
@@ -45,25 +45,29 @@ ExptCell::ExptCell(Cell *parent, Configuration **config, CellPointers *cellPoint
   m_close = new TextCell(parent, config, cellPointers, wxT(")"));
 }
 
-void ExptCell::SetGroup(Cell *parent)
+void ExptCell::Draw(wxPoint point)
 {
-  m_group = parent;
-  if (m_baseCell != NULL)
-    m_baseCell->SetGroupList(parent);
-  if (m_powCell != NULL)
-    m_powCell->SetGroupList(parent);
-  if (m_open != NULL)
-    m_open->SetGroupList(parent);
-  if (m_close != NULL)
-    m_close->SetGroupList(parent);
+  Cell::Draw(point);
+  if (DrawThisCell(point) && InUpdateRegion())
+  {    
+    wxPoint bs, pw;
+    bs.x = point.x;
+    bs.y = point.y;
+    m_baseCell->DrawList(bs);
+
+    point.x += m_baseCell->GetFullWidth() - MC_TEXT_PADDING;
+    point.y -= PowRise() + wxMax(m_exptCell->GetMaxCenter()- m_baseCell->GetMaxCenter(),0);
+    m_exptCell->DrawList(point);
+  }
 }
 
+
 Cell *ExptCell::Copy()
 {
   ExptCell *tmp = new ExptCell(m_group, m_configuration, m_cellPointers);
   CopyData(this, tmp);
   tmp->SetBase(m_baseCell->CopyList());
-  tmp->SetPower(m_powCell->CopyList());
+  tmp->SetPower(m_exptCell->CopyList());
   tmp->m_isBrokenIntoLines = m_isBrokenIntoLines;
   tmp->m_open->DontEscapeOpeningParenthesis();
 
@@ -73,11 +77,11 @@ Cell *ExptCell::Copy()
 ExptCell::~ExptCell()
 {
   wxDELETE(m_baseCell);
-  wxDELETE(m_powCell);
+  wxDELETE(m_exptCell);
   wxDELETE(m_exp);
   wxDELETE(m_open);
   wxDELETE(m_close);
-  m_baseCell = m_powCell = m_exp = m_open = m_close = NULL;
+  m_baseCell = m_exptCell = m_exp = m_open = m_close = NULL;
   MarkAsDeleted();
 }
 
@@ -86,8 +90,8 @@ std::list<Cell *> ExptCell::GetInnerCells()
   std::list<Cell *> innerCells;
   if(m_baseCell)
     innerCells.push_back(m_baseCell);
-  if(m_powCell)
-    innerCells.push_back(m_powCell);
+  if(m_exptCell)
+    innerCells.push_back(m_exptCell);
   if(m_exp)
     innerCells.push_back(m_exp);
   if(m_open)
@@ -102,10 +106,10 @@ void ExptCell::SetPower(Cell *power)
 {
   if (power == NULL)
     return;
-  wxDELETE(m_powCell);
-  m_powCell = power;
+  wxDELETE(m_exptCell);
+  m_exptCell = power;
 
-  if (!m_powCell->IsCompound())
+  if (!m_exptCell->IsCompound())
   {
     m_open->m_isHidden = true;
     m_close->m_isHidden = true;
@@ -132,20 +136,19 @@ void ExptCell::SetBase(Cell *base)
 
 void ExptCell::RecalculateWidths(int fontsize)
 {
-  Cell::RecalculateWidths(fontsize);
   m_baseCell->RecalculateWidthsList(fontsize);
   if (m_isBrokenIntoLines)
-    m_powCell->RecalculateWidthsList(fontsize);
+    m_exptCell->RecalculateWidthsList(fontsize);
   else
-    m_powCell->RecalculateWidthsList(MAX(MC_MIN_SIZE, fontsize - EXPT_DEC));
-  m_width = m_baseCell->GetFullWidth() + m_powCell->GetFullWidth() -
+    m_exptCell->RecalculateWidthsList(wxMax(MC_MIN_SIZE, fontsize - EXPT_DEC));
+  m_width = m_baseCell->GetFullWidth() + m_exptCell->GetFullWidth() -
             MC_TEXT_PADDING;
   m_exp->RecalculateWidthsList(fontsize);
   m_open->RecalculateWidthsList(fontsize);
   m_close->RecalculateWidthsList(fontsize);
-  ResetData();
   if(m_isBrokenIntoLines)
     m_width = 0;
+  Cell::RecalculateWidths(fontsize);
 }
 
 void ExptCell::RecalculateHeight(int fontsize)
@@ -153,39 +156,23 @@ void ExptCell::RecalculateHeight(int fontsize)
   Cell::RecalculateHeight(fontsize);
   m_baseCell->RecalculateHeightList(fontsize);
   if (m_isBrokenIntoLines)
-    m_powCell->RecalculateHeightList(fontsize);
+    m_exptCell->RecalculateHeightList(fontsize);
   else
-    m_powCell->RecalculateHeightList(MAX(MC_MIN_SIZE, fontsize - EXPT_DEC));
-  m_height = m_baseCell->GetMaxHeight() + m_powCell->GetMaxHeight() -
-             Scale_Px((8 * fontsize) / 10 + MC_EXP_INDENT);
-  m_center = m_powCell->GetMaxHeight() + m_baseCell->GetMaxCenter() -
-             Scale_Px((8 * fontsize) / 10 + MC_EXP_INDENT);
+    m_exptCell->RecalculateHeightList(wxMax(MC_MIN_SIZE, fontsize - EXPT_DEC));
+
+  m_center = wxMax (
+    m_baseCell->GetMaxCenter(),
+    PowRise() + m_exptCell->GetMaxHeight() - m_baseCell->GetMaxCenter());
+  
+  m_height = m_baseCell->GetMaxHeight() +
+    wxMax(0,m_center - m_baseCell->GetMaxCenter());
   m_exp->RecalculateHeightList(fontsize);
   m_open->RecalculateHeightList(fontsize);
   m_close->RecalculateHeightList(fontsize);
   if (m_isBrokenIntoLines)
   {
-    m_height = MAX(m_baseCell->GetMaxHeight(), m_open->GetMaxHeight());
-    m_center = MAX(m_baseCell->GetMaxCenter(), m_open->GetMaxCenter());
-  }
-}
-
-void ExptCell::Draw(wxPoint point)
-{
-  if (DrawThisCell(point) && InUpdateRegion())
-  {
-    
-    Cell::Draw(point);
-    wxPoint bs, pw;
-    bs.x = point.x;
-    bs.y = point.y;
-    m_baseCell->DrawList(bs);
-
-    pw.x = point.x + m_baseCell->GetFullWidth() - MC_TEXT_PADDING;
-    pw.y = point.y - m_baseCell->GetMaxCenter() - m_powCell->GetMaxHeight()
-           + m_powCell->GetMaxCenter() +
-           Scale_Px((8 * m_fontSize) / 10 + MC_EXP_INDENT);
-    m_powCell->DrawList(pw);
+    m_height = wxMax(m_baseCell->GetMaxHeight(), m_open->GetMaxHeight());
+    m_center = wxMax(m_baseCell->GetMaxCenter(), m_open->GetMaxCenter());
   }
 }
 
@@ -198,10 +185,26 @@ wxString ExptCell::ToString()
   wxString s = m_baseCell->ListToString() + wxT("^");
   if (m_isMatrix)
     s += wxT("^");
-  if (m_powCell->IsCompound())
-    s += wxT("(") + m_powCell->ListToString() + wxT(")");
+  if (m_exptCell->IsCompound())
+    s += wxT("(") + m_exptCell->ListToString() + wxT(")");
   else
-    s += m_powCell->ListToString();
+    s += m_exptCell->ListToString();
+  return s;
+}
+
+wxString ExptCell::ToMatlab()
+{
+  if (m_altCopyText != wxEmptyString)
+	return m_altCopyText;
+  if (m_isBrokenIntoLines)
+	return wxEmptyString;
+  wxString s = m_baseCell->ListToMatlab() + wxT("^");
+  if (m_isMatrix)
+	s += wxT("^");
+  if (m_exptCell->IsCompound())
+	s += wxT("(") + m_exptCell->ListToMatlab() + wxT(")");
+  else
+	s += m_exptCell->ListToMatlab();
   return s;
 }
 
@@ -210,7 +213,7 @@ wxString ExptCell::ToTeX()
   if (m_isBrokenIntoLines)
     return wxEmptyString;
   wxString s = wxT("{{") + m_baseCell->ListToTeX() + wxT("}^{") +
-               m_powCell->ListToTeX() + wxT("}}");
+               m_exptCell->ListToTeX() + wxT("}}");
   return s;
 }
 
@@ -220,8 +223,8 @@ wxString ExptCell::GetDiffPart()
   if (m_baseCell != NULL)
     s += m_baseCell->ListToString();
   s += wxT(",");
-  if (m_powCell != NULL)
-    s += m_powCell->ListToString();
+  if (m_exptCell != NULL)
+    s += m_exptCell->ListToString();
   return s;
 }
 
@@ -229,15 +232,15 @@ wxString ExptCell::ToMathML()
 {
   return wxT("<msup>") +
          m_baseCell->ListToMathML() +
-         m_powCell->ListToMathML() +
+         m_exptCell->ListToMathML() +
          wxT("</msup>\n");
-//  return wxT("<apply><power/>") + m_baseCell->ListToMathML() + m_powCell->ListToMathML() + wxT("</apply>");
+//  return wxT("<apply><power/>") + m_baseCell->ListToMathML() + m_exptCell->ListToMathML() + wxT("</apply>");
 }
 
 wxString ExptCell::ToOMML()
 {
   return wxT("<m:sSup><m:e>") + m_baseCell->ListToOMML() + wxT("</m:e><m:sup>") +
-         m_powCell->ListToOMML() + wxT("</m:sup></m:sSup>\n");
+         m_exptCell->ListToOMML() + wxT("</m:sup></m:sSup>\n");
 }
 
 wxString ExptCell::ToXML()
@@ -249,7 +252,7 @@ wxString ExptCell::ToXML()
     flags += wxT(" breakline=\"true\"");
 
   return wxT("<e") + flags + wxT("><r>") + m_baseCell->ListToXML() + _T("</r><r>") +
-         m_powCell->ListToXML() + _T("</r></e>");
+         m_exptCell->ListToXML() + _T("</r></e>");
 }
 
 bool ExptCell::BreakUp()
@@ -266,8 +269,8 @@ bool ExptCell::BreakUp()
     }
     m_exp->m_nextToDraw = m_open;
     m_open->m_previousToDraw = m_exp;
-    m_open->m_nextToDraw = m_powCell;
-    m_powCell->m_previousToDraw = m_open;
+    m_open->m_nextToDraw = m_exptCell;
+    m_exptCell->m_previousToDraw = m_open;
     wxASSERT_MSG(m_last2 != NULL, _("Bug: No last cell in an exponent of an exptCell!"));
     if (m_last2 != NULL)
     {
@@ -278,8 +281,9 @@ bool ExptCell::BreakUp()
     if (m_nextToDraw != NULL)
       m_nextToDraw->m_previousToDraw = m_close;
     m_nextToDraw = m_baseCell;
-    m_height = MAX(m_baseCell->GetMaxHeight(), m_open->GetMaxHeight());
-    m_center = MAX(m_baseCell->GetMaxCenter(), m_open->GetMaxCenter());
+    ResetData();    
+    m_height = wxMax(m_baseCell->GetMaxHeight(), m_open->GetMaxHeight());
+    m_center = wxMax(m_baseCell->GetMaxCenter(), m_open->GetMaxCenter());
     return true;
   }
   return false;
@@ -290,7 +294,7 @@ void ExptCell::Unbreak()
   if (m_isBrokenIntoLines)
   {
     m_baseCell->UnbreakList();
-    m_powCell->UnbreakList();
+    m_exptCell->UnbreakList();
   }
   Cell::Unbreak();
 }
