@@ -50,26 +50,6 @@ FracCell::FracCell(Cell *parent, Configuration **config, CellPointers *cellPoint
   m_open2 = NULL;
   m_close2 = NULL;
   m_divide = NULL;
-  m_divSign = new TextCell(this, config, cellPointers, wxT("/"));
-}
-
-void FracCell::SetGroup(Cell *parent)
-{
-  m_group = parent;
-  if (m_num != NULL)
-    m_num->SetGroupList(parent);
-  if (m_denom != NULL)
-    m_denom->SetGroupList(parent);
-  if (m_open1 != NULL)
-    m_open1->SetGroupList(parent);
-  if (m_close1 != NULL)
-    m_close2->SetGroupList(parent);
-  if (m_open2 != NULL)
-    m_open2->SetGroupList(parent);
-  if (m_close2 != NULL)
-    m_close2->SetGroupList(parent);
-  if (m_divide != NULL)
-    m_divide->SetGroupList(parent);
 }
 
 Cell *FracCell::Copy()
@@ -137,7 +117,12 @@ void FracCell::SetDenom(Cell *denom)
 
 void FracCell::RecalculateWidths(int fontsize)
 {
-  Cell::RecalculateWidths(fontsize);
+  m_open1->RecalculateWidths(fontsize);
+  m_close1->RecalculateWidths(fontsize);
+  m_open2->RecalculateWidths(fontsize);
+  m_close2->RecalculateWidths(fontsize);
+  m_divide->RecalculateWidths(fontsize);
+  
   wxASSERT(fontsize >= 1);
   Configuration *configuration = (*m_configuration);
   if (m_isBrokenIntoLines || m_exponent)
@@ -147,8 +132,8 @@ void FracCell::RecalculateWidths(int fontsize)
   }
   else
   {
-    m_num->RecalculateWidthsList(MAX(MC_MIN_SIZE, fontsize - FRAC_DEC));
-    m_denom->RecalculateWidthsList(MAX(MC_MIN_SIZE, fontsize - FRAC_DEC));
+    m_num->RecalculateWidthsList(wxMax(MC_MIN_SIZE, fontsize - FRAC_DEC));
+    m_denom->RecalculateWidthsList(wxMax(MC_MIN_SIZE, fontsize - FRAC_DEC));
   }
   wxDC *dc = configuration->GetDC();
   dc->SetFont(configuration->GetFont(TS_VARIABLE,fontsize));
@@ -179,22 +164,22 @@ void FracCell::RecalculateWidths(int fontsize)
     else
       m_horizontalGapRight = 0;
     
-    m_width = MAX(m_num->GetFullWidth(), m_denom->GetFullWidth()) +
+    m_width = wxMax(m_num->GetFullWidth(), m_denom->GetFullWidth()) +
               2 * m_protrusion + m_horizontalGapLeft + m_horizontalGapRight;
   }
-  m_open1->RecalculateWidths(fontsize);
-  m_close1->RecalculateWidths(fontsize);
-  m_open2->RecalculateWidths(fontsize);
-  m_close2->RecalculateWidths(fontsize);
-  m_divide->RecalculateWidths(fontsize);
-  ResetData();
   if(m_isBrokenIntoLines)
     m_width = 0;
+  Cell::RecalculateWidths(fontsize);
 }
 
 void FracCell::RecalculateHeight(int fontsize)
 {
   Cell::RecalculateHeight(fontsize);
+  m_open1->RecalculateHeight(fontsize);
+  m_close1->RecalculateHeight(fontsize);
+  m_open2->RecalculateHeight(fontsize);
+  m_close2->RecalculateHeight(fontsize);
+  m_divide->RecalculateHeight(fontsize);
   if (m_isBrokenIntoLines || m_exponent)
   {
     m_num->RecalculateHeightList(fontsize);
@@ -202,8 +187,8 @@ void FracCell::RecalculateHeight(int fontsize)
   }
   else
   {
-    m_num->RecalculateHeightList(MAX(MC_MIN_SIZE, fontsize - FRAC_DEC));
-    m_denom->RecalculateHeightList(MAX(MC_MIN_SIZE, fontsize - FRAC_DEC));
+    m_num->RecalculateHeightList(wxMax(MC_MIN_SIZE, fontsize - FRAC_DEC));
+    m_denom->RecalculateHeightList(wxMax(MC_MIN_SIZE, fontsize - FRAC_DEC));
   }
   if(m_isBrokenIntoLines)
   {
@@ -215,10 +200,7 @@ void FracCell::RecalculateHeight(int fontsize)
     if (!m_exponent)
     {
       m_height = m_num->GetMaxHeight() + m_denom->GetMaxHeight() +
-        Scale_Px(4.5); // TODO: Theoretically this should be a 4. But
-                       // the example a:1/b$for i:1 thru 15 do a:subst(b=1/b+1,a)$a;kill(a)$
-                       // shows that 4 isn't sufficient => a rounding error? Or what did I get
-                       // completely wrong?
+        Scale_Px(4);
       m_center = m_num->GetMaxHeight() + Scale_Px(2);
     }
     else
@@ -227,25 +209,19 @@ void FracCell::RecalculateHeight(int fontsize)
       m_center = m_height / 2;
     }
   }
-
-  m_open1->RecalculateHeight(fontsize);
-  m_close1->RecalculateHeight(fontsize);
-  m_open2->RecalculateHeight(fontsize);
-  m_close2->RecalculateHeight(fontsize);
-  m_divide->RecalculateHeight(fontsize);
 }
 
 void FracCell::Draw(wxPoint point)
 {
+  Cell::Draw(point);
   if (DrawThisCell(point) && InUpdateRegion())
   {
-    Cell::Draw(point);
     Configuration *configuration = (*m_configuration);
     
     wxDC *dc = configuration->GetDC();
     wxPoint num, denom;
 
-    if (m_exponent && !m_isBrokenIntoLines)
+    if (m_exponent)
     {
       num.x = point.x;
       num.y = point.y;
@@ -331,6 +307,49 @@ wxString FracCell::ToString()
   return s;
 }
 
+wxString FracCell::ToMatlab()
+{
+  wxString s;
+  if (!m_isBrokenIntoLines)
+  {
+	if (m_fracStyle == FC_NORMAL)
+	{
+	  if (m_num->IsCompound())
+		s += wxT("(") + m_num->ListToMatlab() + wxT(")/");
+	  else
+		s += m_num->ListToMatlab() + wxT("/");
+	  if (m_denom->IsCompound())
+		s += wxT("(") + m_denom->ListToMatlab() + wxT(")");
+	  else
+		s += m_denom->ListToMatlab();
+	}
+	else if (m_fracStyle == FC_CHOOSE)
+	{
+	  s = wxT("binomial(") + m_num->ListToMatlab() + wxT(",") +
+		  m_denom->ListToMatlab() + wxT(")");
+	}
+	else
+	{
+	  Cell *tmp = m_denom;
+	  while (tmp != NULL)
+	  {
+		tmp = tmp->m_next;   // Skip the d
+		if (tmp == NULL)
+		  break;
+		tmp = tmp->m_next;   // Skip the *
+		if (tmp == NULL)
+		  break;
+		s += tmp->GetDiffPart();
+		tmp = tmp->m_next;   // Skip the *
+		if (tmp == NULL)
+		  break;
+		tmp = tmp->m_next;
+	  }
+	}
+  }
+  return s;
+}
+
 wxString FracCell::ToTeX()
 {
   wxString s;
@@ -471,6 +490,7 @@ bool FracCell::BreakUp()
     if (m_nextToDraw != NULL)
       m_nextToDraw->m_previousToDraw = m_close2;
     m_nextToDraw = m_open1;
+    ResetData();    
     return true;
   }
   return false;
