@@ -45,6 +45,7 @@ GroupCell::GroupCell(Configuration **config, GroupType groupType, CellPointers *
 {
   m_next = m_previous = m_nextToDraw = m_previousToDraw = NULL;
   m_autoAnswer = false;
+  m_cellsInGroup = 1;
   m_cellPointers = cellPointers;
   m_inEvaluationQueue = false;
   m_lastInEvaluationQueue = false;
@@ -244,7 +245,7 @@ void GroupCell::SetCellStyle(int style)
 
 */
 void GroupCell::SetGroup(Cell *parent)
-{
+{  
   //m_group = parent;
   if (m_inputLabel != NULL)
     m_inputLabel->SetGroupList(parent);
@@ -514,7 +515,10 @@ void GroupCell::SetOutput(Cell *output)
     m_output->SetGroup(this);
     while (m_lastInOutput->m_next != NULL)
       m_lastInOutput = m_lastInOutput->m_next;
+    m_output->ResetSizeList();
   }
+  UpdateCellsInGroup();
+  Recalculate();
 }
 
 void GroupCell::RemoveOutput()
@@ -544,6 +548,7 @@ void GroupCell::RemoveOutput()
   GroupCell *cell = dynamic_cast<GroupCell *>(this->m_next);
   while(cell != NULL)
     cell = cell->UpdateYPosition();
+  UpdateCellsInGroup();
 }
 
 void GroupCell::AppendOutput(Cell *cell)
@@ -582,12 +587,14 @@ void GroupCell::AppendOutput(Cell *cell)
 
   if (m_appendedCells == NULL)
     m_appendedCells = cell;
+
+  UpdateCellsInGroup();
 }
 
 void GroupCell::Recalculate()
 {
   int fontsize = (*m_configuration)->GetDefaultFontSize();
-  
+
   m_fontSize = fontsize;
   m_mathFontSize = (*m_configuration)->GetMathFontSize();
 
@@ -611,8 +618,6 @@ void GroupCell::RecalculateWidths(int fontsize)
       return;
     }
     
-    UnBreakUpCells();
-
     if(m_inputLabel != NULL)
     {
       m_inputLabel->RecalculateWidthsList(fontsize);
@@ -643,8 +648,6 @@ void GroupCell::RecalculateWidths(int fontsize)
         }
       }
     }
-
-    BreakUpCells();
     BreakLines();
   }
   ResetData();
@@ -663,7 +666,12 @@ void GroupCell::InputHeightChanged()
     m_inputLabel->ResetData();
   }
   RecalculateHeightInput();
-  RecalculateHeightOutput();
+
+  if(m_output != NULL)
+  {
+    m_height += m_outputRect.height;
+    m_outputRect.y = m_currentPoint.y + m_center;
+  }
 }
 
 // Called on resize events
@@ -676,11 +684,10 @@ void GroupCell::OnSize()
   while (tmp != NULL)
   {
     tmp->Unbreak();
-    tmp->BreakLine(false);
+    tmp->SoftLineBreak(false);
     tmp->ResetData();
     tmp = tmp->m_next;
   }
-  BreakUpCells();
   BreakLines();
   InputHeightChanged();
 }
@@ -688,6 +695,16 @@ void GroupCell::OnSize()
 void GroupCell::RecalculateHeightInput()
 {
   Configuration *configuration = (*m_configuration);
+
+  if (m_inputLabel)
+    m_inputLabel->SetCurrentPoint(m_currentPoint);
+  if (GetEditable())
+  {
+    wxPoint in = GetCurrentPoint();
+    
+    in.x += GetInputIndent();
+    GetEditable()->SetCurrentPoint(GetCurrentPoint());
+  }
   
   // special case
   if (m_groupType == GC_TYPE_PAGEBREAK)
@@ -731,12 +748,15 @@ void GroupCell::RecalculateHeightInput()
     m_currentPoint.y = (*m_configuration)->GetBaseIndent() + GetMaxCenter();
   }
   else
-    m_currentPoint.y = dynamic_cast<GroupCell *>(m_previous)->m_currentPoint.y +
-      dynamic_cast<GroupCell *>(m_previous)->GetMaxDrop() + GetMaxCenter() +
-      (*m_configuration)->GetGroupSkip();
+  {
+    if(dynamic_cast<GroupCell *>(m_previous)->m_currentPoint.y > 0)
+      m_currentPoint.y = dynamic_cast<GroupCell *>(m_previous)->m_currentPoint.y +
+        dynamic_cast<GroupCell *>(m_previous)->GetMaxDrop() + GetMaxCenter() +
+        (*m_configuration)->GetGroupSkip();
+  }
   
   m_outputRect.x = m_currentPoint.x;
-  m_outputRect.y = m_currentPoint.y;
+  m_outputRect.y = m_currentPoint.y + m_center;
   if (m_output) m_outputRect.y -= m_output->GetMaxCenter();
   else
   {
@@ -771,22 +791,20 @@ void GroupCell::RecalculateHeightOutput()
 
 bool GroupCell::NeedsRecalculation()
 {
-  bool result = ((m_width < 0) || (m_height < 0) ||
-                 (m_currentPoint.x < 0) || (m_currentPoint.y < 0) ||
-                 ((*m_configuration)->RecalculationForce() ||
-                  ((GetInput() != NULL) &&
-                   ((GetInput()->GetWidth() < 0) || (GetInput()->GetHeight() < 0))
-                    )));
-  return result;
+  return Cell::NeedsRecalculation() ||
+    ((GetInput() != NULL) &&
+     ((GetInput()->GetWidth() <= 0) || (GetInput()->GetHeight() <= 0) ||
+      (GetInput()->GetCurrentPoint().x <= 0) || (GetInput()->GetCurrentPoint().y <= 0)
+       ));
 }
 
 void GroupCell::RecalculateHeight(int fontsize)
 {
   Cell::RecalculateHeight(fontsize);
-  Configuration *configuration = (*m_configuration);
 
   if(NeedsRecalculation())
   {
+    m_outputRect.SetHeight(0);
     RecalculateHeightInput();   
     RecalculateHeightOutput();
   }
@@ -794,51 +812,39 @@ void GroupCell::RecalculateHeight(int fontsize)
 //  if (((m_height <= 0) || (m_next == NULL)) && (m_height < configuration->GetCellBracketWidth()))
 //    m_height = configuration->GetCellBracketWidth();
   
-  configuration= (*m_configuration);
-  if (m_previous == NULL)
-  {
-    m_currentPoint.x = configuration->GetIndent();
-    m_currentPoint.y = (*m_configuration)->GetBaseIndent() + GetMaxCenter();
-  }
-  else
-  {
-    m_currentPoint.x = configuration->GetIndent();
-    if(dynamic_cast<GroupCell *>(m_previous)->m_height > 0)
-      m_currentPoint.y = dynamic_cast<GroupCell *>(m_previous)->m_currentPoint.y +
-        dynamic_cast<GroupCell *>(m_previous)->GetMaxDrop() + GetMaxCenter() +
-        (*m_configuration)->GetGroupSkip();
-    else
-      m_currentPoint.y = dynamic_cast<GroupCell *>(m_previous)->m_currentPoint.y;
-  }
-
+  UpdateYPosition();
   // If code is hidden and there is no output a cell can have the height
   // 0. If it is higher than that we make our cell high enough to fit the 
   // bracket in.  m_appendedCells = NULL;
-
-  if (m_inputLabel)
-    m_inputLabel->SetCurrentPoint(m_currentPoint);
-  if (GetEditable())
-  {
-    wxPoint in = GetCurrentPoint();
-    
-    in.x += GetInputIndent();
-    GetEditable()->SetCurrentPoint(GetCurrentPoint());
-  }
 }
 
 // We assume that appended cells will be in a new line!
 void GroupCell::RecalculateAppended()
-{
+{  
   if(m_hide)
     return;
+  if(NeedsRecalculation())
+    m_appendedCells = m_output;
   Configuration *configuration = (*m_configuration);
+    
+  if(m_height < 0)
+  {
+    int fontsize = configuration->GetDefaultFontSize();
+      
+    m_fontSize = fontsize;
+    m_mathFontSize = (*m_configuration)->GetMathFontSize();
+    
+    RecalculateWidths(fontsize);
+    RecalculateHeightInput();
+    m_appendedCells = m_output;
+  }
   if (m_appendedCells == NULL)
     m_appendedCells = m_inputLabel;
   if (m_appendedCells == NULL)
-    m_appendedCells = GetInput();
+    m_appendedCells = GetOutput();
   if (m_appendedCells == NULL)
     return;
-  m_appendedCells->ForceBreakLineHere();
+  m_appendedCells->HardLineBreak();
 
   Cell *tmp = m_appendedCells;
   m_fontSize = configuration->GetFontSize(TS_TEXT);
@@ -852,7 +858,6 @@ void GroupCell::RecalculateAppended()
   }
 
   // Breakup cells and break lines
-  BreakUpCells(m_appendedCells);
   BreakLines(m_appendedCells);
 
   // Recalculate size of cells
@@ -872,7 +877,7 @@ void GroupCell::RecalculateAppended()
     if (tmp->BreakLineHere())
     {
       int height_Delta = tmp->GetMaxHeight();
-      m_width = MAX(m_width, tmp->GetLineWidth());
+      m_width = wxMax(m_width, tmp->GetLineWidth());
       m_height            += height_Delta;
       m_outputRect.width = m_width;
       m_outputRect.height += height_Delta;
@@ -892,13 +897,12 @@ void GroupCell::RecalculateAppended()
     }
     tmp = tmp->m_nextToDraw;
   }
-
   m_appendedCells = NULL;
 
   ResetData();
   
   // Move all cells that follow the current one down by the amount this cell has grown.
-  GroupCell *cell = dynamic_cast<GroupCell *>(this->m_next);
+  GroupCell *cell = this;
   while(cell != NULL)
     cell = cell->UpdateYPosition();
   (*m_configuration)->AdjustWorksheetSize(true);
@@ -906,13 +910,22 @@ void GroupCell::RecalculateAppended()
 
 GroupCell *GroupCell::UpdateYPosition()
 {
-  if (m_previous != NULL)
+  Configuration *configuration = (*m_configuration);
+  
+  if (m_previous == NULL)
   {
-    m_currentPoint.x = (*m_configuration)->GetIndent();
-    m_currentPoint = m_previous->GetCurrentPoint();
-    m_currentPoint.y += m_previous->GetMaxHeight();
-    if(m_previous->GetMaxDrop() > 0)
-      m_currentPoint.y += (*m_configuration)->GetGroupSkip();
+    m_currentPoint.x = configuration->GetIndent();
+    m_currentPoint.y = configuration->GetBaseIndent() + GetMaxCenter();
+  }
+  else
+  {
+    m_currentPoint.x = configuration->GetIndent();
+    if(dynamic_cast<GroupCell *>(m_previous)->m_height > 0)
+      m_currentPoint.y = dynamic_cast<GroupCell *>(m_previous)->m_currentPoint.y +
+        dynamic_cast<GroupCell *>(m_previous)->GetMaxDrop() + GetMaxCenter() +
+        configuration->GetGroupSkip();
+    else
+      m_currentPoint.y = dynamic_cast<GroupCell *>(m_previous)->m_currentPoint.y;
   }
   return dynamic_cast<GroupCell *>(m_next);
 }
@@ -926,9 +939,9 @@ int GroupCell::GetInputIndent()
   if(m_inputLabel != NULL)
   {
     if(m_inputLabel->GetWidth() >= 0)
-      labelWidth = MAX(m_inputLabel->GetWidth() + MC_TEXT_PADDING,labelWidth);
+      labelWidth = wxMax(m_inputLabel->GetWidth() + MC_TEXT_PADDING,labelWidth);
     else
-      labelWidth = MAX(m_labelWidth_cached,labelWidth);
+      labelWidth = wxMax(m_labelWidth_cached,labelWidth);
   }
   
   m_labelWidth_cached = labelWidth;
@@ -938,20 +951,15 @@ int GroupCell::GetInputIndent()
 
 void GroupCell::Draw(wxPoint point)
 {
+  if ((m_width < 1 || m_height < 1) ||
+      ((GetEditable() != NULL) &&
+     (
+       ((GetEditable()->GetHeight() < 1) ||
+        (GetEditable()->GetWidth() < 1)
+         ))))
+    Recalculate();
   Cell::Draw(point);
 
-   // std::cerr<<"GroupCell("<<
-   //   GetRect().GetLeft()<<","<<
-   //   GetRect().GetTop()<<","<<
-   //   GetRect().GetRight()<<","<<
-   //   GetRect().GetBottom()<<"),"<<
-   //   "), UpdateRegion=("<<
-   //   (*m_configuration)->GetUpdateRegion().GetLeft()<<","<<
-   //   (*m_configuration)->GetUpdateRegion().GetTop()<<","<<
-   //   (*m_configuration)->GetUpdateRegion().GetRight()<<","<<
-   //   (*m_configuration)->GetUpdateRegion().GetBottom()<<"),"<<
-   //   DrawThisCell(point)<<!(*m_configuration)->GetUpdateRegion().Intersect(GetRect()).IsEmpty()<<"\n";
-
   Configuration *configuration = (*m_configuration);
 
   if (configuration->ShowBrackets())
@@ -959,7 +967,7 @@ void GroupCell::Draw(wxPoint point)
 
   if (DrawThisCell(point))
   {
-    if (m_width == -1 || m_height == -1)
+    if (m_width < 0|| m_height < 0)
       return;
     
     wxDC *dc = configuration->GetDC();
@@ -1051,6 +1059,12 @@ void GroupCell::Draw(wxPoint point)
   }
 }
 
+wxRect GroupCell::GetRect(bool WXUNUSED(all))
+{
+  return wxRect(m_currentPoint.x, m_currentPoint.y - m_center,
+                m_width, m_height);
+}
+
 int GroupCell::GetLineIndent(Cell *cell)
 {
   int indent = 0;
@@ -1108,49 +1122,38 @@ void GroupCell::DrawBracket()
   if ((m_currentPoint.y >= selectionStart_px) &&
       (m_currentPoint.y <= selectionEnd_px))
   {
-#if defined(__WXMAC__)
-    dc->SetPen(wxNullPen); // wxmac doesn't like a border with wxXOR
-#else
     dc->SetPen(*(wxThePenList->FindOrCreatePen(
                   configuration->GetColor(TS_SELECTION),
                   configuration->GetDefaultLineWidth(),
                   wxPENSTYLE_SOLID)
                 ));
 // window linux, set a pen
-#endif
     dc->SetBrush(*(wxTheBrushList->FindOrCreateBrush(configuration->GetColor(TS_SELECTION))));
     drawBracket = true;
   }
   else if (m_cellPointers->m_errorList.Contains(this))
   {
-#if defined(__WXMAC__)
-    dc->SetPen(wxNullPen); // wxmac doesn't like a border with wxXOR
-#else
     dc->SetPen(*wxRED_PEN);
-// window linux, set a pen
-#endif
     dc->SetBrush(*wxRED_BRUSH);
     drawBracket = true;
   }
-  else if ((m_cellPointers->m_answerCell) && (m_cellPointers->m_answerCell->GetGroup() == this))
-  {
-#if defined(__WXMAC__)
-    dc->SetPen(wxNullPen); // wxmac doesn't like a border with wxXOR
-#else
-    dc->SetPen(*wxYELLOW_PEN);
-// window linux, set a pen
-#endif
-    dc->SetBrush(*wxYELLOW_BRUSH);
-    drawBracket = true;
-  }
   else
   {
-    dc->SetBrush(*wxWHITE_BRUSH);
-    dc->SetPen(*(wxThePenList->FindOrCreatePen(
-            *wxWHITE,
-            configuration->GetDefaultLineWidth(),
-            wxPENSTYLE_SOLID)
-    ));
+    if ((m_cellPointers->m_answerCell) && (m_cellPointers->m_answerCell->GetGroup() == this))
+    {
+      dc->SetPen(*wxYELLOW_PEN);
+      dc->SetBrush(*wxYELLOW_BRUSH);
+      drawBracket = true;
+    }
+    else
+    {
+      dc->SetBrush((*m_configuration)->GetBackgroundBrush());
+      dc->SetPen(*(wxThePenList->FindOrCreatePen(
+                     configuration->DefaultBackgroundColor(),
+                     configuration->GetDefaultLineWidth(),
+                     wxPENSTYLE_SOLID)
+                   ));
+    }
   }
   wxRect rect = GetRect();
   rect = wxRect(
@@ -1354,7 +1357,7 @@ wxString GroupCell::ToString()
     bool firstCell = true;
     while (tmp != NULL)
     {
-      if (firstCell || (tmp->ForceBreakLineHere() && str.Length() > 0))
+      if (firstCell || (tmp->HardLineBreak() && str.Length() > 0))
       {
         if (firstCell)
           str += wxT("\n");
@@ -1446,11 +1449,10 @@ wxString GroupCell::ToTeX(wxString imgDir, wxString filename, int *imgCounter)
         if (dynamic_cast<ImgCell *>(copy)->ToImageFile(file).x >= 0)
         {
           str << wxT("\\begin{figure}[htb]\n")
-              << wxT("  \\begin{center}\n")
+              << wxT("  \\centering\n")
               << wxT("    \\includeimage{")
               << filename << wxT("_img/") << image << wxT("}\n")
-              << wxT("  \\caption{") << m_inputLabel->m_next->ToTeX() << wxT("}\n")
-              << wxT("  \\end{center}\n")
+              << wxT("  \\caption{") << m_inputLabel->m_next->ToTeX().Trim() << wxT("}\n")
               << wxT("\\end{figure}\n");
         }
       }
@@ -1470,24 +1472,29 @@ wxString GroupCell::ToTeX(wxString imgDir, wxString filename, int *imgCounter)
         switch (GetEditable()->GetStyle())
         {
           case TS_TITLE:
-            str = wxT("\n\\pagebreak{}\n{\\Huge {\\sc ") + str + wxT("}}\n");
+            str = wxT("\n\\pagebreak{}\n{\\Huge {\\scshape ") + str + wxT("}}\n");
             str += wxT("\\setcounter{section}{0}\n\\setcounter{subsection}{0}\n");
             str += wxT("\\setcounter{figure}{0}\n");
             break;
           case TS_SECTION:
-            str = wxT("\n\\section{") + str + wxT("}\n");
+            // Trim() strings for TeX export to remove newlines in \section{}, \subsection{}, ... commands
+	    // LaTeX creates an error on the following code:
+	    // \section{Chapter 1
+	    //
+	    // }
+            str = wxT("\n\\section{") + str.Trim() + wxT("}\n");
             break;
           case TS_SUBSECTION:
-            str = wxT("\n\\subsection{") + str + wxT("}\n");
+            str = wxT("\n\\subsection{") + str.Trim() + wxT("}\n");
             break;
           case TS_SUBSUBSECTION:
-            str = wxT("\n\\subsubsection{") + str + wxT("}\n");
+            str = wxT("\n\\subsubsection{") + str.Trim() + wxT("}\n");
             break;
           case TS_HEADING5:
-            str = wxT("\n\\paragraph{") + str + wxT("}\n");
+            str = wxT("\n\\paragraph{") + str.Trim() + wxT("}\n");
             break;
           case TS_HEADING6:
-            str = wxT("\n\\subparagraph{") + str + wxT("}\n");
+            str = wxT("\n\\subparagraph{") + str.Trim() + wxT("}\n");
             break;
           default:
             if (str.StartsWith(wxT("TeX:")))
@@ -1515,7 +1522,7 @@ wxString GroupCell::ToTeXCodeCell(wxString imgDir, wxString filename, int *imgCo
     str = wxT("\n\n\\noindent\n%%%%%%%%%%%%%%%\n")
       wxT("%%% INPUT:\n") +
       wxString::Format(
-        wxT("\\begin{minipage}[t]{%iem}\\color{red}\\bf\n"),
+        wxT("\\begin{minipage}[t]{%iem}\\color{red}\\bfseries\n"),
         configuration->GetLabelWidth()/14
         ) +
       m_inputLabel->ToTeX() +
@@ -1973,10 +1980,25 @@ void GroupCell::BreakLines()
 
 void GroupCell::BreakLines(Cell *cell)
 {
+  if(cell == NULL)
+    return;
+
+  UnBreakUpCells(cell);
+  if(BreakUpCells(cell))
+  {
+    if(m_output != NULL)
+    {
+      m_output->ResetSizeList();
+      m_output->RecalculateList(m_mathFontSize);
+    }
+    ResetData();
+  }
+
   int fullWidth = (*m_configuration)->GetClientWidth();
   Configuration *configuration = (*m_configuration);
   int currentWidth = GetLineIndent(cell);
-  fullWidth -= configuration->GetIndent();
+  if((cell->GetStyle() != TS_LABEL) && (cell->GetStyle() != TS_USERLABEL))
+    fullWidth -= configuration->GetIndent();
 
   // Don't let the layout degenerate for small window widths
   if (fullWidth < Scale_Px(150)) fullWidth = Scale_Px(150);
@@ -1984,12 +2006,12 @@ void GroupCell::BreakLines(Cell *cell)
   while (cell != NULL && !m_hide)
   {
     cell->ResetData();
-    cell->BreakLine(false);
+    cell->SoftLineBreak(false);
     if (!cell->m_isBrokenIntoLines)
     {
       if (cell->BreakLineHere() || (currentWidth + cell->GetWidth() >= fullWidth))
       {
-        cell->BreakLine(true);
+        cell->SoftLineBreak(true);
         Cell *nextCell = cell;
         if(cell->m_nextToDraw)
           nextCell = cell->m_nextToDraw;
@@ -2026,54 +2048,98 @@ void GroupCell::SelectOutput(Cell **start, Cell **end)
     *end = *start = NULL;
 }
 
-void GroupCell::BreakUpCells()
+bool GroupCell::BreakUpCells(Cell *cell)
 {
-  BreakUpCells(m_output);
-}
-
-void GroupCell::BreakUpCells(Cell *cell)
-{
-  int clientWidth = (*m_configuration)->GetClientWidth();
-  if(cell == NULL)
-    return;
-
-  Cell *tmp = cell;
-
   bool lineHeightsChanged = false;
-    
-  while (tmp != NULL && !m_hide)
+
+  if(cell == NULL)
+    return false;
+
+  int showLength;
+  switch ((*m_configuration)->ShowLength())
   {
-    if (tmp->GetWidth() +
-        (*m_configuration)->GetIndent() +
-        Scale_Px((*m_configuration)->GetLabelWidth()) > clientWidth)
-    {
-      if (tmp->BreakUp())
-        lineHeightsChanged = true;
-    }
-    tmp = tmp->m_nextToDraw;
+  case 0:
+    showLength = 5000;
+    break;
+  case 1:
+    showLength = 10000;
+    break;
+  case 2:
+    showLength = 25000;
+    break;
+  case 3:
+    showLength = 50000;
+    break;
+  default:
+    showLength = 500;    
   }
 
-  if(lineHeightsChanged)
+  // Reduce the number of steps involved in layouting big equations
+  if(m_cellsInGroup > showLength)
   {
-    if(m_output != NULL)
+    wxLogMessage(_("Resolving to 1D layout for one cell in order to save time"));
+    while (cell != NULL && !m_hide)
     {
-      m_output->ResetSizeList();
-      m_output->RecalculateList(m_mathFontSize);
+      if (cell->BreakUp())
+        lineHeightsChanged = true;
+      cell = cell->m_nextToDraw;
     }
-    ResetData();
+    return lineHeightsChanged;
+  }
+  else
+  {
+    int clientWidth = (*m_configuration)->GetClientWidth();
+    
+    while (cell != NULL && !m_hide)
+    {
+      if ((!cell->m_isBrokenIntoLines) &&
+          ((cell->GetWidth() +
+            (*m_configuration)->GetIndent() +
+            Scale_Px((*m_configuration)->GetLabelWidth()) > clientWidth)))
+      {
+        if (cell->BreakUp())
+          lineHeightsChanged = true;
+      }
+      cell = cell->m_nextToDraw;
+    }
+    
+    return lineHeightsChanged;
   }
 }
 
-void GroupCell::UnBreakUpCells()
+void GroupCell::UnBreakUpCells(Cell *cell)
 {
-  Cell *tmp = m_output;
-  while (tmp != NULL)
+  int showLength;
+  switch ((*m_configuration)->ShowLength())
   {
-    if (tmp->m_isBrokenIntoLines)
-    {
-      tmp->Unbreak();
-    }
-    tmp = tmp->m_nextToDraw;
+  case 0:
+    showLength = 50;
+    break;
+  case 1:
+    showLength = 500;
+    break;
+  case 2:
+    showLength = 2500;
+    break;
+  case 3:
+    showLength = 5000;
+    break;
+  default:
+    showLength = 500;    
+  }
+
+  // Reduce the number of steps involved in layouting big equations
+  if(m_cellsInGroup > showLength)
+  {
+    wxLogMessage(_("Resolving to linear layout for one big cell in order to save time"));
+    return;
+  }
+ 
+  while (cell != NULL)
+  {
+    if (cell->m_isBrokenIntoLines)
+      cell->Unbreak();
+    cell = cell->m_nextToDraw;
   }
 }
 
