@@ -236,6 +236,7 @@ public:
   void RecalculateHeight(int fontsize);
   //! Recalculate the height of the input part of the cell
   void RecalculateHeightInput();
+  virtual wxRect GetRect(bool all = false);
   /*! Recalculate the height of the output part of the cell
 
     \attention Needs to be in sync with the height calculation done during Draw() and
@@ -253,14 +254,14 @@ public:
   */
   void Recalculate();
 
-  //! Attempt to split math objects that are wider than the screen into multiple lines.
-  void BreakUpCells();
-
-  //! Attempt to split math objects that are wider than the screen into multiple lines.
-  void BreakUpCells(Cell *cell);
+  /*! Attempt to split math objects that are wider than the screen into multiple lines.
+    
+    \retval true, if this action has changed the height of cells.
+   */
+  bool BreakUpCells(Cell *cell);
 
   //! Undo a BreakUpCells
-  void UnBreakUpCells();
+  void UnBreakUpCells(Cell *cell);
 
   //! Break this cell into lines
   void BreakLines();
@@ -504,7 +505,13 @@ private:
   bool m_inEvaluationQueue;
   bool m_lastInEvaluationQueue;
   int m_inputWidth, m_inputHeight, m_outputWidth, m_outputHeight;
-
+  //! The number of cells the current group contains (-1, if no GroupCell)
+  int m_cellsInGroup;
+  void UpdateCellsInGroup(){
+    if(m_output != NULL)
+      m_cellsInGroup = 2 + m_output->CellsInListRecursive();
+    else
+      m_cellsInGroup = 2;
+  }
 };
-
 #endif /* GROUPCELL_H */
