@@ -30,6 +30,7 @@
 #include <wx/zstream.h>
 #include <wx/txtstrm.h>
 #include <wx/regex.h>
+#include <wx/stdpaths.h>
 
 wxMemoryBuffer Image::ReadCompressedImage(wxInputStream *data)
 {
@@ -98,9 +99,11 @@ Image::Image(Configuration **config, wxMemoryBuffer image, wxString type)
 Image::Image(Configuration **config, const wxBitmap &bitmap)
 {
   m_configuration = config;
-  LoadImage(bitmap);
+  m_width = 1;
+  m_height = 1;
   m_maxWidth = -1;
   m_maxHeight = -1;
+  LoadImage(bitmap);
 }
 
 // constructor which loads an image
@@ -108,9 +111,11 @@ Image::Image(Configuration **config, wxString image, bool remove, wxFileSystem *
 {
   m_configuration = config;
   m_scaledBitmap.Create(1, 1);
-  LoadImage(image, remove, filesystem);
+  m_width = 1;
+  m_height = 1;
   m_maxWidth = -1;
   m_maxHeight = -1;
+  LoadImage(image, remove, filesystem);
 }
 
 Image::~Image()
@@ -503,9 +508,13 @@ wxBitmap Image::GetBitmap(double scale)
     {
       m_isOk = false;
       // Create a "image not loaded" bitmap.
-      m_scaledBitmap.Create(400, 250);
+      m_scaledBitmap.Create(m_width, m_height);
 
-      wxString error(_("Error"));
+      wxString error;
+      if(m_imageName != wxEmptyString)
+        error = wxString::Format(_("Error: Cannot render %s."), m_imageName);
+      else
+        error = wxString::Format(_("Error: Cannot render the image."));
 
       wxMemoryDC dc;
       dc.SelectObject(m_scaledBitmap);
@@ -513,13 +522,12 @@ wxBitmap Image::GetBitmap(double scale)
       int width = 0, height = 0;
       dc.GetTextExtent(error, &width, &height);
 
-      dc.DrawRectangle(0, 0, 400, 250);
-      dc.DrawLine(0, 0, 400, 250);
-      dc.DrawLine(0, 250, 400, 0);
-      dc.DrawText(error, 200 - width / 2, 125 - height / 2);
+      dc.DrawRectangle(0, 0, m_width - 1, m_height - 1);
+      dc.DrawLine(0, 0, m_width - 1, m_height - 1);
+      dc.DrawLine(0, m_height - 1, m_width - 1, 0);
 
       dc.GetTextExtent(error, &width, &height);
-      dc.DrawText(error, 200 - width / 2, 150 - height / 2);
+      dc.DrawText(error, (m_width - width) / 2, (m_height - height) / 2);
     }
   }
 
@@ -555,6 +563,7 @@ void Image::LoadImage(const wxBitmap &bitmap)
 
 void Image::LoadImage(wxString image, bool remove, wxFileSystem *filesystem)
 {
+  m_imageName = image;
   m_compressedImage.Clear();
   m_scaledBitmap.Create(1, 1);
 
@@ -607,6 +616,9 @@ void Image::LoadImage(wxString image, bool remove, wxFileSystem *filesystem)
 
   m_extension = wxFileName(image).GetExt();
 
+  m_originalWidth = 700;
+  m_originalHeight = 300;
+
   if (Image.Ok())
   {
     m_originalWidth = Image.GetWidth();
@@ -614,12 +626,8 @@ void Image::LoadImage(wxString image, bool remove, wxFileSystem *filesystem)
     m_isOk = true;
   }
   else
-  {
-    // Leave space for an image showing an error message
-    m_originalWidth = 400;
-    m_originalHeight = 250;
     m_isOk = false;
-  }
+
   Recalculate();
 
 }
@@ -640,8 +648,8 @@ void Image::Recalculate(double scale)
 
   if ((width < 1) || (height < 1))
   {
-    m_width = 400;
-    m_height = 250;
+    m_width = 700;
+    m_height = 300;
     return;
   }
 
@@ -673,7 +681,19 @@ void Image::Recalculate(double scale)
   m_height = (int) (scale * height);
   m_width = (int) (scale * width);
 
-  // Clear this cell's image cache if it doesn't contain an image of the size
+  if((m_height < 1) || (m_width < 1))
+  {
+    scale = 1;
+    m_height = (int) (scale * height);
+    m_width = (int) (scale * width);
+  }
+
+  if((m_height < 1) || (m_width < 1))
+  {
+    m_height = 100;
+    m_width = 100;
+  }
+// Clear this cell's image cache if it doesn't contain an image of the size
   // we need right now.
   if (m_scaledBitmap.GetWidth() != m_width)
     ClearCache();
