@@ -82,11 +82,10 @@ wxString MarkDownParser::MarkDown(wxString str)
     {
       // The line contains actual text..
 
-      // If the line begins with a star followed by a space it is part
-      // of a bullet list
-      if (str.Left(2) == wxT("* "))
+      // Let's see if the line is the start of a bullet list item
+      if ((str.StartsWith("* ")) &&
+          ((indentationTypes.empty())||(indentationTypes.back() == wxT('*'))))
       {
-        // We are part of a bullet list.
 
         // Remove the bullet list start marker from our string.
         str = str.Right(str.Length() - 2);
@@ -96,37 +95,46 @@ wxString MarkDownParser::MarkDown(wxString str)
         if (indentationLevels.empty())
         {
           // This is the first item => Start the itemization.
-          result += itemizeBegin() + itemizeItem();
+          result += itemizeBegin();
           indentationLevels.push_back(index);
           indentationTypes.push_back(wxT('*'));
         }
         else
         {
-          // We are inside a bullet list.
+          // End the previous item before we start a new one on the same level.
+          if (index == indentationLevels.back())
+            result += itemizeEndItem();
+        }
 
-          // Are we on a new indentation level?
-          if (indentationLevels.back() < index)
-          {
-            // A new identation level => add the itemization-start-command.
-            result += itemizeEndItem() + itemizeBegin();
-            indentationLevels.push_back(index);
-            indentationTypes.push_back(wxT('*'));
-          }
+        // Did we switch to a higher indentation level?
+        if (index > indentationLevels.back())
+        {
+          // A higher identation level => add the itemization-start-command.
+          result += itemizeBegin();
+          indentationLevels.push_back(index);
+          indentationTypes.push_back(wxT('*'));
+        }
 
-          // End lists if we are at a old indentation level.
-          while (!indentationLevels.empty() && (indentationLevels.back() > index))
+        // Did we switch to a lower indentation level?
+        if (index < indentationLevels.back())
+        {
+          while (!indentationLevels.empty() && (index < indentationLevels.back()))
           {
-            if (indentationTypes.back() == wxT('*'))
-              result += itemizeEnd();
-            else
-              result += quoteEnd();
+            result += itemizeEndItem();
+            result += itemizeEnd();
             indentationLevels.pop_back();
             indentationTypes.pop_back();
           }
-
-          // Add a new item marker.
-          result += itemizeEndItem() + itemizeItem();
+          result += itemizeEndItem();
         }
+
+        // Add a new item marker.
+        result += itemizeItem();
+
+        // Add the item itself
+        str.Trim();
+        if(str.EndsWith(NewLine()))
+          str = str.Left(str.Length() - NewLine().Length());
         result += str += wxT(" ");
       }
       else if (str.StartsWith(quoteChar() + wxT(" ")))
@@ -162,7 +170,10 @@ wxString MarkDownParser::MarkDown(wxString str)
           while (!indentationLevels.empty() && (indentationLevels.back() > index))
           {
             if (indentationTypes.back() == wxT('*'))
+            {
+              result += itemizeEndItem();
               result += itemizeEnd();
+            }
             else
               result += quoteEnd();
             indentationLevels.pop_back();
@@ -179,44 +190,45 @@ wxString MarkDownParser::MarkDown(wxString str)
         // and add a new item if we still are inside a list.
         if (!indentationLevels.empty())
         {
+          // Add the text to the output.
+          if((result != wxEmptyString) &&
+             (!result.EndsWith(itemizeEndItem())) &&
+             (!result.EndsWith(itemizeEnd())) &&
+             (!result.EndsWith(quoteEnd()))
+            )
+            result += NewLine();
           if (indentationLevels.back() > index)
           {
-            if (!m_configuration->GetAutoWrap())
-              result += itemizeEndItem();
             while ((!indentationLevels.empty()) &&
                    (indentationLevels.back() > index))
             {
               if (indentationTypes.back() == wxT('*'))
+              {
+                result += itemizeEndItem();
                 result += itemizeEnd();
+              }
               else
                 result += quoteEnd();
+
               indentationLevels.pop_back();
               indentationTypes.pop_back();
             }
-            if (!indentationLevels.empty()) result += itemizeItem();
           }
-          line = line.Right(line.Length() - index);
         }
-
-        // Add the text to the output.        
-//        if (!m_configuration->GetAutoWrap())
-//          result += line + "\n";
-//        else
-          result += line + NewLine();
-
+        line = line.Right(line.Length() - index);
+        result += line;
       }
     }
-    else
-    {
-      if (lines.HasMoreTokens()) result += NewLine();
-    }
   }
 
   // Close all item lists
   while (!indentationLevels.empty())
   {
     if (indentationTypes.back() == wxT('*'))
+    {
+      result += itemizeEndItem();
       result += itemizeEnd();
+    }
     else
       result += quoteEnd();
     indentationLevels.pop_back();
@@ -250,27 +262,27 @@ MarkDownTeX::MarkDownTeX(Configuration *cfg) : MarkDownParser(cfg)
   regexReplaceList.push_back(
           new RegexReplacer(wxT("\\\\verb\\|<\\|\\\\verb\\|<\\|"), wxT("\\\\ensuremath{\\\\ll}")));
   regexReplaceList.push_back(
-          new RegexReplacer(L"\xDCB6", wxT("~")));
+          new RegexReplacer(wxT("\xDCB6"), wxT("~")));
 }
 
 MarkDownHTML::MarkDownHTML(Configuration *cfg) : MarkDownParser(cfg)
 {
   regexReplaceList.push_back(
-          new RegexReplacer(wxT("\\&lt;=\\&gt;"), wxT("\\&hArr;")));
+          new RegexReplacer(wxT("\\&lt;=\\&gt;"), wxT("\x21d4")));
   regexReplaceList.push_back(
-          new RegexReplacer(wxT("=\\&gt;"), wxT("\\&rArr;")));
+          new RegexReplacer(wxT("=\\&gt;"), wxT("\x21d2")));
   regexReplaceList.push_back(
-          new RegexReplacer(wxT("&lt;-\\&gt;"), wxT("\\&harr;")));
+          new RegexReplacer(wxT("&lt;-\\&gt;"), wxT("\x2194")));
   regexReplaceList.push_back(
-          new RegexReplacer(wxT("-\\&gt;"), wxT("\\&rarr;")));
+          new RegexReplacer(wxT("-\\&gt;"), wxT("\x2192")));
   regexReplaceList.push_back(
-          new RegexReplacer(wxT("\\&lt;-"), wxT("\\&larr;")));
+          new RegexReplacer(wxT("\\&lt;-"), wxT("\x2190")));
   regexReplaceList.push_back(
-          new RegexReplacer(wxT("\\&lt;="), wxT("\\&le;")));
+          new RegexReplacer(wxT("\\&lt;="), wxT("\x2264")));
   regexReplaceList.push_back(
-          new RegexReplacer(wxT("\\&gt;="), wxT("\\&ge;")));;
+          new RegexReplacer(wxT("\\&gt;="), wxT("\x2265")));
   regexReplaceList.push_back(
-          new RegexReplacer(wxT("\\+/-"), wxT("\\&plusmn;")));;
+          new RegexReplacer(wxT("\\+/-"), wxT("\xB1")));
   regexReplaceList.push_back(
-          new RegexReplacer(L"\xDCB6", wxT("&nbsp;")));
+    new RegexReplacer(wxT("\xDCB6"), wxT("\xA0")));
 }
