@@ -467,10 +467,19 @@ Cell *MathParser::ParseText(wxXmlNode *node, TextStyle style)
       case TS_ERROR:
         cell->SetType(MC_TYPE_ERROR);
         break;
-        
+
       case TS_WARNING:
         cell->SetType(MC_TYPE_WARNING);
         break;
+
+      case TS_LABEL:
+        cell->SetType(MC_TYPE_LABEL);
+        break;
+
+      case TS_USERLABEL:
+        cell->SetType(MC_TYPE_LABEL);
+        break;
+
       default:
         cell->SetType(m_ParserStyle);
       }
@@ -923,6 +932,12 @@ Cell *MathParser::ParseTag(wxXmlNode *node, bool all)
           if (framerate.ToLong(&fr))
             slideShow->SetFrameRate(fr);
         }
+        if (node->GetAttribute(wxT("frame"), &framerate))
+        {
+          long frame;
+          if (framerate.ToLong(&frame))
+            slideShow->SetDisplayedIndex(frame);
+        }
         if (node->GetAttribute(wxT("running"), wxT("true")) == wxT("false"))
           slideShow->AnimationRunning(false);
         while (tokens.HasMoreTokens())
@@ -1022,17 +1037,17 @@ Cell *MathParser::ParseLine(wxString s, CellType style)
   Cell *cell = NULL;
 
   int showLength;
-  
+
   switch ((*m_configuration)->ShowLength())
   {
     case 0:
-      showLength = 50000;
+      showLength = 6000;
       break;
     case 1:
-      showLength = 500000;
+      showLength = 20000;
       break;
     case 2:
-      showLength = 5000000;
+      showLength = 250000;
       break;
     case 3:
       showLength = 0;
@@ -1060,8 +1075,9 @@ Cell *MathParser::ParseLine(wxString s, CellType style)
   else
   {
     cell = new TextCell(NULL, m_configuration, m_cellPointers,
-                        _(" << Expression longer than allowed by the configuration setting! >>"));
-    cell->SetToolTip(_("The maximum size of a expression wxMaxima is allowed to be displayed "
+                        _("(Expression longer than allowed by the configuration setting)"),
+      TS_WARNING);
+    cell->SetToolTip(_("The maximum size of the expressions wxMaxima is allowed to display "
                        "can be changed in the configuration dialogue."
                        ));
     cell->ForceBreakLine(true);
