@@ -52,17 +52,6 @@ ParenCell::ParenCell(Cell *parent, Configuration **config, CellPointers *cellPoi
   m_close = new TextCell(parent, config, cellPointers, wxT(")"));
 }
 
-void ParenCell::SetGroup(Cell *parent)
-{
-  m_group = parent;
-  if (m_innerCell != NULL)
-    m_innerCell->SetGroupList(parent);
-  if (m_open != NULL)
-    m_open->SetGroupList(parent);
-  if (m_close != NULL)
-    m_close->SetGroupList(parent);
-}
-
 Cell *ParenCell::Copy()
 {
   ParenCell *tmp = new ParenCell(m_group, m_configuration, m_cellPointers);
@@ -170,7 +159,6 @@ void ParenCell::SetFont(int fontsize)
 
 void ParenCell::RecalculateWidths(int fontsize)
 {
-  Cell::RecalculateWidths(fontsize);
   Configuration *configuration = (*m_configuration);
 
   // Add a dummy contents to empty parenthesis
@@ -233,14 +221,14 @@ void ParenCell::RecalculateWidths(int fontsize)
   m_width = m_innerCell->GetFullWidth() + m_signWidth * 2;
   if(m_isBrokenIntoLines)
     m_width = 0;
-  ResetData();
+  Cell::RecalculateWidths(fontsize);
 }
 
 void ParenCell::RecalculateHeight(int fontsize)
 {
   Cell::RecalculateHeight(fontsize);
   Configuration *configuration = (*m_configuration);
-  m_height = MAX(m_signHeight,m_innerCell->GetMaxHeight()) + Scale_Px(2);
+  m_height = wxMax(m_signHeight,m_innerCell->GetMaxHeight()) + Scale_Px(2);
   m_center = m_height / 2;
 
   SetFont(fontsize);
@@ -254,8 +242,8 @@ void ParenCell::RecalculateHeight(int fontsize)
 
   if (m_isBrokenIntoLines)
   {
-    m_height = MAX(m_innerCell->GetMaxHeight(), m_open->GetMaxHeight());
-    m_center = MAX(m_innerCell->GetMaxCenter(), m_open->GetMaxCenter());
+    m_height = wxMax(m_innerCell->GetMaxHeight(), m_open->GetMaxHeight());
+    m_center = wxMax(m_innerCell->GetMaxCenter(), m_open->GetMaxCenter());
   }
   else
   {
@@ -288,7 +276,7 @@ void ParenCell::RecalculateHeight(int fontsize)
           wxPoint(m_currentPoint.x + m_signWidth,
                   m_currentPoint.y));
       
-      m_height = MAX(m_signHeight,m_innerCell->GetMaxHeight()) + Scale_Px(2);      
+      m_height = wxMax(m_signHeight,m_innerCell->GetMaxHeight()) + Scale_Px(2);      
       m_center = m_height / 2;   
     }
   }
@@ -410,6 +398,19 @@ wxString ParenCell::ToString()
   return s;
 }
 
+wxString ParenCell::ToMatlab()
+{
+  wxString s;
+  if (!m_isBrokenIntoLines)
+  {
+	if (m_print)
+	  s = wxT("(") + m_innerCell->ListToMatlab() + wxT(")");
+	else
+	  s = m_innerCell->ListToMatlab();
+  }
+  return s;
+}
+
 wxString ParenCell::ToTeX()
 {
   wxString s;
@@ -489,8 +490,9 @@ bool ParenCell::BreakUp()
       m_nextToDraw->m_previousToDraw = m_close;
     m_nextToDraw = m_open;
 
-    m_height = MAX(m_innerCell->GetMaxHeight(), m_open->GetMaxHeight());
-    m_center = MAX(m_innerCell->GetMaxCenter(), m_open->GetMaxCenter());
+    ResetData();
+    m_height = wxMax(m_innerCell->GetMaxHeight(), m_open->GetMaxHeight());
+    m_center = wxMax(m_innerCell->GetMaxCenter(), m_open->GetMaxCenter());
     return true;
   }
   return false;
