@@ -76,6 +76,8 @@ public:
 
   wxString ToString();
 
+  wxString ToMatlab();
+
   wxString ToTeX();
 
   wxString ToMathML();
@@ -84,8 +86,6 @@ public:
 
   wxString ToXML();
 
-  void SetGroup(Cell *parent);
-
 protected:
    /*! How to create a big parenthesis sign?
    */
