@@ -29,6 +29,7 @@
 #include <wx/txtstrm.h> 
 #include <wx/filename.h> 
 #include <wx/wfstream.h>
+#include <wx/stdpaths.h>
 #include "Configuration.h"
 #include "GroupCell.h"
 #include <wx/config.h>
@@ -65,7 +66,7 @@ Svgout::Svgout(Configuration **configuration, wxString filename, double scale)
 #if wxCHECK_VERSION(3, 1, 0)
   m_recalculationDc->SetBitmapHandler(new wxSVGBitmapEmbedHandler());
 #endif
-  *m_configuration = new Configuration(*m_recalculationDc);
+  *m_configuration = new Configuration(m_recalculationDc);
   (*m_configuration)->ShowCodeCells(m_oldconfig->ShowCodeCells());
   (*m_configuration)->SetClientWidth(700*m_scale);
   (*m_configuration)->SetZoomFactor_temporarily(1);
@@ -207,13 +208,13 @@ void Svgout::BreakLines()
   {
     if (!tmp->m_isBrokenIntoLines)
     {
-      tmp->BreakLine(false);
+      tmp->SoftLineBreak(false);
       tmp->ResetData();
       if (tmp->BreakLineHere() ||
           (currentWidth + tmp->GetWidth() >= fullWidth))
       {
         currentWidth = tmp->GetWidth();
-        tmp->BreakLine(true);
+        tmp->SoftLineBreak(true);
       }
       else
         currentWidth += (tmp->GetWidth());
@@ -243,12 +244,12 @@ void Svgout::GetMaxPoint(int *width, int *height)
           currentHeight += MC_LINE_SKIP;
         *height = currentHeight;
         currentWidth = tmp->GetWidth();
-        *width = MAX(currentWidth, *width);
+        *width = wxMax(currentWidth, *width);
       }
       else
       {
         currentWidth += (tmp->GetWidth());
-        *width = MAX(currentWidth, *width);
+        *width = wxMax(currentWidth, *width);
       }
       bigSkip = tmp->m_bigSkip;
     }
