@@ -198,7 +198,6 @@ void SlideShow::SetDisplayedIndex(int ind)
 
 void SlideShow::RecalculateWidths(int fontsize)
 {
-  Cell::RecalculateWidths(fontsize);
   // Here we recalculate the height, as well:
   //  - This doesn't cost much time and
   //  - as image cell's sizes might change when the resolution does
@@ -226,6 +225,7 @@ void SlideShow::RecalculateWidths(int fontsize)
     }
   }       
   m_center = m_height / 2;
+  Cell::RecalculateWidths(fontsize);
 }
 
 void SlideShow::RecalculateHeight(int fontsize)
@@ -299,6 +299,11 @@ wxString SlideShow::ToString()
   return wxT(" << Animation >> ");
 }
 
+wxString SlideShow::ToMatlab()
+{
+  return wxT(" << Animation >> ");
+}
+
 wxString SlideShow::ToTeX()
 {
   return wxT(" << Graphics >> ");
@@ -331,6 +336,8 @@ wxString SlideShow::ToXML()
     flags += wxT(" running=\"true\"");
   else
     flags += wxT(" running=\"false\"");
+  flags += wxString::Format(wxT(" frame=\"%i\""),m_displayed);
+
   if (m_framerate > 0)
     flags +=  wxString::Format(wxT(" fr=\"%i\""), GetFrameRate());
   return wxT("\n<slide") + flags + wxT(">") + images + wxT("</slide>");
