@@ -26,6 +26,7 @@
 */
 #include "StatusBar.h"
 #include <wx/artprov.h>
+#include <wx/display.h>
 #include "invalidImage.h"
 #include "../art/statusbar/images.h"
 #include <wx/mstream.h>
@@ -33,6 +34,7 @@
 
 StatusBar::StatusBar(wxWindow *parent, int id) : wxStatusBar(parent, id)
 {
+  m_ppi = wxSize(-1,-1);
   int widths[] = {-1, 300, GetSize().GetHeight()};
   m_maximaPercentage = -1;
   m_oldmaximaPercentage = -1;
@@ -43,6 +45,39 @@ StatusBar::StatusBar(wxWindow *parent, int id) : wxStatusBar(parent, id)
           "Maxima, the program that does the actual mathematics and wxMaxima, which displays the worksheet are kept in separate processes. This means that even if maxima crashes wxMaxima (and therefore the worksheet) stays intact. Both programs communicate over a local network socket. This time this socket could not be created which might be caused by a firewall that it setup to not only intercepts connections from the outside, but also to intercept connections between two programs that run on the same computer.");
   m_noConnectionToolTip = _(
           "Maxima, the program that does the actual mathematics and wxMaxima, which displays the worksheet are kept in separate processes. This means that even if maxima crashes wxMaxima (and therefore the worksheet) stays intact. Currently the two programs aren't connected to each other which might mean that maxima is still starting up or couldn't be started. Alternatively it can be caused by a firewall that it setup to not only intercepts connections from the outside, but also to intercept connections between two programs that run on the same computer. Another reason for maxima not starting up might be that maxima cannot be found (see wxMaxima's Configuration dialogue for a way to specify maxima's location) or isn't in a working order.");
+  UpdateBitmaps();
+  m_networkStatus = new wxStaticBitmap(this, wxID_ANY, m_network_offline);
+  m_networkStatus->SetToolTip(m_stdToolTip);
+  ReceiveTimer.SetOwner(this, wxID_ANY);
+  SendTimer.SetOwner(this, wxID_ANY);
+  m_icon_shows_receive = m_icon_shows_transmit = false;
+  m_networkState = offline;
+  // Mark the network state as "to be changed"
+  m_oldNetworkState = receive;
+}
+
+void StatusBar::UpdateBitmaps()
+{
+  wxSize ppi(-1,-1);
+#if wxCHECK_VERSION(3, 1, 1)
+  wxDisplay display;
+  
+  int display_idx = wxDisplay::GetFromWindow(GetParent());
+  if (display_idx < 0)
+    ppi = wxSize(72,72);
+  else
+    ppi = wxDisplay(display_idx).GetPPI();
+#endif
+
+  if((ppi.x < 10) || (ppi.y < 10))
+    ppi = wxGetDisplayPPI();
+  if((ppi.x <= 10) || (ppi.y <= 10))
+    ppi = wxSize(72,72);
+
+  if((ppi.x == m_ppi.x) && (ppi.y == m_ppi.y))
+    return;
+
+  m_ppi = ppi;
   m_network_error = GetImage("network-error",
                              network_error_128_png,network_error_128_png_len,
                              network_error_192_png,network_error_192_png_len
@@ -68,14 +103,6 @@ StatusBar::StatusBar(wxWindow *parent, int id) : wxStatusBar(parent, id)
                               network_transmit_receive_128_png,network_transmit_receive_128_png_len,
                               network_transmit_receive_192_png,network_transmit_receive_192_png_len
     );
-  m_networkStatus = new wxStaticBitmap(this, wxID_ANY, m_network_offline);
-  m_networkStatus->SetToolTip(m_stdToolTip);
-  ReceiveTimer.SetOwner(this, wxID_ANY);
-  SendTimer.SetOwner(this, wxID_ANY);
-  m_icon_shows_receive = m_icon_shows_transmit = false;
-  m_networkState = offline;
-  // Mark the network state as "to be changed"
-  m_oldNetworkState = receive;
 }
 
 void StatusBar::OnTimerEvent(wxTimerEvent &WXUNUSED(event))
@@ -126,6 +153,7 @@ void StatusBar::OnTimerEvent(wxTimerEvent &WXUNUSED(event))
 
 void StatusBar::NetworkStatus(networkState status)
 {
+  UpdateBitmaps();
   if((status != m_oldNetworkState) || (m_maximaPercentage !=
                                        m_oldmaximaPercentage))
   {
@@ -203,9 +231,31 @@ wxBitmap StatusBar::GetImage(wxString name,
                           unsigned char *data_128, size_t len_128,
                           unsigned char *data_192, size_t len_192)
 {
-  double targetWidth = static_cast<double>(GetSize().GetHeight()) / wxGetDisplayPPI().y * wxGetDisplayPPI().x*GetContentScaleFactor();
+  wxSize ppi;
+#if wxCHECK_VERSION(3, 1, 1)
+  wxDisplay display;
+  
+  int display_idx = wxDisplay::GetFromWindow(GetParent());
+  if (display_idx < 0)
+    ppi = wxSize(72,72);
+  else
+    ppi = wxDisplay(display_idx).GetPPI();
+#else
+  ppi = wxGetDisplayPPI();
+
+  if (ppi.x < 72)
+    ppi.x = 72;
+  if (ppi.y < 72)
+    ppi.y = 72;
+#endif
+  double targetWidth = static_cast<double>(GetSize().GetHeight()) / ppi.y * ppi.x*GetContentScaleFactor();
   double targetHeight = static_cast<double>(GetSize().GetHeight())*GetContentScaleFactor();
 
+  if(targetWidth < 16)
+    targetWidth = 16;
+  if(targetHeight < 16)
+    targetHeight = 16;
+  
   wxBitmap bmp = wxArtProvider::GetBitmap(name, wxART_TOOLBAR, wxSize(targetWidth, targetHeight));
   wxImage img;
 
@@ -253,12 +303,12 @@ wxBitmap StatusBar::GetImage(wxString name,
     img = wxImage(invalidImage_xpm);
   }
 
-  targetWidth = static_cast<double>(GetSize().GetHeight()) / wxGetDisplayPPI().y * wxGetDisplayPPI().x;
+  targetWidth = static_cast<double>(GetSize().GetHeight());
   targetHeight = static_cast<double>(GetSize().GetHeight());
 
   img.Rescale(targetWidth, targetHeight, wxIMAGE_QUALITY_HIGH);
 
-#if defined __WXMAC__
+#if defined __WXOSX__
   return wxBitmap(img,wxBITMAP_SCREEN_DEPTH,GetContentScaleFactor());
 #else
   return wxBitmap(img,wxBITMAP_SCREEN_DEPTH);
