@@ -37,15 +37,6 @@ SubCell::SubCell(Cell *parent, Configuration **config, CellPointers *cellPointer
   m_indexCell = NULL;
 }
 
-void SubCell::SetGroup(Cell *parent)
-{
-  m_group = parent;
-  if (m_baseCell != NULL)
-    m_baseCell->SetGroupList(parent);
-  if (m_indexCell != NULL)
-    m_indexCell->SetGroupList(parent);
-}
-
 Cell *SubCell::Copy()
 {
   SubCell *tmp = new SubCell(m_group, m_configuration, m_cellPointers);
@@ -94,19 +85,18 @@ void SubCell::SetBase(Cell *base)
 
 void SubCell::RecalculateWidths(int fontsize)
 {
-  Cell::RecalculateWidths(fontsize);
   m_baseCell->RecalculateWidthsList(fontsize);
-  m_indexCell->RecalculateWidthsList(MAX(MC_MIN_SIZE, fontsize - SUB_DEC));
+  m_indexCell->RecalculateWidthsList(wxMax(MC_MIN_SIZE, fontsize - SUB_DEC));
   m_width = m_baseCell->GetFullWidth() + m_indexCell->GetFullWidth() -
             Scale_Px(2);
-  ResetData();
+  Cell::RecalculateWidths(fontsize);
 }
 
 void SubCell::RecalculateHeight(int fontsize)
 {
   Cell::RecalculateHeight(fontsize);
   m_baseCell->RecalculateHeightList(fontsize);
-  m_indexCell->RecalculateHeightList(MAX(MC_MIN_SIZE, fontsize - SUB_DEC));
+  m_indexCell->RecalculateHeightList(wxMax(MC_MIN_SIZE, fontsize - SUB_DEC));
   m_height = m_baseCell->GetMaxHeight() + m_indexCell->GetMaxHeight() -
              Scale_Px((8 * fontsize) / 10 + MC_EXP_INDENT);
   m_center = m_baseCell->GetCenter();
@@ -147,6 +137,22 @@ wxString SubCell::ToString()
   return s;
 }
 
+wxString SubCell::ToMatlab()
+{
+  if (m_altCopyText != wxEmptyString)
+  {
+	return m_altCopyText;
+  }
+
+  wxString s;
+  if (m_baseCell->IsCompound())
+	s += wxT("(") + m_baseCell->ListToMatlab() + wxT(")");
+  else
+	s += m_baseCell->ListToMatlab();
+  s += wxT("[") + m_indexCell->ListToMatlab() + wxT("]");
+  return s;
+}
+
 wxString SubCell::ToTeX()
 {
   wxString s;
