@@ -48,6 +48,8 @@ public:
 
   wxString ToString();
 
+  wxString ToMatlab();
+
   wxString ToTeX();
 
   wxString ToMathML();
@@ -56,8 +58,6 @@ public:
 
   wxString ToXML();
 
-  void SetGroup(Cell *parent);
-
 protected:
   Cell *m_baseCell;
   Cell *m_indexCell;
