@@ -50,6 +50,8 @@ public:
 
   wxString ToString();
 
+  wxString ToMatlab();
+
   wxString ToTeX();
 
   wxString ToXML();
@@ -58,8 +60,6 @@ public:
 
   wxString ToMathML();
 
-  void SetGroup(Cell *parent);
-
 protected:
   Cell *m_baseCell;
   Cell *m_exptCell;
