@@ -30,8 +30,34 @@
 #include "TextCell.h"
 #include "wx/config.h"
 
-TextCell::TextCell(Cell *parent, Configuration **config, CellPointers *cellPointers, wxString text) : Cell(parent, config)
+TextCell::TextCell(Cell *parent, Configuration **config, CellPointers *cellPointers,
+                   wxString text, TextStyle style) : Cell(parent, config)
 {
+  switch(m_textStyle = style)
+  {
+  case TS_DEFAULT: m_type = MC_TYPE_DEFAULT; break;
+  case TS_VARIABLE: m_type = MC_TYPE_DEFAULT; break;
+  case TS_NUMBER: m_type = MC_TYPE_DEFAULT; break;
+  case TS_FUNCTION: m_type = MC_TYPE_DEFAULT; break;
+  case TS_SPECIAL_CONSTANT: m_type = MC_TYPE_DEFAULT; break;
+  case TS_GREEK_CONSTANT: m_type = MC_TYPE_DEFAULT; break;
+  case TS_STRING: m_type = MC_TYPE_DEFAULT; break;
+  case TS_INPUT: m_type = MC_TYPE_INPUT; break;
+  case TS_MAIN_PROMPT: m_type = MC_TYPE_MAIN_PROMPT; break;
+  case TS_OTHER_PROMPT: m_type = MC_TYPE_PROMPT; break;
+  case TS_LABEL: m_type = MC_TYPE_LABEL; break;
+  case TS_USERLABEL: m_type = MC_TYPE_LABEL; break;
+  case TS_HIGHLIGHT: m_type = MC_TYPE_DEFAULT; break;
+  case TS_WARNING: m_type = MC_TYPE_WARNING; break;
+  case TS_ERROR: m_type = MC_TYPE_ERROR; break;
+  case TS_TEXT: m_type = MC_TYPE_TEXT; break;
+  case TS_HEADING6: m_type = MC_TYPE_HEADING6; break;
+  case TS_HEADING5: m_type = MC_TYPE_HEADING5; break;
+  case TS_SUBSUBSECTION: m_type = MC_TYPE_SUBSUBSECTION; break;
+  case TS_SUBSECTION: m_type = MC_TYPE_SUBSECTION; break;
+  case TS_SECTION: m_type = MC_TYPE_SECTION; break;
+  case TS_TITLE: m_type = MC_TYPE_TITLE; break;
+  }
   m_cellPointers = cellPointers;
   m_displayedDigits_old = -1;
   m_height = -1;
@@ -67,10 +93,16 @@ void TextCell::SetStyle(TextStyle style)
     m_displayedText = wxT("\x03A8");
   if((style == TS_LABEL) || (style == TS_USERLABEL)||
      (style == TS_MAIN_PROMPT) || (style == TS_OTHER_PROMPT))
-    ForceBreakLineHere();
+    HardLineBreak();
   ResetSize();
 }
 
+void TextCell::SetType(CellType type)
+{
+  Cell::SetType(type);
+  SetFont((*m_configuration)->GetDefaultFontSize());
+}
+
 void TextCell::SetValue(const wxString &text)
 {
   m_toolTip = m_initialToolTip;
@@ -207,28 +239,33 @@ void TextCell::SetValue(const wxString &text)
   }
   else
   {
-    if((text == wxT(" LINE SEARCH FAILED. SEE"))||
-       text.Contains(wxT(" DOCUMENTATION OF ROUTINE MCSRCH")) ||
-       text.Contains(wxT(" POSSIBLE CAUSES: FUNCTION OR GRADIENT ARE INCORRECT")))
+    if((text.Contains(wxT("LINE SEARCH FAILED. SEE")))||
+       (text.Contains(wxT("DOCUMENTATION OF ROUTINE MCSRCH"))) ||
+       (text.Contains(wxT("ERROR RETURN OF LINE SEARCH:"))) ||
+       text.Contains(wxT("POSSIBLE CAUSES: FUNCTION OR GRADIENT ARE INCORRECT")))
       m_toolTip = _("This message can appear when trying to numerically find an optimum. "
                     "In this case it might indicate that a starting point lies in a local "
                     "optimum that fits the data best if one parameter is increased to "
                     "infinity or decreased to -infinity. It also can indicate that an "
                     "attempt was made to fit data to an equation that actually matches "
-		    "the data best if one parameter is set to +/- infinity.");
+                    "the data best if one parameter is set to +/- infinity.");
     if(text.StartsWith(wxT("incorrect syntax")) && (text.Contains(wxT("is not an infix operator"))))
       m_toolTip = _("A command or number wasn't preceded by a \":\", a \"$\", a \";\" or a \",\".\n"
-        "Most probable cause: A missing comma between two list items.");
+                    "Most probable cause: A missing comma between two list items.");
     if(text.StartsWith(wxT("incorrect syntax")) && (text.Contains(wxT("Found LOGICAL expression where ALGEBRAIC expression expected"))))
       m_toolTip = _("Most probable cause: A dot instead a comma between two list items containing assignments.");
+    if(text.StartsWith(wxT("incorrect syntax")) && (text.Contains(wxT("is not a prefix operator"))))
+      m_toolTip = _("Most probable cause: Two commas or similar separators in a row.");
     if(text.Contains(wxT("Illegal use of delimiter")))
       m_toolTip = _("Most probable cause: an operator was directly followed by a closing parenthesis.");
-
+    
     if(text.StartsWith(wxT("part: fell off the end.")))
-       m_toolTip = _("part() or the [] operator was used in order to extract the nth element "
-                     "of something that was less than n elements long.");
+      m_toolTip = _("part() or the [] operator was used in order to extract the nth element "
+                    "of something that was less than n elements long.");
+    if(text.StartsWith(wxT("rest: fell off the end.")))
+      m_toolTip = _("rest() tried to drop more entries from a list than the list was long.");
     if(text.StartsWith(wxT("assignment: cannot assign to")))
-       m_toolTip = _("The value of few special variables is assigned by Maxima and cannot be changed by the user. Also a few constructs aren't variable names and therefore cannot be written to.");
+      m_toolTip = _("The value of few special variables is assigned by Maxima and cannot be changed by the user. Also a few constructs aren't variable names and therefore cannot be written to.");
     if(text.StartsWith(wxT("rat: replaced ")))
       m_toolTip = _("Normally computers use floating-point numbers that can be handled "
                     "incredibly fast while being accurate to dozens of digits. "
@@ -262,7 +299,7 @@ void TextCell::SetValue(const wxString &text)
                     "that isn't a list.\n"
                     "Enclosing the new element for the list in brackets ([]) "
                     "converts it to a list and makes it appendable.");
-    if(text.StartsWith(wxT("part: invalid index of list or matrix.")))
+    if(text.Contains(wxT(": invalid index")))
       m_toolTip = _("The [] or the part() command tried to access a list or matrix "
                     "element that doesn't exist.");
     if(text.StartsWith(wxT("apply: subscript must be an integer; found:")))
@@ -271,7 +308,8 @@ void TextCell::SetValue(const wxString &text)
                     "something was used whose numerical value is unknown or not an "
                     "integer.\n"
                     "Floating-point numbers are bound to contain small rounding errors "
-                    "and aren't allowed as an array index.");
+                    "and therefore in most cases don't work as an array index that"
+                    "needs to be an exact integer number.");
     if(text.StartsWith(wxT(": improper argument: ")))
     {
       if((m_previous) && (m_previous->ToString() == wxT("at")))
@@ -306,67 +344,64 @@ Cell *TextCell::Copy()
   return retval;
 }
 
+bool TextCell::NeedsRecalculation()
+{
+  return Cell::NeedsRecalculation() ||
+    (
+      (m_textStyle == TS_USERLABEL) &&
+      (!(*m_configuration)->UseUserLabels())
+      ) ||
+    (
+      (m_textStyle == TS_LABEL) &&
+      ((*m_configuration)->UseUserLabels()) &&
+    (m_userDefinedLabel != wxEmptyString)
+      ) ||
+    (
+      (m_textStyle == TS_NUMBER) &&
+      (m_displayedDigits_old != (*m_configuration)->GetDisplayedDigits())
+      );
+}
+
 void TextCell::RecalculateWidths(int fontsize)
 {
   if(fontsize != m_fontsize_old)
     ResetSize();
   m_fontsize_old = fontsize;
-  Cell::RecalculateWidths(fontsize);
   Configuration *configuration = (*m_configuration);
-
-  bool recalculateNeeded = false;
-
-  if(m_lastZoomFactor != configuration->GetZoomFactor())
-  {
-    m_lastZoomFactor = configuration->GetZoomFactor();
-    recalculateNeeded = true;
-  }
   
-  // If the setting has changed and we want to show a user-defined label
-  // instead of an automatic one or vice versa we decide that here.
-  if(
-    (m_textStyle == TS_USERLABEL) &&
-    (!configuration->UseUserLabels())
-    )
-  {
-    m_textStyle = TS_LABEL;
-    recalculateNeeded = true;
-  }
-  if(
-    (m_textStyle == TS_LABEL) &&
-    (configuration->UseUserLabels()) &&
-    (m_userDefinedLabel != wxEmptyString)
-    )
-  {
-    m_textStyle = TS_USERLABEL;
-    recalculateNeeded = true;
-  }
-  
-  SetAltText();
-
-  // If the config settings about how many digits to display has changed we
-  // need to regenerate the info which number to show.
-  if (
-          (m_textStyle == TS_NUMBER) &&
-          (m_displayedDigits_old != (*m_configuration)->GetDisplayedDigits())
-          )
-  {
-    SetValue(m_text);
-    recalculateNeeded = true;
-  }
-
-  if (m_height == -1 || m_width == -1 || configuration->FontChanged())
-    recalculateNeeded = true;
-
-  if(recalculateNeeded)
+  if(NeedsRecalculation())
   {
+    // If the setting has changed and we want to show a user-defined label
+    // instead of an automatic one or vice versa we decide that here.
+    if(
+      (m_textStyle == TS_USERLABEL) &&
+      (!configuration->UseUserLabels())
+      )
+      m_textStyle = TS_LABEL;
+    if(
+      (m_textStyle == TS_LABEL) &&
+      (configuration->UseUserLabels()) &&
+      (m_userDefinedLabel != wxEmptyString)
+      )
+      m_textStyle = TS_USERLABEL;
+    
+    SetAltText();
+    
+    // If the config settings about how many digits to display has changed we
+    // need to regenerate the info which number to show.
+    if (
+      (m_textStyle == TS_NUMBER) &&
+      (m_displayedDigits_old != (*m_configuration)->GetDisplayedDigits())
+      )
+      SetValue(m_text);
+    
     m_lastCalculationFontSize = fontsize;
     wxDC *dc = configuration->GetDC();
     SetFont(fontsize);
 
     // Labels and prompts are fixed width - adjust font size so that
     // they fit in
-    if ((m_textStyle == TS_LABEL) || (m_textStyle == TS_USERLABEL) || (m_textStyle == TS_MAIN_PROMPT) || (m_textStyle == TS_OTHER_PROMPT))
+    if ((m_textStyle == TS_LABEL) || (m_textStyle == TS_USERLABEL) || (m_textStyle == TS_MAIN_PROMPT))
     {
       wxString text = m_text;
 
@@ -447,16 +482,16 @@ void TextCell::RecalculateWidths(int fontsize)
   }
   if(m_height < Scale_Px(4)) m_height = Scale_Px(4);
   m_realCenter = m_center = m_height / 2;
-  ResetData();
+  Cell::RecalculateWidths(fontsize);
 }
 
 void TextCell::Draw(wxPoint point)
 {
+  Cell::Draw(point);
   if (DrawThisCell(point) && !m_isHidden)
   {
     
     Configuration *configuration = (*m_configuration);
-    Cell::Draw(point);
     wxDC *dc = configuration->GetDC();
     
     if (m_width == -1 || m_height == -1 || m_fontSize != m_lastCalculationFontSize)
@@ -477,15 +512,20 @@ void TextCell::Draw(wxPoint point)
           if(m_textStyle == TS_USERLABEL)
           {
             wxString text = m_userDefinedLabel;
+            SetToolTip(m_text);
             m_unescapeRegEx.ReplaceAll(&text,wxT("\\1"));
             dc->DrawText(wxT("(") + text + wxT(")"),
                          point.x + MC_TEXT_PADDING,
                          point.y - m_realCenter + MC_TEXT_PADDING);
           }
           else
+          {
+//            SetToolTip(m_userDefinedLabel);
+            SetToolTip(wxEmptyString);
             dc->DrawText(m_displayedText,
                          point.x + MC_TEXT_PADDING,
                          point.y - m_realCenter + MC_TEXT_PADDING);
+          }
         }
       }
 
@@ -566,10 +606,15 @@ void TextCell::SetFont(int fontsize)
   {
     // Font within maths has a dynamic font size that might be reduced for example
     // within fractions, subscripts or superscripts.
-    if ((m_textStyle != TS_MAIN_PROMPT) && (m_textStyle != TS_OTHER_PROMPT))
+    if (
+      (m_textStyle != TS_MAIN_PROMPT) &&
+      (m_textStyle != TS_OTHER_PROMPT) &&
+      (m_textStyle != TS_ERROR) &&
+      (m_textStyle != TS_WARNING)
+      )
       m_fontSize = fontsize;
   }
-
+    
   wxFont font = configuration->GetFont(m_textStyle,fontsize);
 
   // Use jsMath
@@ -654,14 +699,17 @@ wxString TextCell::ToString()
     {
       // TODO: We could escape the - char inside a variable name.
       // But we get false positives, then.
-      wxString charsNeedingQuotes("\\'\"()[]{}^+*/&§?:;=#<>$");
+      wxString charsNeedingQuotes("\\'\"()[]-{}^+*/&§?:;=#<>$");
       bool isOperator = true;
-      for (size_t i = 0; i < m_text.Length(); i++)
+      if(m_text.Length() > 1)
       {
-        if ((m_text[i] == wxT(' ')) || (charsNeedingQuotes.Find(m_text[i]) == wxNOT_FOUND))
+        for (size_t i = 0; i < m_text.Length(); i++)
         {
-          isOperator = false;
-          break;
+          if ((m_text[i] == wxT(' ')) || (charsNeedingQuotes.Find(m_text[i]) == wxNOT_FOUND))
+          {
+            isOperator = false;
+            break;
+          }
         }
       }
 
@@ -698,12 +746,94 @@ wxString TextCell::ToString()
   default:
   {}
   }
-  if((m_next != NULL) && (m_next->BreakLine()))
+  if((m_next != NULL) && (m_next->BreakLineHere()))
     text += "\n";
   
   return text;
 }
 
+wxString TextCell::ToMatlab()
+{
+	wxString text;
+	if (m_altCopyText != wxEmptyString)
+	  text = m_altCopyText;
+	else
+	{
+	  text = m_text;
+	  if(((*m_configuration)->UseUserLabels())&&(m_userDefinedLabel != wxEmptyString))
+		text = wxT("(") + m_userDefinedLabel + wxT(")");
+	  text.Replace(wxT("\x2212"), wxT("-")); // unicode minus sign
+	  text.Replace(wxT("\x2794"), wxT("-->"));
+	  text.Replace(wxT("\x2192"), wxT("->"));
+
+	  if (text == wxT("%e"))
+		text = wxT("e");
+	  else if (text == wxT("%i"))
+		text = wxT("i");
+	  else if (text == wxT("%pi"))
+		text = wxString(wxT("pi"));
+	}
+	switch (m_textStyle)
+	{
+	  case TS_VARIABLE:
+	  case TS_FUNCTION:
+		// The only way for variable or function names to contain quotes and
+		// characters that clearly represent operators is that these chars
+		// are quoted by a backslash: They cannot be quoted by quotation
+		// marks since maxima would'nt allow strings here.
+	  {
+		// TODO: We could escape the - char inside a variable name.
+		// But we get false positives, then.
+		wxString charsNeedingQuotes("\\'\"()[]{}^+*/&§?:;=#<>$");
+		bool isOperator = true;
+		for (size_t i = 0; i < m_text.Length(); i++)
+		{
+		  if ((m_text[i] == wxT(' ')) || (charsNeedingQuotes.Find(m_text[i]) == wxNOT_FOUND))
+		  {
+			isOperator = false;
+			break;
+		  }
+		}
+
+		if (!isOperator)
+		{
+		  wxString lastChar;
+		  if ((m_dontEscapeOpeningParenthesis) && (text.Length() > 0) && (text[text.Length() - 1] == wxT('(')))
+		  {
+			lastChar = text[text.Length() - 1];
+			text = text.Left(text.Length() - 1);
+		  }
+		  for (size_t i = 0; i < charsNeedingQuotes.Length(); i++)
+			text.Replace(charsNeedingQuotes[i], wxT("\\") + wxString(charsNeedingQuotes[i]));
+		  text += lastChar;
+		}
+		break;
+	  }
+	  case TS_STRING:
+		text = wxT("\"") + text + wxT("\"");
+		break;
+
+		// Labels sometimes end with a few spaces. But if they are long they don't do
+		// that any more => Add a TAB to the end of any label replacing trailing
+		// whitespace. But don't do this if we copy only the label.
+	  case TS_LABEL:
+	  case TS_USERLABEL:
+	  case TS_MAIN_PROMPT:
+	  case TS_OTHER_PROMPT:
+		{
+		  text.Trim();
+		  text += wxT("\t");
+		  break;
+		}
+	default:
+	{}
+	}
+	if((m_next != NULL) && (m_next->BreakLineHere()))
+	  text += "\n";
+
+	return text;
+}
+
 wxString TextCell::ToTeX()
 {
   wxString text = m_displayedText;
@@ -768,96 +898,96 @@ wxString TextCell::ToTeX()
   text.Replace(wxT("<"), mathModeStart + wxT("<") + mathModeEnd);
   text.Replace(wxT(">"), mathModeStart + wxT(">") + mathModeEnd);
   text.Replace(wxT("\x2212"), wxT("-")); // unicode minus sign
-  text.Replace(L"\x00B1", mathModeStart + wxT("\\pm") + mathModeEnd);
-  text.Replace(L"\x03B1", mathModeStart + wxT("\\alpha") + mathModeEnd);
-  text.Replace(L"\x00B2", mathModeStart + wxT("^2") + mathModeEnd);
-  text.Replace(L"\x00B3", mathModeStart + wxT("^3") + mathModeEnd);
-  text.Replace(L"\x221A", mathModeStart + wxT("\\sqrt{}") + mathModeEnd);
-  text.Replace(L"\x2148", mathModeStart + wxT("\\mathbbm{i}") + mathModeEnd);
-  text.Replace(L"\x2147", mathModeStart + wxT("\\mathbbm{e}") + mathModeEnd);
-  text.Replace(L"\x210f", mathModeStart + wxT("\\hbar") + mathModeEnd);
-  text.Replace(L"\x2203", mathModeStart + wxT("\\exists") + mathModeEnd);
-  text.Replace(L"\x2204", mathModeStart + wxT("\\nexists") + mathModeEnd);
-  text.Replace(L"\x2208", mathModeStart + wxT("\\in") + mathModeEnd);
-  text.Replace(L"\x21D2", mathModeStart + wxT("\\Longrightarrow") + mathModeEnd);
-  text.Replace(L"\x221e", mathModeStart + wxT("\\infty") + mathModeEnd);
-  text.Replace(L"\x22C0", mathModeStart + wxT("\\wedge") + mathModeEnd);
-  text.Replace(L"\x22C1", mathModeStart + wxT("\\vee") + mathModeEnd);
-  text.Replace(L"\x22bb", mathModeStart + wxT("\\oplus") + mathModeEnd);
-  text.Replace(L"\x22BC", mathModeStart + wxT("\\overline{\\wedge}") + mathModeEnd);
-  text.Replace(L"\x22BB", mathModeStart + wxT("\\overline{\\vee}") + mathModeEnd);
-  text.Replace(L"\x00AC", mathModeStart + wxT("\\setminus") + mathModeEnd);
-  text.Replace(L"\x22C3", mathModeStart + wxT("\\cup") + mathModeEnd);
-  text.Replace(L"\x22C2", mathModeStart + wxT("\\cap") + mathModeEnd);
-  text.Replace(L"\x2286", mathModeStart + wxT("\\subseteq") + mathModeEnd);
-  text.Replace(L"\x2282", mathModeStart + wxT("\\subset") + mathModeEnd);
-  text.Replace(L"\x2288", mathModeStart + wxT("\\not\\subseteq") + mathModeEnd);
-  text.Replace(L"\x0127", mathModeStart + wxT("\\hbar") + mathModeEnd);
-  text.Replace(L"\x0126", mathModeStart + wxT("\\Hbar") + mathModeEnd);
-  text.Replace(L"\x2205", mathModeStart + wxT("\\emptyset") + mathModeEnd);
-  text.Replace(L"\x00BD", mathModeStart + wxT("\\frac{1}{2}") + mathModeEnd);
-  text.Replace(L"\x03B2", mathModeStart + wxT("\\beta") + mathModeEnd);
-  text.Replace(L"\x03B3", mathModeStart + wxT("\\gamma") + mathModeEnd);
-  text.Replace(L"\x03B4", mathModeStart + wxT("\\delta") + mathModeEnd);
-  text.Replace(L"\x03B5", mathModeStart + wxT("\\epsilon") + mathModeEnd);
-  text.Replace(L"\x03B6", mathModeStart + wxT("\\zeta") + mathModeEnd);
-  text.Replace(L"\x03B7", mathModeStart + wxT("\\eta") + mathModeEnd);
-  text.Replace(L"\x03B8", mathModeStart + wxT("\\theta") + mathModeEnd);
-  text.Replace(L"\x03B9", mathModeStart + wxT("\\iota") + mathModeEnd);
-  text.Replace(L"\x03BA", mathModeStart + wxT("\\kappa") + mathModeEnd);
-  text.Replace(L"\x03BB", mathModeStart + wxT("\\lambda") + mathModeEnd);
-  text.Replace(L"\x03BC", mathModeStart + wxT("\\mu") + mathModeEnd);
-  text.Replace(L"\x03BD", mathModeStart + wxT("\\nu") + mathModeEnd);
-  text.Replace(L"\x03BE", mathModeStart + wxT("\\xi") + mathModeEnd);
-  text.Replace(L"\x03BF", mathModeStart + wxT("\\omicron") + mathModeEnd);
-  text.Replace(L"\x03C0", mathModeStart + wxT("\\pi") + mathModeEnd);
-  text.Replace(L"\x03C1", mathModeStart + wxT("\\rho") + mathModeEnd);
-  text.Replace(L"\x03C3", mathModeStart + wxT("\\sigma") + mathModeEnd);
-  text.Replace(L"\x03C4", mathModeStart + wxT("\\tau") + mathModeEnd);
-  text.Replace(L"\x03C5", mathModeStart + wxT("\\upsilon") + mathModeEnd);
-  text.Replace(L"\x03C6", mathModeStart + wxT("\\phi") + mathModeEnd);
-  text.Replace(L"\x03C7", mathModeStart + wxT("\\chi") + mathModeEnd);
-  text.Replace(L"\x03C8", mathModeStart + wxT("\\psi") + mathModeEnd);
-  text.Replace(L"\x03C9", mathModeStart + wxT("\\omega") + mathModeEnd);
-  text.Replace(L"\x0391", mathModeStart + wxT("\\Alpha") + mathModeEnd);
-  text.Replace(L"\x0392", mathModeStart + wxT("\\Beta") + mathModeEnd);
-  text.Replace(L"\x0393", mathModeStart + wxT("\\Gamma") + mathModeEnd);
-  text.Replace(L"\x0394", mathModeStart + wxT("\\Delta") + mathModeEnd);
-  text.Replace(L"\x0395", mathModeStart + wxT("\\Epsilon") + mathModeEnd);
-  text.Replace(L"\x0396", mathModeStart + wxT("\\Zeta") + mathModeEnd);
-  text.Replace(L"\x0397", mathModeStart + wxT("\\Eta") + mathModeEnd);
-  text.Replace(L"\x0398", mathModeStart + wxT("\\Theta") + mathModeEnd);
-  text.Replace(L"\x0399", mathModeStart + wxT("\\Iota") + mathModeEnd);
-  text.Replace(L"\x039A", mathModeStart + wxT("\\Kappa") + mathModeEnd);
-  text.Replace(L"\x039B", mathModeStart + wxT("\\Lambda") + mathModeEnd);
-  text.Replace(L"\x039C", mathModeStart + wxT("\\Mu") + mathModeEnd);
-  text.Replace(L"\x039D", mathModeStart + wxT("\\Nu") + mathModeEnd);
-  text.Replace(L"\x039E", mathModeStart + wxT("\\Xi") + mathModeEnd);
-  text.Replace(L"\x039F", mathModeStart + wxT("\\Omicron") + mathModeEnd);
-  text.Replace(L"\x03A0", mathModeStart + wxT("\\Pi") + mathModeEnd);
-  text.Replace(L"\x03A1", mathModeStart + wxT("\\Rho") + mathModeEnd);
-  text.Replace(L"\x03A3", mathModeStart + wxT("\\Sigma") + mathModeEnd);
-  text.Replace(L"\x03A4", mathModeStart + wxT("\\Tau") + mathModeEnd);
-  text.Replace(L"\x03A5", mathModeStart + wxT("\\Upsilon") + mathModeEnd);
-  text.Replace(L"\x03A6", mathModeStart + wxT("\\Phi") + mathModeEnd);
-  text.Replace(L"\x03A7", mathModeStart + wxT("\\Chi") + mathModeEnd);
-  text.Replace(L"\x03A8", mathModeStart + wxT("\\Psi") + mathModeEnd);
-  text.Replace(L"\x03A9", mathModeStart + wxT("\\Omega") + mathModeEnd);
-  text.Replace(L"\x2202", mathModeStart + wxT("\\partial") + mathModeEnd);
-  text.Replace(L"\x222b", mathModeStart + wxT("\\int") + mathModeEnd);
-  text.Replace(L"\x2245", mathModeStart + wxT("\\approx") + mathModeEnd);
-  text.Replace(L"\x221d", mathModeStart + wxT("\\propto") + mathModeEnd);
-  text.Replace(L"\x2260", mathModeStart + wxT("\\neq") + mathModeEnd);
-  text.Replace(L"\x2264", mathModeStart + wxT("\\leq") + mathModeEnd);
-  text.Replace(L"\x2265", mathModeStart + wxT("\\geq") + mathModeEnd);
-  text.Replace(L"\x226A", mathModeStart + wxT("\\ll") + mathModeEnd);
-  text.Replace(L"\x226B", mathModeStart + wxT("\\gg") + mathModeEnd);
-  text.Replace(L"\x220e", mathModeStart + wxT("\\blacksquare") + mathModeEnd);
-  text.Replace(L"\x2263", mathModeStart + wxT("\\equiv") + mathModeEnd);
-  text.Replace(L"\x2211", mathModeStart + wxT("\\sum") + mathModeEnd);
-  text.Replace(L"\x220F", mathModeStart + wxT("\\prod") + mathModeEnd);
-  text.Replace(L"\x2225", mathModeStart + wxT("\\parallel") + mathModeEnd);
-  text.Replace(L"\x27C2", mathModeStart + wxT("\\bot") + mathModeEnd);
+  text.Replace(wxT("\x00B1"), mathModeStart + wxT("\\pm") + mathModeEnd);
+  text.Replace(wxT("\x03B1"), mathModeStart + wxT("\\alpha") + mathModeEnd);
+  text.Replace(wxT("\x00B2"), mathModeStart + wxT("^2") + mathModeEnd);
+  text.Replace(wxT("\x00B3"), mathModeStart + wxT("^3") + mathModeEnd);
+  text.Replace(wxT("\x221A"), mathModeStart + wxT("\\sqrt{}") + mathModeEnd);
+  text.Replace(wxT("\x2148"), mathModeStart + wxT("\\mathbbm{i}") + mathModeEnd);
+  text.Replace(wxT("\x2147"), mathModeStart + wxT("\\mathbbm{e}") + mathModeEnd);
+  text.Replace(wxT("\x210f"), mathModeStart + wxT("\\hbar") + mathModeEnd);
+  text.Replace(wxT("\x2203"), mathModeStart + wxT("\\exists") + mathModeEnd);
+  text.Replace(wxT("\x2204"), mathModeStart + wxT("\\nexists") + mathModeEnd);
+  text.Replace(wxT("\x2208"), mathModeStart + wxT("\\in") + mathModeEnd);
+  text.Replace(wxT("\x21D2"), mathModeStart + wxT("\\Longrightarrow") + mathModeEnd);
+  text.Replace(wxT("\x221e"), mathModeStart + wxT("\\infty") + mathModeEnd);
+  text.Replace(wxT("\x22C0"), mathModeStart + wxT("\\wedge") + mathModeEnd);
+  text.Replace(wxT("\x22C1"), mathModeStart + wxT("\\vee") + mathModeEnd);
+  text.Replace(wxT("\x22bb"), mathModeStart + wxT("\\oplus") + mathModeEnd);
+  text.Replace(wxT("\x22BC"), mathModeStart + wxT("\\overline{\\wedge}") + mathModeEnd);
+  text.Replace(wxT("\x22BB"), mathModeStart + wxT("\\overline{\\vee}") + mathModeEnd);
+  text.Replace(wxT("\x00AC"), mathModeStart + wxT("\\setminus") + mathModeEnd);
+  text.Replace(wxT("\x22C3"), mathModeStart + wxT("\\cup") + mathModeEnd);
+  text.Replace(wxT("\x22C2"), mathModeStart + wxT("\\cap") + mathModeEnd);
+  text.Replace(wxT("\x2286"), mathModeStart + wxT("\\subseteq") + mathModeEnd);
+  text.Replace(wxT("\x2282"), mathModeStart + wxT("\\subset") + mathModeEnd);
+  text.Replace(wxT("\x2288"), mathModeStart + wxT("\\not\\subseteq") + mathModeEnd);
+  text.Replace(wxT("\x0127"), mathModeStart + wxT("\\hbar") + mathModeEnd);
+  text.Replace(wxT("\x0126"), mathModeStart + wxT("\\Hbar") + mathModeEnd);
+  text.Replace(wxT("\x2205"), mathModeStart + wxT("\\emptyset") + mathModeEnd);
+  text.Replace(wxT("\x00BD"), mathModeStart + wxT("\\frac{1}{2}") + mathModeEnd);
+  text.Replace(wxT("\x03B2"), mathModeStart + wxT("\\beta") + mathModeEnd);
+  text.Replace(wxT("\x03B3"), mathModeStart + wxT("\\gamma") + mathModeEnd);
+  text.Replace(wxT("\x03B4"), mathModeStart + wxT("\\delta") + mathModeEnd);
+  text.Replace(wxT("\x03B5"), mathModeStart + wxT("\\epsilon") + mathModeEnd);
+  text.Replace(wxT("\x03B6"), mathModeStart + wxT("\\zeta") + mathModeEnd);
+  text.Replace(wxT("\x03B7"), mathModeStart + wxT("\\eta") + mathModeEnd);
+  text.Replace(wxT("\x03B8"), mathModeStart + wxT("\\theta") + mathModeEnd);
+  text.Replace(wxT("\x03B9"), mathModeStart + wxT("\\iota") + mathModeEnd);
+  text.Replace(wxT("\x03BA"), mathModeStart + wxT("\\kappa") + mathModeEnd);
+  text.Replace(wxT("\x03BB"), mathModeStart + wxT("\\lambda") + mathModeEnd);
+  text.Replace(wxT("\x03BC"), mathModeStart + wxT("\\mu") + mathModeEnd);
+  text.Replace(wxT("\x03BD"), mathModeStart + wxT("\\nu") + mathModeEnd);
+  text.Replace(wxT("\x03BE"), mathModeStart + wxT("\\xi") + mathModeEnd);
+  text.Replace(wxT("\x03BF"), wxT("o"));
+  text.Replace(wxT("\x03C0"), mathModeStart + wxT("\\pi") + mathModeEnd);
+  text.Replace(wxT("\x03C1"), mathModeStart + wxT("\\rho") + mathModeEnd);
+  text.Replace(wxT("\x03C3"), mathModeStart + wxT("\\sigma") + mathModeEnd);
+  text.Replace(wxT("\x03C4"), mathModeStart + wxT("\\tau") + mathModeEnd);
+  text.Replace(wxT("\x03C5"), mathModeStart + wxT("\\upsilon") + mathModeEnd);
+  text.Replace(wxT("\x03C6"), mathModeStart + wxT("\\phi") + mathModeEnd);
+  text.Replace(wxT("\x03C7"), mathModeStart + wxT("\\chi") + mathModeEnd);
+  text.Replace(wxT("\x03C8"), mathModeStart + wxT("\\psi") + mathModeEnd);
+  text.Replace(wxT("\x03C9"), mathModeStart + wxT("\\omega") + mathModeEnd);
+  text.Replace(wxT("\x0391"), wxT("A"));
+  text.Replace(wxT("\x0392"), wxT("B"));
+  text.Replace(wxT("\x0393"), mathModeStart + wxT("\\Gamma") + mathModeEnd);
+  text.Replace(wxT("\x0394"), mathModeStart + wxT("\\Delta") + mathModeEnd);
+  text.Replace(wxT("\x0395"), wxT("E"));
+  text.Replace(wxT("\x0396"), wxT("Z"));
+  text.Replace(wxT("\x0397"), wxT("H"));
+  text.Replace(wxT("\x0398"), mathModeStart + wxT("\\Theta") + mathModeEnd);
+  text.Replace(wxT("\x0399"), wxT("I"));
+  text.Replace(wxT("\x039A"), wxT("K"));
+  text.Replace(wxT("\x039B"), mathModeStart + wxT("\\Lambda") + mathModeEnd);
+  text.Replace(wxT("\x039C"), wxT("M"));
+  text.Replace(wxT("\x039D"), wxT("N"));
+  text.Replace(wxT("\x039E"), mathModeStart + wxT("\\Xi") + mathModeEnd);
+  text.Replace(wxT("\x039F"), wxT("O"));
+  text.Replace(wxT("\x03A0"), mathModeStart + wxT("\\Pi") + mathModeEnd);
+  text.Replace(wxT("\x03A1"), wxT("P"));
+  text.Replace(wxT("\x03A3"), mathModeStart + wxT("\\Sigma") + mathModeEnd);
+  text.Replace(wxT("\x03A4"), wxT("T"));
+  text.Replace(wxT("\x03A5"), mathModeStart + wxT("\\Upsilon") + mathModeEnd);
+  text.Replace(wxT("\x03A6"), mathModeStart + wxT("\\Phi") + mathModeEnd);
+  text.Replace(wxT("\x03A7"), wxT("X"));
+  text.Replace(wxT("\x03A8"), mathModeStart + wxT("\\Psi") + mathModeEnd);
+  text.Replace(wxT("\x03A9"), mathModeStart + wxT("\\Omega") + mathModeEnd);
+  text.Replace(wxT("\x2202"), mathModeStart + wxT("\\partial") + mathModeEnd);
+  text.Replace(wxT("\x222b"), mathModeStart + wxT("\\int") + mathModeEnd);
+  text.Replace(wxT("\x2245"), mathModeStart + wxT("\\approx") + mathModeEnd);
+  text.Replace(wxT("\x221d"), mathModeStart + wxT("\\propto") + mathModeEnd);
+  text.Replace(wxT("\x2260"), mathModeStart + wxT("\\neq") + mathModeEnd);
+  text.Replace(wxT("\x2264"), mathModeStart + wxT("\\leq") + mathModeEnd);
+  text.Replace(wxT("\x2265"), mathModeStart + wxT("\\geq") + mathModeEnd);
+  text.Replace(wxT("\x226A"), mathModeStart + wxT("\\ll") + mathModeEnd);
+  text.Replace(wxT("\x226B"), mathModeStart + wxT("\\gg") + mathModeEnd);
+  text.Replace(wxT("\x220e"), mathModeStart + wxT("\\blacksquare") + mathModeEnd);
+  text.Replace(wxT("\x2263"), mathModeStart + wxT("\\equiv") + mathModeEnd);
+  text.Replace(wxT("\x2211"), mathModeStart + wxT("\\sum") + mathModeEnd);
+  text.Replace(wxT("\x220F"), mathModeStart + wxT("\\prod") + mathModeEnd);
+  text.Replace(wxT("\x2225"), mathModeStart + wxT("\\parallel") + mathModeEnd);
+  text.Replace(wxT("\x27C2"), mathModeStart + wxT("\\bot") + mathModeEnd);
   text.Replace(wxT("~"), mathModeStart + wxT("\\sim ") + mathModeEnd);
   text.Replace(wxT("_"), wxT("\\_ "));
   text.Replace(wxT("$"), wxT("\\$ "));
@@ -987,9 +1117,9 @@ wxString TextCell::ToTeX()
     else if (text == wxT("\\% omega"))
       return wxT("\\omega ");
     else if (text == wxT("\\% Alpha"))
-      return wxT("\\Alpha ");
+      return wxT("A");
     else if (text == wxT("\\% Beta"))
-      return wxT("\\Beta ");
+      return wxT("B");
     else if (text == wxT("\\% Gamma"))
       return wxT("\\Gamma ");
     else if (text == wxT("\\% Delta"))
@@ -1207,8 +1337,8 @@ wxString TextCell::ToOMML()
 {
   //Text-only lines are better handled in RTF.
   if (
-          ((m_previous != NULL) && (m_previous->GetStyle() != TS_LABEL) && (!m_previous->ForceBreakLineHere())) &&
-          (ForceBreakLineHere())
+          ((m_previous != NULL) && (m_previous->GetStyle() != TS_LABEL) && (!m_previous->HardLineBreak())) &&
+          (HardLineBreak())
           )
     return wxEmptyString;
 
@@ -1249,9 +1379,7 @@ wxString TextCell::ToOMML()
     /* FALLTHRU */
   case TS_VARIABLE:
     {
-      bool keepPercent = (*m_configuration)->CheckKeepPercent();
-
-      if (!keepPercent)
+      if (!(*m_configuration)->CheckKeepPercent())
       {
         if (text == wxT("%pi"))
           text = wxT("\x03C0");
@@ -1276,7 +1404,7 @@ wxString TextCell::ToOMML()
     {
     }
   }
-  text = wxT("<m:t>") + text + wxT("</m:t>\n");
+  text = wxT("<m:r>") + text + wxT("</m:r>\n");
   return text;
 }
 
@@ -1523,7 +1651,7 @@ wxString TextCell::GetGreekStringUnicode()
   else if (txt == wxT("%Omega"))
     return wxT("\x03A9");
 
-  return wxEmptyString;
+  return m_text;
 }
 
 wxString TextCell::GetSymbolUnicode(bool keepPercent)
