@@ -37,7 +37,7 @@ private:
   //! Is an ending "(" of a function name the opening parenthesis of the function?
   bool m_dontEscapeOpeningParenthesis;
 public:
-  TextCell(Cell *parent, Configuration **config, CellPointers *cellPointers, wxString text = wxEmptyString);
+  TextCell(Cell *parent, Configuration **config, CellPointers *cellPointers, wxString text = wxEmptyString, TextStyle style = TS_DEFAULT);
 
   std::list<Cell *> GetInnerCells();
   
@@ -68,6 +68,8 @@ public:
 
   wxString ToString();
 
+  wxString ToMatlab();
+
   wxString ToTeX();
 
   wxString ToMathML();
@@ -94,13 +96,16 @@ public:
   wxString GetSymbolUnicode(bool keepPercent);
 
   bool IsShortNum();
-  
+
+  virtual void SetType(CellType type);
+
 protected:
   void SetAltText();
   
   //! Resets the font size to label size
   void SetFontSizeForLabel(wxDC *dc);
 
+  bool NeedsRecalculation();
   static wxRegEx m_unescapeRegEx;
   static wxRegEx m_roundingErrorRegEx1;
   static wxRegEx m_roundingErrorRegEx2;
