@@ -28,14 +28,13 @@
 #include "TipOfTheDay.h"
 #include <wx/config.h>
 #include <wx/mstream.h>
+#include <wx/display.h>
 #include <wx/wfstream.h>
 #include <wx/persist.h>
 #include <wx/persist/toplevel.h>
 
 #define ICON_SCALE (0.35)
 #define ABS(val) ((val) >= 0 ? (val) : -(val))
-#define MAX(a, b) ((a)>(b) ? (a) : (b))
-#define MIN(a, b) ((a)>(b) ? (b) : (a))
 
 TipOfTheDay::TipOfTheDay(wxWindow *parent)
   : wxDialog(parent,-1,_("Tip of the Day"))
@@ -256,7 +255,24 @@ TipOfTheDay::~TipOfTheDay()
 wxImage TipOfTheDay::GetImage(unsigned char *data_128, size_t len_128,
                               unsigned char *data_192, size_t len_192)
 {
-  double targetSize = wxGetDisplayPPI().x * ICON_SCALE;
+  int ppi;
+#if wxCHECK_VERSION(3, 1, 1)
+  wxDisplay display;
+  
+  int display_idx = wxDisplay::GetFromWindow(GetParent());
+  if (display_idx < 0)
+    ppi = 72;
+  else
+    ppi = wxDisplay(display_idx).GetPPI().x;
+#else
+  ppi = wxGetDisplayPPI().x;
+#endif
+  if(ppi < 10)
+    ppi = wxGetDisplayPPI().x;
+  if(ppi <= 10)
+    ppi = 72;
+  
+  double targetSize = wxMax(ppi,75) * ICON_SCALE;
   int prescale;
 
   int sizeA = 128 << 4;
