@@ -31,21 +31,41 @@
 #include <wx/mstream.h>
 #include <wx/wfstream.h>
 #include "GroupCell.h"
+#include <wx/display.h>
 #include <wx/artprov.h>
 #include <wx/filename.h>
 #include "invalidImage.h"
 
+#if wxCHECK_VERSION(3, 1, 0)
+#define TOOLBAR_ICON_SCALE (0.25)
+#else
 #define TOOLBAR_ICON_SCALE (0.35)
+#endif
 
 #define ABS(val) ((val) >= 0 ? (val) : -(val))
-#define MAX(a, b) ((a)>(b) ? (a) : (b))
-#define MIN(a, b) ((a)>(b) ? (b) : (a))
 
 wxBitmap ToolBar::GetImage(wxString name,
                           unsigned char *data_128, size_t len_128,
                           unsigned char *data_192, size_t len_192)
 {
-  double targetSize = wxGetDisplayPPI().x * TOOLBAR_ICON_SCALE * GetContentScaleFactor();
+#if wxCHECK_VERSION(3, 1, 1)
+  wxDisplay display;
+  
+  int display_idx = wxDisplay::GetFromWindow(GetParent());
+  if (display_idx < 0)
+    m_ppi = wxSize(72,72);
+  else
+    m_ppi = wxDisplay(display_idx).GetPPI();
+#else
+  m_ppi = wxGetDisplayPPI();
+#endif
+  if((m_ppi.x <= 10) || (m_ppi.y <= 10))
+    m_ppi = wxGetDisplayPPI();
+
+  if((m_ppi.x <= 10) || (m_ppi.y <= 10))
+    m_ppi = wxSize(72,72);
+
+  double targetSize = wxMax(m_ppi.x,75) * TOOLBAR_ICON_SCALE * GetContentScaleFactor();
   int prescale;
 
   int sizeA = 128 << 4;
@@ -93,7 +113,7 @@ wxBitmap ToolBar::GetImage(wxString name,
   }
 
   img.Rescale(targetSize, targetSize, wxIMAGE_QUALITY_HIGH);
-#if defined __WXMAC__
+#if defined __WXOSX__
   return wxBitmap(img,wxBITMAP_SCREEN_DEPTH,GetContentScaleFactor());
 #else
   return wxBitmap(img,wxBITMAP_SCREEN_DEPTH);
@@ -129,6 +149,7 @@ ToolBar::ToolBar(wxWindow *parent) : wxAuiToolBar(parent,-1, wxDefaultPosition,
                                                   wxAUI_TB_OVERFLOW | wxAUI_TB_PLAIN_BACKGROUND |
                                                   wxAUI_TB_HORIZONTAL)
 {
+  m_ppi = wxSize(-1,-1);
   m_defaultCellStyle = GC_TYPE_CODE;
   m_canCopy_old = true;
   m_canCut_old = true;
@@ -160,7 +181,7 @@ ToolBar::ToolBar(wxWindow *parent) : wxAuiToolBar(parent,-1, wxDefaultPosition,
                               gtk_save_192_png,gtk_save_192_png_len
                        ),
                      _("Save document"));
-#ifndef __WXMAC__
+#ifndef __WXOSX__
   AddSeparator();
 #endif
   AddTool(tb_print, _("Print"),
@@ -175,7 +196,7 @@ ToolBar::ToolBar(wxWindow *parent) : wxAuiToolBar(parent,-1, wxDefaultPosition,
                               gtk_preferences_192_png,gtk_preferences_192_png_len
                        ),
                      _("Configure wxMaxima"));
-#ifndef __WXMAC__
+#ifndef __WXOSX__
   AddSeparator();
 #endif
   AddTool(tb_cut, _("Cut"),
@@ -202,7 +223,7 @@ ToolBar::ToolBar(wxWindow *parent) : wxAuiToolBar(parent,-1, wxDefaultPosition,
                               gtk_select_all_192_png,gtk_select_all_192_png_len
                        ),
                      _("Select all"));
-#ifndef __WXMAC__
+#ifndef __WXOSX__
   AddSeparator();
 #endif
   AddTool(tb_find, _("Find"),
@@ -211,7 +232,7 @@ ToolBar::ToolBar(wxWindow *parent) : wxAuiToolBar(parent,-1, wxDefaultPosition,
                               gtk_find_192_png,gtk_find_192_png_len
                        ),
                      _("Find and replace"));
-#ifndef __WXMAC__
+#ifndef __WXOSX__
   AddSeparator();
 #endif
   AddTool(menu_restart_id, _("Restart maxima"),
@@ -252,7 +273,7 @@ ToolBar::ToolBar(wxWindow *parent) : wxAuiToolBar(parent,-1, wxDefaultPosition,
                        ),
                      _("Evaluate the file from the cursor to its end"));
 
-#ifndef __WXMAC__
+#ifndef __WXOSX__
     AddSeparator();
 #endif
     AddTool(tb_hideCode, _("Hide Code"),
@@ -261,7 +282,7 @@ ToolBar::ToolBar(wxWindow *parent) : wxAuiToolBar(parent,-1, wxDefaultPosition,
                      weather_few_clouds_192_png,weather_few_clouds_192_png_len
               ),
             _("Toggle the visibility of code cells"));
-#ifndef __WXMAC__
+#ifndef __WXOSX__
   AddSeparator();
 #endif
   wxArrayString textStyle;
@@ -305,11 +326,27 @@ ToolBar::ToolBar(wxWindow *parent) : wxAuiToolBar(parent,-1, wxDefaultPosition,
                      _("Start or stop the currently selected animation that has been created with the with_slider class of commands"));
   EnableTool(tb_animation_startStop, false);
 
-  int sliderWidth = wxGetDisplayPPI().x * 200 / 72;
+#if wxCHECK_VERSION(3, 1, 1)
+  wxDisplay display;
+  
+  int display_idx = wxDisplay::GetFromWindow(GetParent());
+  if (display_idx < 0)
+    m_ppi = wxSize(72,72);
+  else
+    m_ppi = wxDisplay(display_idx).GetPPI();
+#else
+  m_ppi = wxGetDisplayPPI();
+#endif
+  if((m_ppi.x <= 10) || (m_ppi.y <= 10))
+    m_ppi = wxGetDisplayPPI();
+  if((m_ppi.x <= 10) || (m_ppi.y <= 10))
+    m_ppi = wxSize(72,72);
+
+  int sliderWidth = wxMax(m_ppi.x,75) * 200 / 72;
   int width, height;
   wxDisplaySize(&width, &height);
   if (width < 800)
-    sliderWidth = MIN(sliderWidth, 100);
+    sliderWidth = wxMin(sliderWidth, 100);
   m_plotSlider = new wxSlider(this, plot_slider_id, 0, 0, 10,
                               wxDefaultPosition, wxSize(sliderWidth, -1),
                               wxSL_HORIZONTAL | !wxSL_AUTOTICKS);
@@ -319,7 +356,7 @@ ToolBar::ToolBar(wxWindow *parent) : wxAuiToolBar(parent,-1, wxDefaultPosition,
   m_slideShowMaxIndex = -1;
   m_slideShowDisplayedIndex = -1;
   AddControl(m_plotSlider);
-  AddStretchSpacer();
+  AddStretchSpacer(100);
   AddTool(tb_help, _("Help"),
                      GetImage(wxT("gtk-help"),
                               gtk_help_128_png,gtk_help_128_png_len,
@@ -330,7 +367,128 @@ ToolBar::ToolBar(wxWindow *parent) : wxAuiToolBar(parent,-1, wxDefaultPosition,
           wxSizeEventHandler(ToolBar::OnSize),
           NULL, this);
   Realize();
-  SetInitialSize(wxSize(GetBestSize().x,100000));
+  SetInitialSize(wxSize(100000,GetBestSize().y));
+  Realize();
+}
+
+void ToolBar::UpdateBitmaps()
+{
+  wxSize ppi(-1,-1);
+#if wxCHECK_VERSION(3, 1, 1)
+  wxDisplay display;
+  
+  int display_idx = wxDisplay::GetFromWindow(GetParent());
+  if (display_idx < 0)
+    ppi = wxSize(72,72);
+  else
+    ppi = wxDisplay(display_idx).GetPPI();
+#endif
+  
+  if((ppi.x <= 10) || (ppi.y <= 10))
+    ppi = wxGetDisplayPPI();
+
+  if((ppi.x <= 10) || (ppi.y <= 10))
+    ppi = wxSize(72,72);
+
+  if((ppi.x == m_ppi.x) && (ppi.y == m_ppi.y))
+    return;
+
+  wxLogMessage(
+    wxString::Format(
+      _("Display resolution according to wxWidgets: %i x %i ppi"),
+      ppi.x,
+      ppi.y)
+    );
+
+  if(ppi.x == 0)
+    return;
+
+  m_ppi = ppi;
+  
+  SetToolBitmap(tb_new,GetImage(wxT("gtk-new"),
+                                gtk_new_128_png,gtk_new_128_png_len,
+                                gtk_new_192_png,gtk_new_192_png_len));
+  SetToolBitmap(tb_open,GetImage(wxT("gtk-open"),
+                                 gtk_open_128_png,gtk_open_128_png_len,
+                                 gtk_open_192_png,gtk_open_192_png_len));
+  SetToolBitmap(tb_save,GetImage(wxT("gtk-save"),
+                                 gtk_save_128_png,gtk_save_128_png_len,
+                                 gtk_save_192_png,gtk_save_192_png_len
+                  ));
+  SetToolBitmap(tb_print,GetImage(wxT("gtk-print"),
+                                  gtk_print_128_png,gtk_print_128_png_len,
+                                  gtk_print_192_png,gtk_print_192_png_len
+                  ));
+  SetToolBitmap(tb_pref,GetImage(wxT("gtk-preferences"),
+                                 gtk_preferences_128_png,gtk_preferences_128_png_len,
+                                 gtk_preferences_192_png,gtk_preferences_192_png_len
+                  ));
+  SetToolBitmap(tb_cut,GetImage(wxT("gtk-cut"),
+                                gtk_cut_128_png,gtk_cut_128_png_len,
+                                gtk_cut_192_png,gtk_cut_192_png_len
+                  ));
+  SetToolBitmap(tb_copy,GetImage(wxT("gtk-copy"),
+                                 gtk_copy_128_png,gtk_copy_128_png_len,
+                                 gtk_copy_192_png,gtk_copy_192_png_len
+                  ));
+  SetToolBitmap(tb_paste,GetImage(wxT("gtk-paste"),
+                                  gtk_paste_128_png,gtk_paste_128_png_len,
+                                  gtk_paste_192_png,gtk_paste_192_png_len
+                  ));
+  SetToolBitmap(tb_select_all,
+                GetImage(wxT("gtk-select-all"),
+                         gtk_select_all_128_png,gtk_select_all_128_png_len,
+                         gtk_select_all_192_png,gtk_select_all_192_png_len
+                  ));
+  SetToolBitmap(tb_find,GetImage(wxT("gtk-find"),
+                                 gtk_find_128_png,gtk_find_128_png_len,
+                                 gtk_find_192_png,gtk_find_192_png_len
+                  ));
+  SetToolBitmap(menu_restart_id,GetImage(wxT("view-refresh"),
+                                         view_refresh_128_png,view_refresh_128_png_len,
+                                         view_refresh_192_png,view_refresh_192_png_len
+                  ));
+  SetToolBitmap(tb_interrupt,
+                GetImage(wxT("gtk-stop"),
+                         gtk_stop_128_png,gtk_stop_128_png_len,
+                         gtk_stop_192_png,gtk_stop_192_png_len
+                  ));
+  m_followIcon = GetImage(wxT("weather-clear"),
+                          weather_clear_128_png,weather_clear_128_png_len,
+                          weather_clear_192_png,weather_clear_192_png_len
+    );
+  m_needsInformationIcon = GetImage(wxT("software-update-urgent"),
+                                    software_update_urgent_128_png,software_update_urgent_128_png_len,
+                                    software_update_urgent_192_png,software_update_urgent_192_png_len
+    );
+  SetToolBitmap(tb_follow, m_followIcon);
+  SetToolBitmap(tb_evaltillhere,GetImage(wxT("go-bottom"),
+                                         go_bottom_128_png,go_bottom_128_png_len,
+                                         go_bottom_192_png,go_bottom_192_png_len
+                  ));
+  SetToolBitmap(tb_evaluate_rest, 
+                GetImage(wxT("go-last"),
+                         go_last_128_png,go_last_128_png_len,
+                         go_last_192_png,go_last_192_png_len
+                  ));
+  SetToolBitmap(tb_hideCode,GetImage(wxT("weather-few-clouds"),
+                                     weather_few_clouds_128_png,weather_few_clouds_128_png_len,
+                                     weather_few_clouds_192_png,weather_few_clouds_192_png_len
+                  ));
+  m_PlayButton = GetImage(wxT("media-playback-start"),
+                          media_playback_start_128_png,media_playback_start_128_png_len,
+                          media_playback_start_192_png,media_playback_start_192_png_len
+    );
+  m_StopButton = GetImage(wxT("media-playback-stop"),
+                          media_playback_stop_128_png,media_playback_stop_128_png_len,
+                          media_playback_stop_192_png,media_playback_stop_192_png_len
+    );
+  SetToolBitmap(tb_animation_startStop,
+                m_PlayButton);
+  SetToolBitmap(tb_help,GetImage(wxT("gtk-help"),
+                                 gtk_help_128_png,gtk_help_128_png_len,
+                                 gtk_help_192_png,gtk_help_192_png_len
+                  ));
   Realize();
 }
 
@@ -493,7 +651,7 @@ void ToolBar::AnimationButtonState(AnimationStartStopState state)
 
 void ToolBar::OnSize(wxSizeEvent &event)
 {
+  UpdateBitmaps();
   Refresh();
   event.Skip();
 }
-
