@@ -161,6 +161,10 @@ public:
   //! Updates the slider to match the Slide Show cell.
   void UpdateSlider(SlideShow *cell);
 
+  int GetIdealHeight(){
+    return m_needsInformationIcon.GetSize().y;
+  }
+  
   //! Get the cell style for new cells
   GroupType GetCellType();
   //! Set the cell style to show for the current cell
@@ -174,10 +178,14 @@ public:
     }
   //! The current style is the new style for new cells
   void SetDefaultCellStyle();
+  //! Update the bitmaps on ppi changes.
+  void UpdateBitmaps();
 
 protected:
     void OnSize(wxSizeEvent &event);
 private:
+  //! The ppi rate.
+  wxSize m_ppi;
   //! The default style for new cells.
   int m_defaultCellStyle;
   //! The drop-down-box for text styles
