@@ -36,6 +36,7 @@
 #include <wx/fdrepdlg.h>
 #include <list>
 
+#include "VariablesPane.h"
 #include "Notification.h"
 #include "Cell.h"
 #include "EditorCell.h"
@@ -84,6 +85,8 @@ private:
   bool m_scrollToTopOfCell;
   //! Is our window currently active?
   bool m_windowActive;
+  //! The configuration storage
+  Configuration m_configurationTopInstance;
   //! The rectangle we need to refresh. -1 as "left" coordinate means: No rectangle
   wxRect m_rectToRefresh;
   /*! The size of a scroll step
@@ -368,7 +371,7 @@ private:
 
   /*! Escape all chars that aren't allowed in html.
 
-    Also converts \n to <BR>
+    Also converts \n to <br/>
    */
   wxString EscapeHTMLChars(wxString input);
 
@@ -567,7 +570,6 @@ private:
   wxTimer m_timer;
   //! The cursor blink rate. Also the timeout for redrawing the worksheet
   wxTimer m_caretTimer;
-  wxBitmap m_memory;
   //! True if no changes have to be saved.
   bool m_saved;
   AutoComplete *m_autocomplete;
@@ -621,7 +623,11 @@ public:
 
   //! Is called if this element looses or gets the focus
   void OnActivate(wxActivateEvent &event);
-  //! The central settings storage
+  /*! The pointer to the currently active central settings storage
+
+    Whilst printing or exporting the worksheet a settings storage
+    with the print settings might be active.
+   */
   Configuration *m_configuration;
   //! Get the currently active EditorCell
   EditorCell *GetActiveCell()
@@ -807,10 +813,8 @@ public:
 
       This item is the first of the enum and is assigned a high enough number
       that it won't collide with the numbers to be found in wxFrame::Event
-
-      wxID_HIGHEST + 500...503 are assigned in TableOfContents.h
      */
-            popid_copy = wxID_HIGHEST + 504,
+            popid_copy = wxID_HIGHEST + 500,
     popid_cut,
     popid_paste,
     popid_select_all,
@@ -821,6 +825,8 @@ public:
     popid_copy_svg,
     popid_copy_emf,
     popid_copy_rtf,
+    popid_add_watch,
+    popid_add_watch_label,
     popid_delete,
     popid_simplify,
     popid_expand,
@@ -836,6 +842,7 @@ public:
     popid_edit,
     popid_add_comment,
     popid_insert_input,
+	popid_copy_matlab,
     popid_copy_tex,
     popid_copy_text,
     popid_copy_mathml,
@@ -912,7 +919,7 @@ public:
   bool RecalculateIfNeeded();
 
   //! Schedule a recalculation of the worksheet starting with the cell start.
-  void Recalculate(GroupCell *start, bool force = false);
+  void Recalculate(Cell *start, bool force = false);
 
   void Recalculate(bool force = false)
   { Recalculate(m_tree, force); }
@@ -1040,6 +1047,9 @@ public:
   //! Copy the selection to the clipboard as it would appear in a .wxm file
   bool CopyCells();
 
+  //! Copy a Matlab representation of the current selection to the clipboard
+  bool CopyMatlab();
+
   //! Copy a textual representation of the current selection to the clipboard
   bool CopyText();
 
@@ -1146,7 +1156,8 @@ public:
 
   bool ActivateNextInput(bool input = false);
 
-  //! Scrolls to the cursor
+  /*! Scrolls to the cursor
+  */
   void ScrollToCaret();
 
   //! Scrolls to the cell given by ScheduleScrollToCell; Is called once we have time to do so.
@@ -1184,11 +1195,14 @@ public:
    */
   void ShowPoint(wxPoint point);
 
+  //! What to do if the worksheet is in the input focus
   void OnSetFocus(wxFocusEvent &event);
 
+  //! What to do if the worksheet looses the input focus
   void OnKillFocus(wxFocusEvent &event);
 
-  bool IsSelected(int type);
+  //! Does the selection start with a cell of the type "type"
+  bool IsSelected(CellType type);
 
   /*! Set the slide of the currently selected slideshow or advance it by one step
 
@@ -1208,13 +1222,21 @@ public:
     return m_last;
   }
 
-  bool IsSelectionInWorking();
+  //! Is the selection in the current working group?
+  bool IsSelectionInWorkingGroup();
 
   void SetActiveCell(EditorCell *cell, bool callRefresh = true);
 
+/*!
+  Set the HCaret to its default location, at the end of the document.
+ */
   void SetDefaultHCaret();
 
-  void SetHCaret(GroupCell *where, bool callRefresh = true); // call with false, when manually refreshing
+  /*! Set the HCaret at the location of the given Cell.
+    
+    @param where   The cell to place the cursor before.
+  */
+  void SetHCaret(GroupCell *where); // call with false, when manually refreshing
   //! The cell the horizontal cursor is above. NULL means at the start of the document.
   GroupCell *GetHCaret();
 
@@ -1230,6 +1252,7 @@ public:
   //! Place the cursor into a new cell where the horizontal cursor is
   void OpenHCaret(wxString txt, GroupType type);
 
+  //! Activates the horizontal cursor
   void ShowHCaret();
 
   /*! Is it possible to issue an undo in the currently selected cell?
@@ -1439,8 +1462,6 @@ public:
   bool GCContainsCurrentQuestion(GroupCell *cell);
 
   /*! Move the cursor to the question maxima currently asks and if needed add a cell for user input
-
-    \todo Currently scrolls to the GroupCell the question is in, not to the actual question.
    */
   void OpenQuestionCaret(wxString txt = wxT(""));
 
@@ -1451,6 +1472,9 @@ public:
   */
   GroupCell *GetWorkingGroup(bool resortToLast = false);
 
+  //! The panel the user can display variable contents in
+  Variablespane *m_variablesPane;
+
 #if wxUSE_ACCESSIBILITY
   class AccessibilityInfo: public wxAccessible
   {
