@@ -34,7 +34,8 @@
 XmlInspector::XmlInspector(wxWindow *parent, int id) : wxRichTextCtrl(parent, id,
                                                                       wxEmptyString,
                                                                       wxDefaultPosition,
-                                                                      wxDefaultSize,
+                                                                      wxSize(wxSystemSettings::GetMetric ( wxSYS_SCREEN_X )/10,
+                    wxSystemSettings::GetMetric ( wxSYS_SCREEN_Y )/10),
                                                                       wxTE_READONLY |
                                                                       wxTE_RICH |
                                                                       wxHSCROLL |
@@ -44,8 +45,6 @@ XmlInspector::XmlInspector(wxWindow *parent, int id) : wxRichTextCtrl(parent, id
   m_indentLevel = 0;
   BeginSuppressUndo();
   Clear();
-  SetMinSize(wxSize(wxSystemSettings::GetMetric ( wxSYS_SCREEN_X )/10,
-                    wxSystemSettings::GetMetric ( wxSYS_SCREEN_Y )/10));
 }
 
 XmlInspector::~XmlInspector()
@@ -124,8 +123,10 @@ void XmlInspector::Update()
 
     // Indent the XML
     wxString textWithIndention;
+    long length = 0;
     for ( wxString::iterator it = m_fromMaxima.begin(); it!=m_fromMaxima.end(); ++it)
-    {    
+    {
+      length++;
       // Assume that all tags add indentation
       if (*it == wxT('>'))
         m_indentLevel++;
@@ -151,6 +152,10 @@ void XmlInspector::Update()
     WriteText(textWithIndention);
     EndTextColour();
     m_fromMaxima = wxEmptyString;
+
+    // Adding many chars to the XML inspector is slow => Warn our users if we do so.
+    if(length > 100000)
+      wxLogMessage(wxString::Format(_("Added much text (%li chars) to the XML inspector.")));
   }
 }
 
@@ -164,6 +169,12 @@ wxString XmlInspector::IndentString(int level)
 
 void XmlInspector::Add_ToMaxima(wxString text)
 {
+  if(m_toMaxima != wxEmptyString)
+  {
+    m_toMaxima += wxT("\n\x2015\n");
+    if(m_fromMaxima != wxEmptyString)
+      m_fromMaxima += wxT("\n\x2015\n");
+  }
   m_toMaxima += text;
   m_updateNeeded = true;
 }
