@@ -55,15 +55,11 @@ IMPLEMENT_APP(MyApp)
 std::list<wxMaxima *> MyApp::m_topLevelWindows;
 
 
-void MyApp::Cleanup()
-{
-  for (std::list<wxMaxima *>::iterator it=m_topLevelWindows.begin();
-       it != m_topLevelWindows.end(); ++it)
-    (*it)->CleanUp();
-}
-
 bool MyApp::OnInit()
 {
+  #if wxUSE_ON_FATAL_EXCEPTION
+  wxHandleFatalExceptions(true);
+  #endif
   // MSW: Perhaps that is faster.
   wxSystemOptions::SetOption("msw.display.directdraw","1");
   // No spell checking in our dialog's input portions on the mac.
@@ -89,14 +85,73 @@ bool MyApp::OnInit()
     wxString dirName(xdgDir.GetPath());
     if(!wxDirExists(dirName))
       wxMkDir(dirName,0x700);
+    wxLogNull blocker;
     if(wxFileExists(configFileOld))
       wxCopyFile(configFileOld,configFileXDG);
   }
   #endif
   #endif
-  
-  atexit(Cleanup);
+
+  wxConfig::Set(new wxFileConfig(wxT("wxMaxima"), wxEmptyString, m_configFileName));
+
+  m_locale.AddCatalogLookupPathPrefix(m_dirstruct->LocaleDir());
+  m_locale.AddCatalogLookupPathPrefix(m_dirstruct->LocaleDir()+wxT("/wxwin"));
+  m_locale.AddCatalogLookupPathPrefix(wxT("/usr/share/locale"));
+  m_locale.AddCatalogLookupPathPrefix(wxT("/usr/local/share/locale"));
+  wxConfigBase *config = wxConfig::Get();
   int lang = wxLANGUAGE_UNKNOWN;
+  lang = wxLocale::GetSystemLanguage();
+  if(lang == wxLANGUAGE_UNKNOWN)
+    lang = wxLANGUAGE_DEFAULT;
+  if(config->Read(wxT("language"), &lang))
+  {
+    if(wxLocale::IsAvailable(lang))
+    {
+      if((lang != wxLANGUAGE_UNKNOWN) && (lang != wxLANGUAGE_DEFAULT) &&
+         (lang != wxLocale::GetSystemLanguage()))
+      {
+        m_locale.Init(lang);
+        wxString localeName = m_locale.GetCanonicalName();
+        wxLogDebug(wxString::Format(_("wxMaxima's locale is set to to %s"),localeName));
+        if((m_locale.GetSystemEncoding() == wxFONTENCODING_UTF8) ||
+           (m_locale.GetSystemEncoding() == wxFONTENCODING_SYSTEM) ||
+           (m_locale.GetSystemEncoding() == wxFONTENCODING_UNICODE) ||
+           (m_locale.GetSystemEncoding() == wxFONTENCODING_DEFAULT))
+          localeName+=wxT(".UTF-8");
+        if(m_locale.GetSystemEncoding() == wxFONTENCODING_UTF16)
+          localeName+=wxT(".UTF-16");
+        if(m_locale.GetSystemEncoding() == wxFONTENCODING_UTF32)
+          localeName+=wxT(".UTF-32");
+        
+        wxLogDebug(wxString::Format(_("Setting maxima's locale to %s."),localeName));
+        wxSetEnv(wxT("LANG"), localeName);
+      }
+      else
+      {
+        if((lang != wxLANGUAGE_UNKNOWN) && (lang != wxLANGUAGE_DEFAULT))
+        {
+          m_locale.Init(lang);
+          wxString localeName = m_locale.GetCanonicalName();
+          wxLogDebug(
+            wxString::Format(_("Setting wxMaxima's language to the system language %s"),
+                             localeName)
+            );
+        }
+        else
+          wxLogDebug(wxString::Format(_("Not setting language %i"), lang));
+      }
+    }
+    else
+    {
+      wxLogDebug("According to the OS the current language isn't available!");
+      wxLogNull blocker;
+      m_locale.Init(lang);
+    }
+  }
+  else
+    m_locale.Init(wxLANGUAGE_DEFAULT);
+  m_locale.AddCatalog(wxT("wxMaxima"));
+  m_locale.AddCatalog(wxT("wxMaxima-wxstd"));
 
   bool exitAfterEval = false;
   bool evalOnStartup = false;
@@ -116,6 +171,7 @@ bool MyApp::OnInit()
                   {wxCMD_LINE_SWITCH, "b", "batch",
                    "run the file and exit afterwards. Halts on questions and stops on errors.",  wxCMD_LINE_VAL_NONE, 0},
                   { wxCMD_LINE_OPTION, "f", "ini", "allows to specify a file to store the configuration in", wxCMD_LINE_VAL_STRING , 0},
+                  { wxCMD_LINE_OPTION, "m", "maxima", "allows to specify the location of the maxima binary", wxCMD_LINE_VAL_STRING , 0},
                   {wxCMD_LINE_PARAM, NULL, NULL, "input file", wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL | wxCMD_LINE_PARAM_MULTIPLE},
             {wxCMD_LINE_NONE, "", "", "", wxCMD_LINE_VAL_NONE, 0}
           };
@@ -126,40 +182,38 @@ bool MyApp::OnInit()
   // Attention: The config file is changed by wxMaximaFrame::wxMaximaFrame::ReReadConfig
   if (cmdLineParser.Found(wxT("f"),&ini))
   {
-    wxConfig::Set(new wxFileConfig(wxT("wxMaxima"), wxEmptyString, ini));
-    m_configFileName = ini;
+    Configuration::m_configfileLocation_override = ini;
+    wxConfig::Set(new wxFileConfig(wxT("wxMaxima"),
+                                   wxEmptyString,
+                                   Configuration::m_configfileLocation_override));
   }
   else
     wxConfig::Set(new wxConfig(wxT("wxMaxima")));
-  
+
+  config = wxConfig::Get();
+
+  if (cmdLineParser.Found(wxT("m"),&ini))
+  {
+    Configuration::m_maximaLocation_override = ini;
+  }
+
   wxImage::AddHandler(new wxPNGHandler);
   wxImage::AddHandler(new wxXPMHandler);
   wxImage::AddHandler(new wxJPEGHandler);
 
   wxFileSystem::AddHandler(new wxZipFSHandler);
 
-  wxConfigBase *config = wxConfig::Get();
-  lang = wxLocale::GetSystemLanguage();
-  config->Read(wxT("language"), &lang);
+  m_dirstruct =  new Dirstructure;
 
-  if(wxLocale::IsAvailable(lang))
-    m_locale.Init(lang);
-  else
-    m_locale.Init(wxLANGUAGE_ENGLISH);
-
-  Dirstructure dirstruct;
-  if((lang != wxLANGUAGE_UNKNOWN) && (lang != wxLANGUAGE_DEFAULT) &&
-     (lang != wxLocale::GetSystemLanguage()))
-    wxSetEnv(wxT("LANG"), m_locale.GetCanonicalName());
+#ifdef __WXMSW__
+  wxString oldWorkingDir = wxGetCwd();
   if (!wxGetEnv(wxT("BUILD_DIR"), NULL))
   {
     wxString dir = wxPathOnly(wxStandardPaths::Get().GetExecutablePath());
     if(dir != wxEmptyString)
       wxSetWorkingDirectory(wxPathOnly(wxStandardPaths::Get().GetExecutablePath()));
   }
-
-#ifdef __WXMSW__
-  wxString fontPrefix = dirstruct.FontDir() + wxT("/");  
+  wxString fontPrefix = m_dirstruct->FontDir() + wxT("/");  
   /* Add private jsMath fonts, if they exist */ 
 #if wxCHECK_VERSION(3, 1, 1)
   if (wxFileExists(fontPrefix + wxT(CMEX10) + wxT(".ttf"))) wxFont::AddPrivateFont(fontPrefix + wxT(CMEX10) + wxT(".ttf"));
@@ -180,16 +234,11 @@ bool MyApp::OnInit()
   if (wxFileExists(fontPrefix + wxT(LIBERTINE8))) wxFont::AddPrivateFont(fontPrefix + wxT(LIBERTINE8));
   if (wxFileExists(fontPrefix + wxT(LIBERTINE9))) wxFont::AddPrivateFont(fontPrefix + wxT(LIBERTINE9));
 #endif
+  wxSetWorkingDirectory(oldWorkingDir);
+
 #endif
 
-  m_locale.AddCatalogLookupPathPrefix(dirstruct.LocaleDir());
-  m_locale.AddCatalogLookupPathPrefix(dirstruct.LocaleDir()+wxT("/wxwin"));
-  m_locale.AddCatalogLookupPathPrefix(wxT("/usr/share/locale"));
-  m_locale.AddCatalogLookupPathPrefix(wxT("/usr/local/share/locale"));
-  m_locale.AddCatalog(wxT("wxMaxima"));
-  m_locale.AddCatalog(wxT("wxMaxima-wxstd"));
-
-#if defined __WXMAC__
+#if defined __WXOSX__
   wxString path;
   wxGetEnv(wxT("PATH"), &path);
   wxSetEnv(wxT("PATH"), path << wxT(":/usr/local/bin"));
@@ -215,13 +264,13 @@ bool MyApp::OnInit()
     std::cout << " (Git version: " << WXMAXIMA_GIT_VERSION << ")";
 #endif
     std::cout << "\n";
-    wxExit();
+    exit(0);
   }
   if (cmdLineParser.Found(wxT("h")))
   {
     std::cout << "A feature-rich graphical user interface for the computer algebra system maxima\n";
     std::cout << cmdLineParser.GetUsageString();
-    wxExit();
+    exit(0);
   }
 
   if (cmdLineParser.Found(wxT("b")))
@@ -259,12 +308,12 @@ bool MyApp::OnInit()
   return true;
 }
 
-#if defined (__WXMSW__)
 int MyApp::OnExit()
 {
+  wxDELETE(m_dirstruct);
+  m_dirstruct = NULL;
   return true;
 }
-#endif
 
 void MyApp::NewWindow(wxString file, bool evalOnStartup, bool exitAfterEval)
 {
@@ -297,19 +346,26 @@ void MyApp::OnFileMenu(wxCommandEvent &ev)
   {
     case wxMaximaFrame::menu_new_id:
     case ToolBar::tb_new:
-    case wxMaxima::mac_newId:
-
+  case wxMaxima::mac_newId:
+    {
       // Mac computers insist that all instances of a new application need to share
       // the same process. On all other OSes we create a separate process for each
       // window: This way if one instance of wxMaxima crashes all the other instances
       // are still alive.
-#if defined __WXMAC__
+#if defined __WXOSX__
       NewWindow();
 #else
 //      NewWindow();
-      wxExecute(wxT("\"")+wxStandardPaths::Get().GetExecutablePath()+wxT("\""));
+      wxString args;
+      if(Configuration::m_configfileLocation_override != wxEmptyString)
+        args += " -f \"" + Configuration::m_configfileLocation_override + "\"";
+      if(Configuration::m_maximaLocation_override != wxEmptyString)
+        args += " -m \"" + Configuration::m_maximaLocation_override + "\"";
+
+      wxExecute(wxT("\"") + wxStandardPaths::Get().GetExecutablePath() + wxT("\"") + args);
 #endif
       break;
+    }
     case wxMaxima::mac_openId:
     {
       wxString file = wxFileSelector(_("Open"), wxEmptyString,
@@ -322,18 +378,18 @@ void MyApp::OnFileMenu(wxCommandEvent &ev)
       break;
     case wxID_EXIT:
     {
-      bool quit = true;
       std::list<wxMaxima *>::iterator it=m_topLevelWindows.begin();
       while(it != m_topLevelWindows.end())
       {
-        if (!(*it)->Close())
+        if (*it != NULL)
         {
-          quit = false;
-          break;
+          wxCloseEvent *event = new wxCloseEvent(wxEVT_CLOSE_WINDOW);
+          event->SetCanVeto(true);
+          event->SetLoggingOff(false);
+          (*it)->GetEventHandler()->QueueEvent(event);
         }
+        ++it;
       }
-      if (quit)
-        wxExit();
     }
     break;
   }
