@@ -1,10 +1,30 @@
 #include "wxMathml.h"
 #include "../data/wxMathML.h"
 #include <iostream>
+#include <wx/wx.h>
+#include <wx/mstream.h>
+#include <wx/zstream.h>
+#include <wx/txtstrm.h>
+#include <wx/string.h>
 
 wxMathML::wxMathML()
-{ 
-  m_wxMathML = wxString(wxMathML_lisp, wxMathML_lisp_len);
+{
+  // Unzip wxMathml.lisp: We need to store it in a .zip format
+  // in order to avoid a bug in the ArchLinux C compiler that
+  // seems to replace long strings by a "\0".
+  wxMemoryInputStream istream(wxMathML_lisp_gz, wxMathML_lisp_gz_len);
+
+  wxZlibInputStream zstream(istream);
+  wxTextInputStream textIn(zstream);
+  wxString line;
+  m_wxMathML = wxEmptyString;
+  
+  while(!istream.Eof())
+  {
+    line = textIn.ReadLine();
+    m_wxMathML += line + wxT("\n");
+  }
+  wxASSERT_MSG(m_wxMathML.Length()>64000,_("Compiler-Bug? wxMathml.lisp is shorter than expected!"));
 }
 
 wxString wxMathML::GetCmd()
@@ -18,12 +38,12 @@ wxString wxMathML::GetCmd()
     wxString lineWithoutComments;
 
     bool stringIs = false;
-    wxChar lastChar=wxT(' ');
+    wxChar lastChar = wxT('\n');
     wxString::iterator ch = line.begin();
     while (ch < line.end())
     {
       // Remove formatting spaces
-      if(((lastChar == ' ') && (*ch == ' ')) && (!stringIs))
+      if(((lastChar == '\n') && ((*ch == ' ') || (*ch == '\t'))))
 	  ch++;
       else
 	{
@@ -51,5 +71,6 @@ wxString wxMathML::GetCmd()
     }
     cmd += lineWithoutComments + " ";
   }
+  wxASSERT_MSG(cmd.Length()>54000,_("Bug: After removing the whitespace wxMathml.lisp is shorter than expected!"));
   return wxT(":lisp-quiet ") + cmd + "\n";
 }
