@@ -33,6 +33,7 @@
 
 #include "wxMaximaFrame.h"
 #include "MathParser.h"
+#include "Dirstructure.h"
 
 #include <wx/socket.h>
 #include <wx/config.h>
@@ -98,7 +99,7 @@ class wxMaxima : public wxMaximaFrame
 {
 public:
 
-  wxMaxima(wxWindow *parent, int id, const wxString title, const wxString configFile = wxEmptyString,
+  wxMaxima(wxWindow *parent, int id, const wxString title,
            const wxPoint pos = wxDefaultPosition, const wxSize size = wxDefaultSize);
   
   ~wxMaxima();
@@ -112,7 +113,9 @@ public:
     //! The time between two auto-saves has elapsed.
             AUTO_SAVE_TIMER_ID,
     //! We look if we got new data from maxima's stdout.
-            MAXIMA_STDOUT_POLL_ID
+            MAXIMA_STDOUT_POLL_ID,
+            //! We have finished waiting if the current string ends in a newline
+            WAITFORSTRING_ID
   };
 
   /*! A timer that determines when to do the next autosave;
@@ -127,12 +130,15 @@ public:
    */
   wxTimer m_autoSaveTimer;
 
+  //! A timer that tells us to wait until maxima ends its data.
+  wxTimer m_waitForStringEndTimer;
+
   //! Is triggered when a timer this class is responsible for requires
   void OnTimerEvent(wxTimerEvent &event);
 
   //! A timer that polls for output from the maxima process.
   wxTimer m_maximaStdoutPollTimer;
-  
+
   void ShowTip(bool force);
 
   /*! Get the name of the help file
@@ -141,6 +147,8 @@ public:
     installed instead of the ones from the alphabetically first installation we find.
    */
   wxString GetHelpFile();
+  //! An helper function for GetHelpFile()
+  wxString GetHelpFile2();
 
   void ShowMaximaHelp(wxString keyword = wxEmptyString);
 
@@ -181,7 +189,13 @@ public:
   void LoadImage(wxString file)
   { m_worksheet->OpenHCaret(file, GC_TYPE_IMAGE); }
 
+  //! Query the value of a new maxima variable
+  bool QueryVariableValue();
+
 private:
+  //! The variable names to query for the variables pane
+  wxArrayString m_varNamesToQuery;
+  bool m_isLogTarget;
   //! Is true if opening the file from the command line failed before updating the statusbar.
   bool m_openInitialFileError;
   //! Escape strings into a format lisp accepts
@@ -233,13 +247,6 @@ private:
   wxString GetUnmatchedParenthesisState(wxString text,int &index);
   //! The buffer all text from maxima is stored in before converting it to a wxString.
   wxMemoryBuffer m_uncompletedChars;
-  /*! How many load() commands are currently nested?
-
-    We are only interested on the filenames of load() commands that are 
-    loaded by the user, but not the ones that are loaded by a loaded 
-    package.
-   */
-  int m_nestedLoadCommands;
 
 protected:
   //! The gnuplot process info
@@ -263,10 +270,6 @@ protected:
   /*! Launches the HTML help browser
 
     \param helpfile The name of the file the help browser has to be launched with
-    \param otherhelpfile We offer help for maxima and wxMaxima in separate manuals.
-                         This parameter contains the filename of the manual we aren't
-                         using currently so the help browser can open a tab containing
-                         this file.
     \param keyword The keyword to show help for
   */
   void ShowHTMLHelp(wxString helpfile, wxString keyword = wxEmptyString);
@@ -299,7 +302,14 @@ protected:
     is too busy to execute the idle task at all.
   */
   void OnIdle(wxIdleEvent &event);
+  /*! Interpret the new data from maxima
 
+    We don't interpret this data directly in the idle event since if we
+    block somewhere in the idle event we block gnome.
+   */
+  void InterpretDataFromMaxima();
+  bool m_dataFromMaximaIs;
+  
   void MenuCommand(wxString cmd);                  //!< Inserts command cmd into the worksheet
   void FileMenu(wxCommandEvent &event);            //!< Processes "file menu" clicks
   void MaximaMenu(wxCommandEvent &event);          //!< Processes "maxima menu" clicks
@@ -382,7 +392,10 @@ protected:
     \todo Inform maxima about the new default plot window size.
   */
   void EditInputMenu(wxCommandEvent &event);
-
+  //! Trigger reading all variables from Maxima that are shown in the Variables sidebar
+  void VarReadEvent(wxCommandEvent &event);
+  //! Trigger adding all variables to the variables sidebar
+  void VarAddAllEvent(wxCommandEvent &event);
   void EvaluateEvent(wxCommandEvent &event);       //
   void InsertMenu(wxCommandEvent &event);          //
   void PrintMenu(wxCommandEvent &event);
@@ -403,7 +416,7 @@ protected:
   void DumpProcessOutput();
 
   //! Try to evaluate the next command for maxima that is in the evaluation queue
-  void TryEvaluateNextInQueue();
+  void TriggerEvaluation();
 
   void TryUpdateInspector();
 
@@ -496,9 +509,13 @@ protected:
   //! Read (and discard) suppressed output
   void ReadSuppressedOutput(wxString &data);
 
-    /*! Reads the variable values maxima advertises to us
+  /*! Reads the variable values maxima advertises to us
    */
   void ReadVariables(wxString &data);
+  
+  /*! Reads the "add variable to watch list" tag maxima can send us
+   */
+  void ReadAddVariables(wxString &data);
 
 #ifndef __WXMSW__
 
@@ -539,7 +556,7 @@ protected:
  */
   void SetupVariables();
 
-  void KillMaxima();                 //!< kills the maxima process
+  void KillMaxima(bool logMessage = true);                 //!< kills the maxima process
   /*! Update the title
 
     Updates the "saved" status, as well, but does only do anything if saved has
@@ -549,6 +566,12 @@ protected:
    */
   void ResetTitle(bool saved, bool force = false);
 
+  /*! Makes this window the debug log target of all windows from this maxima process
+
+    Only necessary on the mac where the same process creates loads of windows.
+   */
+  void BecomeLogTarget();
+  
   void FirstOutput();
 
   /*! Opens a content.xml file that has been extracted from a broken .wxmx file
@@ -604,6 +627,8 @@ protected:
   //! The stderr of the maxima process
   wxInputStream *m_maximaStderr;
   int m_port;
+  //! All chars from maxima that still aren't part of m_currentOutput
+  wxString m_newCharsFromMaxima;
   /*! The end of maxima's current uninterpreted output, see m_currentOutput.
    
     If we just want to look if maxima's current output contains an ending tag
@@ -623,6 +648,10 @@ protected:
   wxString m_variablesPrefix;
   //! The marker for the end of a variables section
   wxString m_variablesSuffix;
+  //! The marker for the start of a "add variables" section
+  wxString m_addVariablesPrefix;
+  //! The marker for the end of a "add variables" section
+  wxString m_addVariablesSuffix;
   //! The marker for the start of a list of autocompletion templates
   wxString m_symbolsPrefix;
   //! The marker for the end of a list of autocompletion templates
@@ -637,14 +666,16 @@ protected:
   wxString m_lastPrompt;
   wxString m_lastPath;
   wxPrintData *m_printData;
+  /*! Did we tell maxima to close?
+
+    If we didn't we respan an unexpectedly-closing maxima.
+   */
   bool m_closing;
   wxString m_openFile;
   //! The directory with maxima's temp files
   wxString m_maximaTempDir;
   //! The directory with maxima's documentation
   wxString m_maximaDocDir;
-  //! The directory with maxima's share files
-  wxString m_maximaShareDir;
   bool m_fileSaved;
   wxString m_chmhelpFile;
   wxString m_maximaVersion;
@@ -661,6 +692,9 @@ protected:
   wxRegEx m_funRegEx;
   wxRegEx m_varRegEx;
   wxRegEx m_blankStatementRegEx;
+  wxRegEx m_sbclCompilationRegEx;
+  MathParser *m_parser;
+  bool m_maximaBusy;
 #if wxUSE_DRAG_AND_DROP
 
   friend class MyDropTarget;
@@ -691,9 +725,7 @@ class MyApp : public wxApp
 public:
   virtual bool OnInit();
 
-#if defined (__WXMSW__)
   virtual int OnExit();
-#endif
   wxLocale m_locale;
 
   /*! Create a new window
@@ -708,20 +740,19 @@ public:
     \param exitAfterEval Do we want to close the window after the file has been evaluated?
    */
   void NewWindow(wxString file = wxEmptyString, bool evalOnStartup = false, bool exitAfterEval = false);
-    
-  //! Is called by atExit and tries to close down the maxima process if wxMaxima has crashed.
-  static void Cleanup();
-
+  
   static std::list<wxMaxima *> m_topLevelWindows;
 
   void OnFileMenu(wxCommandEvent &ev);
 
   virtual void MacNewFile();
+  void BecomeLogTarget();
 
   virtual void MacOpenFile(const wxString &file);
 private:
   //! The name of the config file. Empty = Use the default one.
   wxString m_configFileName;
+  Dirstructure *m_dirstruct;
   DECLARE_EVENT_TABLE()
 };
 
