@@ -43,10 +43,12 @@ surrounding the worksheet.
 #include "Worksheet.h"
 #include "RecentDocuments.h"
 #include "Version.h"
+#include "MainMenuBar.h"
 #include "History.h"
 #include "ToolBar.h"
 #include "XmlInspector.h"
 #include "StatusBar.h"
+#include "LogPane.h"
 #include <list>
 
 
@@ -55,10 +57,10 @@ surrounding the worksheet.
 class wxMaximaFrame : public wxFrame
 {
 public:
-  wxMaximaFrame(wxWindow *parent, int id, const wxString &title, const wxString configFile,
+  wxMaximaFrame(wxWindow *parent, int id, const wxString &title,
                 const wxPoint &pos = wxDefaultPosition,
                 const wxSize &size = wxDefaultSize,
-                long style = wxDEFAULT_FRAME_STYLE);
+                long style = wxDEFAULT_FRAME_STYLE, bool becomeLogTarget = true);
 
   /*! The destructor
   */
@@ -100,10 +102,11 @@ public:
     menu_pane_structure,    //!< Both the "toggle the structure pane" command and the structure
     menu_pane_xmlInspector, //!< Both the "toggle the xml monitor" command and the monitor pane
     menu_pane_format,    //!< Both the "toggle the format pane" command and the format pane
-    menu_pane_greek,     //!< Both the "toggle the format pane" command for the "greek" pane
-    menu_pane_log,       //!< Both the "toggle the format pane" command for the "debug" pane
-    menu_pane_draw,      //!< Both the "toggle the format pane" command for the "draw" pane
-    menu_pane_symbols,   //!< Both the "toggle the format pane" command for the "symbols" pane
+    menu_pane_greek,     //!< Both the "toggle the greek pane" command and the "greek" pane
+    menu_pane_log,       //!< Both the "toggle the log pane" command and the "log" pane
+    menu_pane_variables, //!< Both the "toggle the variables pane" command and the "variables" pane
+    menu_pane_draw,      //!< Both the "toggle the draw pane" command for the "draw" pane
+    menu_pane_symbols,   //!< Both the "toggle the symbols pane" command for the "symbols" pane
     /*! Both used as the "toggle the stats pane" command and as the ID of the stats pane
 
       Since this enum is also used for iterating over the panes it is vital 
@@ -267,6 +270,7 @@ public:
     menu_evaluate_all,
     menu_show_tip,
     menu_copy_from_worksheet,
+	menu_copy_matlab_from_worksheet,
     menu_copy_tex_from_worksheet,
     menu_copy_text_from_worksheet,
     menu_undo,
@@ -518,6 +522,7 @@ public:
   {
     wait_for_start,
     process_wont_start,
+    sending,
     waiting,
     calculating,
     parsing,
@@ -564,6 +569,8 @@ public:
   void StatusExportFailed();
 
 protected:
+  //! How many bytes did maxima send us until now?
+  long m_bytesFromMaxima;
   //! The process id of maxima. Is determined by ReadFirstPrompt.
   long m_pid;
   //! Did the user ever give this file a name?
@@ -575,7 +582,7 @@ protected:
   //! The status bar
   StatusBar *m_statusBar;
   //! The menu bar
-  wxMenuBar *m_MenuBar;
+  MainMenuBar *m_MenuBar;
   //! The file menu.
   wxMenu *m_FileMenu;
   //! The edit menu.
@@ -623,6 +630,9 @@ protected:
   //! Remember an temporary autosave file name.
   void RegisterAutoSaveFile();
 private:
+  //! How many bytes did maxima send us when we updated the statusbar?
+  long m_bytesFromMaxima_last;
+  wxTimer m_bytesReadDisplayTimer; 
   //! A panel that shows all user-defined symbols on the symbols pane.
   wxPanel *m_userSymbols;
   //! A button per user defined symbol
@@ -631,10 +641,6 @@ private:
   //! True=We are currently saving.
   bool m_StatusSaving;
 
-  void set_properties();
-
-  void do_layout();
-
   void SetupToolBar();
 
 /*! 
@@ -716,8 +722,6 @@ private:
 protected:
   //! The current length of the evaluation queue of commands we still need to send to maxima
   int m_EvaluationQueueLength;
-  //! The name of the config file or wxEmptyString if the default is used.
-  wxString m_configFileName;
   //! Update the "user symbols" portion of the symbols pane.
   void UpdateUserSymbols();
   //! Do we need to update the display showing the evaluation queue length?
@@ -738,6 +742,8 @@ protected:
   XmlInspector *m_xmlInspector;
   //! true=force an update of the status bar at the next call of StatusMaximaBusy()
   bool m_forceStatusbarUpdate;
+  //! The panel the log and debug messages will appear on
+  LogPane *m_logPane;
   //! The worksheet itself
   Worksheet *m_worksheet;
   //! The history pane
