@@ -18,6 +18,7 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
+//  SPDX-License-Identifier: GPL-2.0+
 
 /*! \file
   This file defines the class Image that stores compressed images and handles scaling and uncompressing them.
@@ -31,7 +32,7 @@
 wxIcon wxMaximaIcon()
 {
     wxImage img;
-    wxMemoryInputStream istream(wxmaxima_png,wxmaxima_png_len);
+    wxMemoryInputStream istream(io_github_wxmaxima_developers_wxMaxima_png,io_github_wxmaxima_developers_wxMaxima_png_len);
     img.LoadFile(istream);
     wxIcon icon;
     icon.CopyFromBitmap(wxBitmap(img));
